@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -92,31 +92,36 @@ let print_type info fmt ty = try print_type info fmt ty
   with Unsupported s -> raise (UnsupportedType (ty,s))
 
 let number_format = {
-  Number.long_int_support = true;
-  Number.extra_leading_zeros_support = false;
-  Number.negative_int_support = Number.Number_default;
-  Number.dec_int_support = Number.Number_default;
-  Number.hex_int_support = Number.Number_unsupported;
-  Number.oct_int_support = Number.Number_unsupported;
-  Number.bin_int_support = Number.Number_unsupported;
-  Number.def_int_support = Number.Number_unsupported;
-  Number.negative_real_support = Number.Number_default;
-  Number.dec_real_support = Number.Number_default;
-  Number.hex_real_support = Number.Number_unsupported;
-  Number.frac_real_support = Number.Number_custom
-    (Number.PrintFracReal ("$to_real(%s)",
-      "$product($to_real(%s),$to_real(%s))",
-      "$quotient($to_real(%s),$to_real(%s))"));
-  Number.def_real_support = Number.Number_unsupported;
+    Number.long_int_support = `Default;
+    Number.negative_int_support = `Default;
+    Number.dec_int_support = `Default;
+    Number.hex_int_support = `Unsupported;
+    Number.oct_int_support = `Unsupported;
+    Number.bin_int_support = `Unsupported;
+    Number.negative_real_support = `Default;
+    Number.dec_real_support = `Default;
+    Number.hex_real_support = `Unsupported;
+    Number.frac_real_support =
+      `Custom
+        ((fun fmt i -> fprintf fmt "$to_real(%s)" i),
+         (fun fmt i n -> fprintf fmt "$product($to_real(%s),$to_real(%s))" i n),
+         (fun fmt i n -> fprintf fmt "$quotient($to_real(%s),$to_real(%s))" i n));
 }
 
-let number_format_metitarski = { number_format with
-  Number.frac_real_support = Number.Number_custom
-    (Number.PrintFracReal ("%s", "(%s * %s)", "(%s / %s)"));
-}
+let number_format_metitarski = {
+    number_format with
+    Number.frac_real_support =
+      `Custom
+        ((fun fmt i -> pp_print_string fmt i),
+         (fun fmt i n -> fprintf fmt "(%s * %s)" i n),
+         (fun fmt i n -> fprintf fmt "(%s / %s)" i n));
+  }
 
-let print_number info fmt c = Number.print (match info.info_num with
-  | TPTP -> number_format | MetiTarski -> number_format_metitarski) fmt c
+let print_const info fmt c =
+  let supported = match info.info_num with
+    | TPTP -> number_format
+    | MetiTarski -> number_format_metitarski in
+  Constant.(print supported unsupported_escape) fmt c
 
 let rec print_app info fmt ls tl oty =
   match query_syntax info.info_syn ls.ls_name with
@@ -140,7 +145,7 @@ and print_term info fmt t = match t.t_node with
   | Tapp (ls, tl) ->
       print_app info fmt ls tl t.t_ty
   | Tconst c ->
-      print_number info fmt c
+      print_const info fmt c
   | Tlet (t1,tb) ->
       let v,t2 = t_open_bound tb in
       fprintf fmt "$let_tt(%a@ =@ %a,@ %a)"
@@ -382,7 +387,7 @@ and print_term info fmt t = match t.t_node with
   | Tapp (ls, tl) ->
       print_app info fmt ls tl t.t_ty
   | Tconst c ->
-      Number.print number_format fmt c
+      Constant.(print number_format unsupported_escape) fmt c
   | Tlet _ -> unsupportedTerm t
       "DFG does not support let, use eliminate_let"
   | Tif _ -> unsupportedTerm t
