@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -285,7 +285,8 @@ let rec ty denv env impl { e_loc = loc; e_node = n } = match n with
   | Enot _ | Eequ _ | Edob _ | Enum _ -> error ~loc TypeExpected
 
 let t_int_const s =
-  t_const (Number.(ConstInt { ic_negative = false; ic_abs = int_literal_dec s})) ty_int
+  let int_lit = Number.(int_literal ILitDec ~neg:false s) in
+  t_const (Constant.ConstInt int_lit) ty_int
 
 (* unused
 let t_real_const r = t_const (Number.ConstReal r)
@@ -308,8 +309,9 @@ let rec term denv env impl { e_loc = loc; e_node = n } = match n with
       find_dobj ~loc denv env impl s
   | Enum (Nint s) -> t_int_const s
   | Enum (Nreal (i,f,e)) ->
-      t_const (Number.(ConstReal { rc_negative = false ;
-                                   rc_abs = real_const_dec i (Opt.get_def "0" f) e})) ty_real
+      let real_lit = Number.(real_literal ~radix:10 ~neg:false ~int:i
+                               ~frac:(Opt.get_def "0" f) ~exp:e) in
+      t_const (Constant.ConstReal real_lit) ty_real
   | Enum (Nrat (n,d)) ->
       let n = t_int_const n and d = t_int_const d in
       let frac = ns_find_ls denv.th_rat.th_export ["frac"] in
@@ -599,7 +601,7 @@ let flush_impl ~strict env uc impl =
           | _ -> f in
         let f = Mstr.fold add env t_true in
         let uc = if t_equal f t_true then uc else
-          let id = ls.ls_name.id_string ^ "_def" in
+          let id = ls.ls_name.id_string ^ "'def" in
           let pr = create_prsymbol (id_fresh id) in
           add_prop_decl uc Paxiom pr f in
         Mstr.add s e env, uc
