@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -68,7 +68,7 @@ let open_ls_defn_cb ld =
   let ls,_,_ = ld in
   let vl,t = open_ls_defn ld in
   let close ls' vl' t' =
-    if t_equal t t' && Lists.equal vs_equal vl vl' && ls_equal ls ls'
+    if t_equal_strict t t' && Lists.equal vs_equal vl vl' && ls_equal ls ls'
     then ls,ld else make_ls_defn ls' vl' t'
   in
   vl,t,close
@@ -319,7 +319,6 @@ type prop_decl = prop_kind * prsymbol * term
 
 type decl = {
   d_node : decl_node;
-  d_syms : Sid.t;         (* idents used in declaration *)
   d_news : Sid.t;         (* idents introduced in declaration *)
   d_tag  : Weakhtbl.tag;  (* unique magical tag *)
 }
@@ -345,10 +344,10 @@ module Hsdecl = Hashcons.Make (struct
     ts_equal ts1 ts2 && Lists.equal cs_equal td1 td2
 
   let eq_ld (ls1,(_,f1,_)) (ls2,(_,f2,_)) =
-    ls_equal ls1 ls2 && t_equal f1 f2
+    ls_equal ls1 ls2 && t_equal_strict f1 f2
 
   let eq_iax (pr1,fr1) (pr2,fr2) =
-    pr_equal pr1 pr2 && t_equal fr1 fr2
+    pr_equal pr1 pr2 && t_equal_strict fr1 fr2
 
   let eq_ind (ps1,al1) (ps2,al2) =
     ls_equal ps1 ps2 && Lists.equal eq_iax al1 al2
@@ -360,7 +359,7 @@ module Hsdecl = Hashcons.Make (struct
     | Dlogic l1, Dlogic l2 -> Lists.equal eq_ld l1 l2
     | Dind   (s1,l1), Dind (s2,l2) -> s1 = s2 && Lists.equal eq_ind l1 l2
     | Dprop (k1,pr1,f1), Dprop (k2,pr2,f2) ->
-        k1 = k2 && pr_equal pr1 pr2 && t_equal f1 f2
+        k1 = k2 && pr_equal pr1 pr2 && t_equal_strict f1 f2
     | _,_ -> false
 
   let cs_hash (cs,pl) =
@@ -368,9 +367,9 @@ module Hsdecl = Hashcons.Make (struct
 
   let hs_td (ts,td) = Hashcons.combine_list cs_hash (ts_hash ts) td
 
-  let hs_ld (ls,(_,f,_)) = Hashcons.combine (ls_hash ls) (t_hash f)
+  let hs_ld (ls,(_,f,_)) = Hashcons.combine (ls_hash ls) (t_hash_strict f)
 
-  let hs_prop (pr,f) = Hashcons.combine (pr_hash pr) (t_hash f)
+  let hs_prop (pr,f) = Hashcons.combine (pr_hash pr) (t_hash_strict f)
 
   let hs_ind (ps,al) = Hashcons.combine_list hs_prop (ls_hash ps) al
 
@@ -405,12 +404,16 @@ let d_hash d = Weakhtbl.tag_hash d.d_tag
 
 (** Declaration constructors *)
 
-let mk_decl node syms news = Hsdecl.hashcons {
-  d_node = node;
-  d_syms = syms;
-  d_news = news;
-  d_tag  = Weakhtbl.dummy_tag;
-}
+let mk_decl node news =
+  let d = {
+      d_node = node;
+      d_news = news;
+      d_tag  = Weakhtbl.dummy_tag;
+    } in
+  match node with
+  | Dprop (Pgoal,_,_) -> Hsdecl.unique d
+  | _ -> Hsdecl.hashcons d
+
 
 exception IllegalTypeAlias of tysymbol
 exception ClashIdent of ident
@@ -418,6 +421,9 @@ exception BadLogicDecl of lsymbol * lsymbol
 exception BadConstructor of lsymbol
 
 exception BadRecordField of lsymbol
+exception BadRecordType of lsymbol * tysymbol
+exception BadRecordUnnamed of lsymbol * tysymbol
+exception BadRecordCons of lsymbol * tysymbol
 exception RecordFieldMissing of lsymbol
 exception DuplicateRecordField of lsymbol
 
@@ -435,10 +441,19 @@ let syms_ls s ls = Sid.add ls.ls_name s
 let syms_ty s ty = ty_s_fold syms_ts s ty
 let syms_term s t = t_s_fold syms_ty syms_ls s t
 
+let syms_ty_decl ts =
+  type_def_fold syms_ty Sid.empty ts.ts_def
+
 let create_ty_decl ts =
-  let syms = type_def_fold syms_ty Sid.empty ts.ts_def in
   let news = Sid.singleton ts.ts_name in
-  mk_decl (Dtype ts) syms news
+  mk_decl (Dtype ts) news
+
+let syms_data_decl tdl =
+  let syms_constr syms (fs,_) =
+    List.fold_left syms_ty syms fs.ls_args in
+  let syms_decl syms (_,cl) =
+    List.fold_left syms_constr syms cl in
+  List.fold_left syms_decl Sid.empty tdl
 
 let create_data_decl tdl =
   if tdl = [] then raise EmptyDecl;
@@ -452,7 +467,7 @@ let create_data_decl tdl =
         Sls.add_new (DuplicateRecordField ls) ls s
     | Some ls -> raise (BadRecordField ls)
   in
-  let check_constr tys ty cll pjs (syms,news) (fs,pl) =
+  let check_constr tys ty cll pjs news (fs,pl) =
     ty_equal_check ty (Opt.get_exn (BadConstructor fs) fs.ls_value);
     let fs_pjs =
       try List.fold_left2 (check_proj ty) Sls.empty fs.ls_args pl
@@ -471,10 +486,9 @@ let create_data_decl tdl =
             else List.iter (check (seen || now)) tl
     in
     List.iter (check false) fs.ls_args;
-    let syms = List.fold_left syms_ty syms fs.ls_args in
-    syms, news_id news fs.ls_name
+    news_id news fs.ls_name
   in
-  let check_decl (syms,news) (ts,cl) =
+  let check_decl news (ts,cl) =
     let cll = List.length cl in
     if cl = [] then raise (EmptyAlgDecl ts);
     if ts.ts_def <> NoDef then raise (IllegalTypeAlias ts);
@@ -483,28 +497,36 @@ let create_data_decl tdl =
       List.fold_left (Opt.fold Sls.add_left) s pl) Sls.empty cl in
     let news = Sls.fold (fun pj s -> news_id s pj.ls_name) pjs news in
     let ty = ty_app ts (List.map ty_var ts.ts_args) in
-    List.fold_left (check_constr ts ty cll pjs) (syms,news) cl
+    List.fold_left (check_constr ts ty cll pjs) news cl
   in
-  let (syms,news) = List.fold_left check_decl (Sid.empty,Sid.empty) tdl in
-  mk_decl (Ddata tdl) syms news
+  let news = List.fold_left check_decl Sid.empty tdl in
+  mk_decl (Ddata tdl) news
+
+let syms_param_decl ls =
+  let syms = Opt.fold syms_ty Sid.empty ls.ls_value in
+  List.fold_left syms_ty syms ls.ls_args
 
 let create_param_decl ls =
-  let syms = Opt.fold syms_ty Sid.empty ls.ls_value in
-  let syms = List.fold_left syms_ty syms ls.ls_args in
   let news = Sid.singleton ls.ls_name in
-  mk_decl (Dparam ls) syms news
+  mk_decl (Dparam ls) news
+
+let syms_logic_decl ldl =
+  let syms_decl syms (ls,ld) =
+    let _, e = open_ls_defn ld in
+    let syms = List.fold_left syms_ty syms ls.ls_args in
+    syms_term syms e
+  in
+  List.fold_left syms_decl Sid.empty ldl
 
 let create_logic_decl ldl =
   if ldl = [] then raise EmptyDecl;
-  let check_decl (syms,news) (ls,((s,_,_) as ld)) =
+  let check_decl news (ls,(s,_,_)) =
     if not (ls_equal s ls) then raise (BadLogicDecl (ls, s));
-    let _, e = open_ls_defn ld in
-    let syms = List.fold_left syms_ty syms ls.ls_args in
-    syms_term syms e, news_id news ls.ls_name
+    news_id news ls.ls_name
   in
-  let (syms,news) = List.fold_left check_decl (Sid.empty,Sid.empty) ldl in
+  let news = List.fold_left check_decl Sid.empty ldl in
   let ldl = check_termination ldl in
-  mk_decl (Dlogic ldl) syms news
+  mk_decl (Dlogic ldl) news
 
 exception InvalidIndDecl of lsymbol * prsymbol
 exception NonPositiveIndDecl of lsymbol * prsymbol * lsymbol
@@ -526,11 +548,18 @@ let rec f_pos_ps sps pol f = match f.t_node, pol with
       f_pos_ps sps None f && f_pos_ps sps pol g && f_pos_ps sps pol h
   | _ -> TermTF.t_all (t_pos_ps sps) (f_pos_ps sps pol) f
 
+let syms_ind_decl idl =
+  let syms_ax syms (_,f) =
+    syms_term syms f in
+  let syms_decl syms (_,al) =
+    List.fold_left syms_ax syms al in
+  List.fold_left syms_decl Sid.empty idl
+
 let create_ind_decl s idl =
   if idl = [] then raise EmptyDecl;
   let add acc (ps,_) = Sls.add ps acc in
   let sps = List.fold_left add Sls.empty idl in
-  let check_ax ps (syms,news) (pr,f) =
+  let check_ax ps news (pr,f) =
     let rec clause acc f = match f.t_node with
       | Tquant (Tforall, f) ->
           let _,_,f = t_open_quant f in clause acc f
@@ -551,21 +580,32 @@ let create_ind_decl s idl =
           let ftv = t_ty_freevars Stv.empty f in
           if not (Stv.subset ftv gtv) then
             raise (UnboundTypeVar (Stv.choose (Stv.diff ftv gtv)));
-          syms_term syms f, news_id news pr.pr_name
+          news_id news pr.pr_name
       | _ -> raise (InvalidIndDecl (ps, pr))
   in
-  let check_decl (syms,news) (ps,al) =
+  let check_decl news (ps,al) =
     if al = [] then raise (EmptyIndDecl ps);
     let news = news_id news ps.ls_name in
-    List.fold_left (check_ax ps) (syms,news) al
+    List.fold_left (check_ax ps) news al
   in
-  let (syms,news) = List.fold_left check_decl (Sid.empty,Sid.empty) idl in
-  mk_decl (Dind (s, idl)) syms news
+  let news = List.fold_left check_decl Sid.empty idl in
+  mk_decl (Dind (s, idl)) news
+
+let syms_prop_decl f =
+  syms_term Sid.empty f
 
 let create_prop_decl k p f =
-  let syms = syms_term Sid.empty f in
   let news = news_id Sid.empty p.pr_name in
-  mk_decl (Dprop (k,p,check_fvs f)) syms news
+  mk_decl (Dprop (k,p,check_fvs f)) news
+
+let get_decl_syms d =
+  match d.d_node with
+  | Dtype ts -> syms_ty_decl ts
+  | Ddata dl -> syms_data_decl dl
+  | Dparam ls -> syms_param_decl ls
+  | Dlogic ldl -> syms_logic_decl ldl
+  | Dind (_, idl) -> syms_ind_decl idl
+  | Dprop (_,_,f) -> syms_prop_decl f
 
 (** Utilities *)
 
@@ -653,7 +693,7 @@ let known_add_decl kn0 decl =
     else raise (RedeclaredIdent id)
   in
   let kn = Mid.union check kn0 kn in
-  let unk = Mid.set_diff decl.d_syms kn in
+  let unk = Mid.set_diff (get_decl_syms decl) kn in
   if Sid.is_empty unk then kn
   else raise (UnknownIdent (Sid.choose unk))
 
@@ -797,8 +837,9 @@ let parse_record kn fll =
     | [{ ty_node = Tyapp (ts,_) }] -> ts
     | _ -> raise (BadRecordField fs) in
   let cs, pjl = match find_constructors kn ts with
-    | [cs,pjl] -> cs, List.map (Opt.get_exn (BadRecordField fs)) pjl
-    | _ -> raise (BadRecordField fs) in
+    | [cs,pjl] -> cs, List.map (Opt.get_exn (BadRecordUnnamed (fs, ts))) pjl
+    | _hd1 :: _hd2 :: _ -> raise (BadRecordCons (fs, ts))
+    | _ -> raise (BadRecordType (fs, ts)) in
   let pjs = Sls.of_list pjl in
   let flm = List.fold_left (fun m (pj,v) ->
     if not (Sls.mem pj pjs) then raise (BadRecordField pj) else
