@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -31,6 +31,7 @@ val dty_unify : dty -> dty -> unit (* raises Exit on failure *)
 val dty_int  : dty
 val dty_real : dty
 val dty_bool : dty
+val dty_str  : dty
 
 val dty_fold : (tysymbol -> 'a list -> 'a) ->
                (tvsymbol -> 'a) -> (int -> 'a) -> dty -> 'a
@@ -69,7 +70,7 @@ type dterm = private {
 and dterm_node =
   | DTvar of string * dty
   | DTgvar of vsymbol
-  | DTconst of Number.constant * dty
+  | DTconst of Constant.constant * dty
   | DTapp of lsymbol * dterm list
   | DTfapp of dterm * dterm
   | DTif of dterm * dterm * dterm
@@ -115,10 +116,14 @@ val dpattern : ?loc:Loc.position -> dpattern_node -> dpattern
 
 val dterm : Coercion.t -> ?loc:Loc.position -> dterm_node -> dterm
 
-(** Final stage *)
-
+(** Unused variables *)
 val debug_ignore_unused_var : Debug.flag
-val attr_w_unused_var_no : Ident.attribute
+val attr_w_unused_var_no : attribute
+
+(* Emit a warning if the variable is unused *)
+val check_used_var: term -> vsymbol -> unit
+
+(** Final stage *)
 
 val term : ?strict:bool -> ?keep_loc:bool -> dterm -> term
 
