@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -45,7 +45,7 @@ let list_attributes () =
 
 let attr_equal : attribute -> attribute -> bool = (==)
 let attr_hash a = a.attr_tag
-let attr_compare a1 a2 = Pervasives.compare a1.attr_tag a2.attr_tag
+let attr_compare a1 a2 = Int.compare a1.attr_tag a2.attr_tag
 
 (** Naming convention *)
 
@@ -151,7 +151,7 @@ type ident = {
   id_attrs  : Sattr.t;              (* identifier attributes *)
   id_loc    : Loc.position option;  (* optional location *)
   id_tag    : Weakhtbl.tag;         (* unique magical tag *)
-}
+  }
 
 module Id = MakeMSHW (struct
   type t = ident
@@ -171,7 +171,7 @@ type preid = {
 
 let id_equal : ident -> ident -> bool = (==)
 let id_hash id = Weakhtbl.tag_hash id.id_tag
-let id_compare id1 id2 = Pervasives.compare (id_hash id1) (id_hash id2)
+let id_compare id1 id2 = Int.compare (id_hash id1) (id_hash id2)
 
 (* constructors *)
 
@@ -275,7 +275,7 @@ let id_unique printer ?(sanitizer = same) id =
     Hid.replace printer.values id name;
     name
 
-let string_unique printer s = find_unique printer.indices s
+let string_unique printer s = find_unique printer.indices (printer.sanitizer s)
 
 let forget_id printer id =
   try
@@ -341,6 +341,8 @@ let sanitizer head rest n = sanitizer' head rest rest n
 
 let proxy_attr = create_attribute "mlw:proxy_symbol"
 
+let useraxiom_attr = create_attribute "useraxiom"
+
 let model_projected_attr = create_attribute "model_projected"
 let model_vc_post_attr = create_attribute "model_vc_post"
 
@@ -450,6 +452,15 @@ let extract_field attr =
   with
   | _ -> None
 
+(* Attributes used to name hypothesis *)
+let is_hyp_name_attr a =
+  Strings.has_prefix "hyp_name:" a.attr_string
+
+let get_hyp_name ~attrs =
+  try Some (Strings.remove_prefix "hyp_name:"
+              (Sattr.choose (Sattr.filter is_hyp_name_attr attrs)).attr_string)
+  with Not_found -> None
+
 (* Functions for working with ITP attributes *)
 
 let is_name_attr a =
@@ -464,12 +475,19 @@ let get_element_name ~attrs =
   | None -> None
   | Some name_attr ->
     let splitted1 = Strings.bounded_split ':' name_attr.attr_string 2 in
-    let correct_word = Str.regexp "^\\([A-Za-z]+\\)\\([A-Za-z0-9_']*\\)$" in
+    let correct_word = Re.Str.regexp "^\\([A-Za-z]+\\)\\([A-Za-z0-9_']*\\)$" in
     match splitted1 with
-    | ["name"; content] when Str.string_match correct_word content 0 ->
+    | ["name"; content] when Re.Str.string_match correct_word content 0 ->
         Some content
     | _ -> None
 
+let suffix_attr_name ~attrs suffix =
+  Sattr.fold (fun x acc ->
+      if is_name_attr x then
+          Sattr.add (create_attribute (x.attr_string ^ suffix)) acc
+      else
+        Sattr.add x acc) attrs Sattr.empty
+
 let id_unique_attr printer ?(sanitizer = same) id =
   try
     Hid.find printer.values id
