@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -166,6 +166,7 @@ val id_unique_attr :
     the "name:" attribute to generate the name instead of id.id_string *)
 
 val proxy_attr: attribute
+val useraxiom_attr: attribute
 
 (** {2 Attributes for handling counterexamples} *)
 
@@ -222,6 +223,11 @@ val get_model_trace_string : name:string -> attrs:Sattr.t -> string
     return ["mt_string"]. Raises [Not_found] if there is no attribute of
     the form ["model_trace:*"]. *)
 
+val get_element_name: attrs:Sattr.t -> string option
+
+val suffix_attr_name: attrs:Sattr.t -> string -> Sattr.t
+(** Add a suffix to all "name" attributes of [attrs] *)
+
 val compute_model_trace_field: ident option -> int -> Sattr.t
 (** Take an optional projection name and the depths of its occurence and return
     the built field attribute associated *)
@@ -232,3 +238,7 @@ val extract_field: attribute -> (int * string) option
 val get_model_trace_attr : attrs:Sattr.t -> attribute
 (** Return an attribute of the form ["model_trace:*"].
     Raises [Not_found] if there is no such attribute. *)
+
+val get_hyp_name: attrs:Sattr.t -> string option
+(** If attrs contains an attribute of the form ["hyp_name:<s>"] returns
+    [Some <s>] or [None] if no attribute have this form *)
