@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -36,63 +36,113 @@ type float_type =
   | Float_value of string * string * string
   | Float_hexa of string * float
 
+type repr = BinRepr | HexaRepr
 
 let interp_float ?(interp=true) b eb sb =
-    try
-      (* We don't interpret when this is disable *)
-      if not interp then
-        raise Exit;
-      let is_neg = match b with
-        | "#b0" -> false
-        | "#b1" -> true
-        | _ -> raise Exit
-      in
-      if String.length eb = 13 && String.sub eb 0 2 = "#b" &&
-         String.length sb = 15 && String.sub sb 0 2 = "#x" then
-         (* binary 64 *)
-         let exp_base2 = String.sub eb 2 11 in
-         let mant_base16 = String.sub sb 2 13 in
-         let exp = int_of_string ("0b" ^ exp_base2) in
-         if exp = 0 then (* subnormals *)
-           let s = (if is_neg then "-" else "")^
-                   "0x0."^mant_base16^"p-1023"
-            in Float_hexa(s,float_of_string s)
-           else if exp = 2047 then (* infinities and NaN *)
-             if mant_base16="0000000000000" then
-                if is_neg then Minus_infinity else Plus_infinity
-                else Not_a_number
-           else
-           let exp = exp - 1023 in
-           let s = (if is_neg then "-" else "")^
-                   "0x1."^mant_base16^"p"^(string_of_int exp)
-           in Float_hexa(s,float_of_string s)
-      else
-      if String.length eb = 4 && String.sub eb 0 2 = "#x" &&
-         String.length sb = 25 && String.sub sb 0 2 = "#b" then
-         (* binary 32 *)
-         let exp_base16 = String.sub eb 2 2 in
-         let mant_base2 = String.sub sb 2 23 in
-         let mant_base16 =
-           Format.asprintf "%06x" (2*int_of_string ("0b" ^ mant_base2))
-         in
-         let exp = int_of_string ("0x" ^ exp_base16) in
-         if exp = 0 then (* subnormals *)
-           let s = (if is_neg then "-" else "")^
-                   "0x0."^mant_base16^"p-127"
-            in Float_hexa(s,float_of_string s)
-           else if exp = 255 then (* infinities and NaN *)
-             if mant_base16="0000000" then
-                if is_neg then Minus_infinity else Plus_infinity
-                else Not_a_number
-           else
-           let exp = exp - 127 in
-           let s = (if is_neg then "-" else "")^
-                   "0x1."^mant_base16^"p"^(string_of_int exp)
-           in Float_hexa(s,float_of_string s)
-      else raise Exit
-   with Exit -> Float_value (b, eb, sb)
+  let convert (r: repr) s =
+    (* Convert number of less than 63 bits TODO *)
+    match r with
+    | BinRepr when String.length s <= 62 -> int_of_string ("0b" ^ s)
+    | HexaRepr when String.length s <= 15 -> int_of_string ("0x" ^ s)
+    | _ -> (* Too big to fit into an integer *) raise Exit in
+
+  let rec pow x n =
+    assert (n <= 63);
+    match n with
+    | p when p <= 0 -> 1
+    | 1 -> x
+    | p -> x * pow x (p-1) in
+
+  let get_repr s len =
+    if String.sub s 0 2 = "#b" then
+      BinRepr, String.sub s 2 (len - 2), (len - 2)
+    else if String.sub s 0 2 = "#x" then
+      HexaRepr, String.sub s 2 (len - 2), (len - 2) * 4
+    else raise Exit in
+
+  let pad_with_zeros width s =
+    let s_len = String.length s in
+    let filled =
+      let len = width - s_len in
+      if len <= 0 then "" else String.make len '0' in
+    filled ^ s in
+
+  let only_zeros s =
+    (* Check that there is only 0 in the string *)
+    let b = ref true in
+    for i = 0 to String.length s - 1 do
+      b := !b && (s.[i] = '0')
+    done;
+    !b in
+
+  try
+    (* We don't interpret when this is disable *)
+    if not interp then
+      raise Exit;
+    let is_neg = match b with
+      | "#b0" -> false
+      | "#b1" -> true
+      | _ -> raise Exit
+    in
+    let eb_len = String.length eb in
+    let sb_len = String.length sb in
+    if eb_len < 2 || sb_len < 2 then raise Exit;
+    let eb_repr, eb, eb_size = get_repr eb eb_len in
+    let sb_repr, sb, sb_size = get_repr sb sb_len in
+
+    (* Values for the type of float obtained *)
+    (* Exponent bias *)
+    let exp_bias = (pow 2 (eb_size - 1)) - 1 in
+    (* Maximum exponent value *)
+    let max_exp = (pow 2 eb_size) - 1 in
+    (* Length of the hexadecimal representation (after the ".") *)
+    let length_of_number =
+      if sb_size mod 4 = 0 then sb_size / 4 else sb_size / 4 + 1 in
+
+    (* Compute exponent (int) and mantissa (string of hexa) *)
+    let exp = convert eb_repr eb in
+    let mant_base16 =
+      let c = convert sb_repr sb in
+      let c =
+        (* The hex value is used after the decimal point. So we need to adjust
+           it to the number of binary elements there are.
+           Example in 32bits: significand is 23 bits, and the hexadecimal
+           representation will have a multiple of 4 bits (ie 24). So, we need to
+           multiply by two to account the difference. *)
+        if sb_repr = BinRepr then
+          let adjust = 4 - sb_size mod 4 in
+          (* No adjustment needed *)
+          if adjust = 4 then
+            c
+          else
+            (pow 2 adjust) * c
+        else
+          c in
+      pad_with_zeros length_of_number (Format.asprintf "%x" c)
+    in
+    match exp with
+    | 0 (* subnormals and zero *) ->
+        (* Case for zero *)
+        if only_zeros mant_base16 then
+          if is_neg then Minus_zero else Plus_zero
+        else
+          (* Subnormals *)
+          let s = (if is_neg then "-" else "")^
+                  "0x0."^mant_base16^"p-"^(string_of_int exp_bias)
+          in Float_hexa (s, float_of_string s)
+    | e when e = max_exp (* infinities and NaN *) ->
+        if only_zeros mant_base16 then
+          if is_neg then Minus_infinity else Plus_infinity
+        else Not_a_number
+    | _ (* Normal floats *) ->
+        let exp = exp - exp_bias in
+        let s = (if is_neg then "-" else "")^
+                "0x1."^mant_base16^"p"^(string_of_int exp)
+        in Float_hexa(s,float_of_string s)
+  with Exit -> Float_value (b, eb, sb)
 
 type model_value =
+ | String of string
  | Integer of string
  | Decimal of (string * string)
  | Fraction of (string * string)
@@ -105,7 +155,7 @@ type model_value =
  | Apply of string * model_value list
  | Unparsed of string
 and  arr_index = {
-  arr_index_key : string; (* Even array indices can exceed MAX_INT with Z3 *)
+  arr_index_key : model_value;
   arr_index_value : model_value;
 }
 and model_array = {
@@ -168,6 +218,10 @@ let convert_float_value f =
 
 let rec convert_model_value value : Json_base.json =
   match value with
+  | String s ->
+      let m = Mstr.add "type" (Json_base.String "String") Mstr.empty in
+      let m = Mstr.add "val" (Json_base.String s) m in
+      Json_base.Record m
   | Integer s ->
       let m = Mstr.add "type" (Json_base.String "Integer") Mstr.empty in
       let m = Mstr.add "val" (Json_base.String s) m in
@@ -225,7 +279,7 @@ and convert_indices indices =
   match indices with
   | [] -> []
   | index :: tail ->
-    let m = Mstr.add "indice" (Json_base.String index.arr_index_key) Mstr.empty in
+    let m = Mstr.add "indice" (convert_model_value index.arr_index_key) Mstr.empty in
     let m = Mstr.add "value" (convert_model_value index.arr_index_value) m in
     Json_base.Record m :: convert_indices tail
 
@@ -275,15 +329,23 @@ let print_float_human fmt f =
   | Not_a_number ->
       fprintf fmt "NaN"
   | Float_value (b, eb, sb) ->
-     fprintf fmt "float_bits(%s,%s,%s)" b eb sb
+      fprintf fmt "float_bits(%s,%s,%s)" b eb sb
   | Float_hexa(s,f) -> fprintf fmt "%s (%g)" s f
 
 let rec print_array_human fmt (arr: model_array) =
-  fprintf fmt "@[(";
-  List.iter (fun e ->
-    fprintf fmt "@[%s =>@ %a,@]" e.arr_index_key print_model_value_human e.arr_index_value)
-    arr.arr_indices;
-  fprintf fmt "@[others =>@ %a@])@]" print_model_value_human arr.arr_others
+  let print_others fmt v =
+    fprintf fmt "@[others =>@ %a@]"
+      print_model_value_human v in
+  let print_key_val fmt arr =
+    let {arr_index_key = key; arr_index_value = v} = arr in
+    fprintf fmt "@[%a =>@ %a@]"
+      print_model_value_human key print_model_value_human v in
+  fprintf fmt
+    "@[(%a%a)@]"
+    (Pp.print_list_delim
+       ~start:Pp.nothing ~stop:Pp.comma ~sep:Pp.comma print_key_val)
+    arr.arr_indices
+    print_others arr.arr_others
 
 and print_record_human fmt r =
   match r with
@@ -301,9 +363,45 @@ and print_proj_human fmt p =
   fprintf fmt "@[{%s =>@ %a}@]"
     s print_model_value_human v
 
+and print_integer fmt (i: string) =
+
+  (* TODO This conversion is adhoc plus it is not the most efficient. But it
+     should be ok as we use this only for display. *)
+  let print_big_int fmt (bn: BigInt.t) =
+    let to_hexa n = sprintf "%X" n in
+    let base = BigInt.of_int 16 in
+    let rec convert_big_int_hexa (bn: BigInt.t) : string =
+      if BigInt.lt bn base then
+        to_hexa (BigInt.to_int bn)
+      else
+        let (q, r) = BigInt.euclidean_div_mod bn base in
+        convert_big_int_hexa q ^ to_hexa (BigInt.to_int r)
+    in
+    fprintf fmt "%s (%s0x%s)" (BigInt.to_string bn)
+      (if (BigInt.sign bn) >= 0 then "" else "-")
+      (convert_big_int_hexa (BigInt.abs bn))
+  in
+  let bn = BigInt.of_string i in
+  try
+    let i = BigInt.to_int bn in
+    if i >= 0 then
+      fprintf fmt "%d (0x%X)" i i
+    else
+      fprintf fmt "%d (-0x%X)" i (-i)
+  with Failure _ (* "int_of_big_int" *) ->
+    print_big_int fmt bn
+
+and print_bv fmt (bv: string) =
+  (* TODO Not implemented yet. Ideally, fix the differentiation made in the
+     parser between Bv_int and Bv_sharp -> convert Bv_int to Bitvector not
+     Integer. And print Bv_int exactly like Bv_sharp.
+  *)
+  Format.fprintf fmt "%s" bv
+
 and print_model_value_human fmt (v: model_value) =
   match v with
-  | Integer s -> fprintf fmt "%s" s
+  | String s -> Constant.print_string_def fmt s
+  | Integer s -> print_integer fmt s
   | Decimal (s1,s2) -> fprintf fmt "%s" (s1 ^ "." ^ s2)
   | Fraction (s1, s2) -> fprintf fmt "%s" (s1 ^ "/" ^ s2)
   | Float f -> print_float_human fmt f
@@ -315,7 +413,8 @@ and print_model_value_human fmt (v: model_value) =
   | Array arr -> print_array_human fmt arr
   | Record r -> print_record_human fmt r
   | Proj p -> print_proj_human fmt p
-  | Bitvector s -> fprintf fmt "%s" s
+  | Bitvector s ->
+      print_bv fmt s
   | Unparsed s -> fprintf fmt "%s" s
 
 (*
@@ -460,6 +559,8 @@ let default_model = {
   model_files = empty_model;
 }
 
+let get_model_elements m = List.concat (List.concat (List.map IntMap.values (StringMap.values m.model_files)))
+
 type model_parser =  string -> Printer.printer_mapping -> model
 
 type raw_model_parser =
@@ -485,7 +586,11 @@ let apply_location_label ~at_loc ~attrs me_name =
           if at_loc = (Filename.basename loc_file, loc_line) &&
              not (Sstr.mem label labels_added)
           then
-            (Sstr.add label labels_added, me_name ^ " at " ^ label)
+            (* Specific case for printing old *)
+            if label = "'Old" then
+              (Sstr.add label labels_added, "old " ^ me_name)
+            else
+              (Sstr.add label labels_added, me_name ^ " at " ^ label)
           else
             (labels_added, me_name)
       | _ -> (labels_added, me_name))
@@ -502,22 +607,23 @@ let print_model_element ~at_loc ~print_attrs print_model_value me_name_trans fmt
     let attrs = m_element.me_name.men_attrs in
     let me_name = apply_location_label ~at_loc ~attrs me_name in
     if print_attrs then
-      fprintf fmt  "%s, [%a] = %a"
+      fprintf fmt  "@[%s, @[[%a]@] =@ %a@]"
         me_name
         (Pp.print_list Pp.comma Pretty.print_attr)
         (Sattr.elements m_element.me_name.men_attrs)
         print_model_value m_element.me_value
     else
-      fprintf fmt  "%s = %a"
+      fprintf fmt  "@[%s =@ %a@]"
         me_name
         print_model_value m_element.me_value
 
-let print_model_elements ~at_loc ~print_attrs ?(sep = "\n")
+let print_model_elements ~at_loc ~print_attrs ?(sep = Pp.newline)
     print_model_value me_name_trans fmt m_elements
   =
-  Pp.print_list (fun fmt () -> Pp.string fmt sep)
-    (print_model_element ~at_loc ~print_attrs print_model_value me_name_trans)
-    fmt m_elements
+  fprintf fmt "@[%a@]"
+    (Pp.print_list sep
+       (print_model_element ~at_loc ~print_attrs print_model_value me_name_trans))
+    m_elements
 
 let print_model_file ~print_attrs ~print_model_value fmt
     me_name_trans filename model_file
@@ -525,13 +631,15 @@ let print_model_file ~print_attrs ~print_model_value fmt
   (* Relativize does not work on nighly bench: using basename instead. It
      hides the local paths.  *)
   let filename = Filename.basename filename  in
-  fprintf fmt "File %s:" filename;
+  fprintf fmt "@[<v 0>File %s:@\n" filename;
   IntMap.iter
     (fun line m_elements ->
-      fprintf fmt "@\nLine %d:@\n" line;
-      print_model_elements ~at_loc:(filename,line) ~print_attrs print_model_value me_name_trans fmt m_elements)
+      fprintf fmt "  @[<v 2>Line %d:@\n" line;
+      print_model_elements ~at_loc:(filename,line) ~print_attrs print_model_value me_name_trans fmt m_elements;
+      fprintf fmt "@]@\n"
+    )
     model_file;
-  fprintf fmt "@\n"
+  fprintf fmt "@]"
 
 let why_name_trans me_name =
   match me_name.men_kind with
@@ -574,9 +682,9 @@ let get_elements model_file line_number =
 
 let get_padding line =
   try
-    let r = Str.regexp " *" in
-    ignore (Str.search_forward r line 0);
-    Str.matched_string line
+    let r = Re.Str.regexp " *" in
+    ignore (Re.Str.search_forward r line 0);
+    Re.Str.matched_string line
   with Not_found -> ""
 
 (* This assumes that l is sorted and split the list of locations in two:
@@ -621,10 +729,10 @@ let interleave_line
   try
     let model_elements = IntMap.find line_number model_file in
     let cntexmp_line =
-      asprintf "%s%s%a%s"
+      asprintf "@[<h 0>%s%s%a%s@]"
         (get_padding line)
         start_comment
-        (print_model_elements ~at_loc:(filename,line_number) ~print_attrs ~sep:"; " print_model_value_human me_name_trans) model_elements
+        (print_model_elements ~sep:Pp.semi ~at_loc:(filename,line_number) ~print_attrs print_model_value_human me_name_trans) model_elements
         end_comment in
 
     (* We need to know how many lines will be taken by the counterexample. This
@@ -639,14 +747,10 @@ let interleave_line
 
 
 let interleave_with_source
-    ~print_attrs
-    ?(start_comment="(* ")
-    ?(end_comment=" *)")
+    ~print_attrs ?(start_comment="(* ") ?(end_comment=" *)")
     ?(me_name_trans = why_name_trans)
-    model
-    ~rel_filename
-    ~source_code
-    ~locations =
+    model ~rel_filename ~source_code ~locations
+  =
   let locations =
     List.sort (fun x y -> compare (fst x) (fst y))
       (List.filter (fun x -> let (f, _, _, _) = Loc.get (fst x) in f = rel_filename) locations)
@@ -664,7 +768,7 @@ let interleave_with_source
     let model_file = snd (StringMap.choose model_files) in
     let src_lines_up_to_last_cntexmp_el source_code model_file =
       let (last_cntexmp_line, _) = IntMap.max_binding model_file in
-      Str.bounded_split (Str.regexp "^") source_code (last_cntexmp_line+1)
+      Re.Str.bounded_split (Re.Str.regexp "^") source_code (last_cntexmp_line+1)
     in
     let (source_code, _, _, _, gen_loc) =
       List.fold_left
@@ -837,7 +941,7 @@ let recover_name list_projs term_map raw_name =
 let rec replace_projection (const_function: string -> string) model_value =
   match model_value with
   | Integer _ | Decimal _ | Fraction _ | Float _ | Boolean _ | Bitvector _
-    | Unparsed _ -> model_value
+    | String  _ | Unparsed _ -> model_value
   | Array a ->
       Array (replace_projection_array const_function a)
   | Record r ->
