@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -23,12 +23,24 @@ open Decl
 open Theory
 open Task
 
+val prio_binop: binop -> int
+
+val protect_on: bool -> ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
+  ('a, 'b, 'c, 'd, 'e, 'f) format6
+
+type syntax =
+| Is_array of string
+| Is_getter of string
+| Is_none
+
+val get_element_syntax: attrs:Sattr.t -> syntax
+
 module type Printer = sig
 
     val tprinter : ident_printer  (* type symbols *)
     val aprinter : ident_printer  (* type variables *)
     val sprinter : ident_printer  (* variables and functions *)
-    val pprinter : ident_printer  (* propoition names *)
+    val pprinter : ident_printer  (* proposition names *)
 
     val forget_all : unit -> unit     (* flush id_unique *)
     val forget_tvs : unit -> unit     (* flush id_unique for type vars *)
@@ -58,7 +70,7 @@ module type Printer = sig
     val print_quant : formatter -> quant -> unit      (* quantifier *)
     val print_binop : asym:bool -> formatter -> binop -> unit (* binary operator *)
     val print_pat : formatter -> pattern -> unit      (* pattern *)
-    val print_term : formatter -> term -> unit        (* term *)
+    val print_term : formatter -> term -> unit   (* term *)
 
     val print_attr : formatter -> attribute -> unit
     val print_loc : formatter -> Loc.position -> unit
@@ -90,6 +102,29 @@ module type Printer = sig
 
 include Printer
 
-val create :  Ident.ident_printer -> Ident.ident_printer ->
-              Ident.ident_printer -> Ident.ident_printer ->
-              bool -> (module Printer)
+type any_pp =
+  | Pp_term of (Term.term * int) (* Term and priority *)
+  | Pp_ty of (Ty.ty * int * bool) (* ty * prio * q *)
+  | Pp_decl of (bool * Ty.tysymbol * (Term.lsymbol * Term.lsymbol option list) list)
+    (* [Pp_decl (fst, ts, csl)]: Declaration of type [ts] with constructors
+       [csl] as [fst] *)
+  | Pp_ts of Ty.tysymbol (* Print tysymbol *)
+  | Pp_ls of Term.lsymbol (* Print lsymbol *)
+  | Pp_id of Ident.ident (* Print ident *)
+  | Pp_cs of Term.lsymbol (* Print constructors *)
+  | Pp_vs of Term.vsymbol (* Print vsymbol *)
+  | Pp_trigger of Term.trigger (* Print triggers *)
+  | Pp_forget of Term.vsymbol list (* Forget all the vsymbols listed *)
+  | Pp_forget_tvs (* execute forget_tvs *)
+
+
+val create :
+  ?print_ext_any:(any_pp Pp.pp -> any_pp Pp.pp) ->
+  Ident.ident_printer -> Ident.ident_printer ->
+  Ident.ident_printer -> Ident.ident_printer ->
+  bool -> (module Printer)
+(** `create spr apr tpr ppr forget` creates a new pretty-printing
+   module from the printer `spr` for variables and functions, `apr`
+   for type variables, `tpr` for type symbols and `ppr for proposition
+   names`. When the Boolean `forget` is true then all recorded names
+   are forgotten between printing of each tasks. *)
