@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -88,7 +88,7 @@ let () = register_printer
 
 (*
 let opt_search_forward re s pos =
-  try Some (Str.search_forward re s pos) with Not_found -> None
+  try Some (Re.Str.search_forward re s pos) with Not_found -> None
 *)
 
 (* specialized version of opt_search_forward, searching for strings
@@ -110,7 +110,8 @@ let opt_search_forward s pos =
         incr i;
         b := !i;
         begin match s.[!i] with
-        | 't' | 'v' -> incr i
+        | '%' -> incr i; raise Exit
+        | 'a'..'z' -> incr i
         | _ -> ()
         end;
         let e = !i in
@@ -132,12 +133,13 @@ let opt_search_forward_literal_format s pos =
         incr i;
         b := !i;
         begin match s.[!i] with
+        | '%' -> incr i; raise Exit
         | 's' | 'e' | 'm' -> incr i; (* float literals *)
         | _ -> ()
         end;
         while !i < l && is_digit s.[!i] do incr i done;
         begin match s.[!i] with
-        | 'b' | 'x' | 'o' | 'd' -> incr i; raise Exit
+        | 'b' | 'x' | 'o' | 'd' | 'c' -> incr i; raise Exit
         | _ -> ()
         end;
       end;
@@ -154,25 +156,25 @@ let global_substitute_fmt search_fun repl_fun text fmt =
       pp_print_string fmt (String.sub text start (len - start))
     | Some(pos,end_pos) ->
       pp_print_string fmt (String.sub text start (pos - start - 1));
-      repl_fun text pos end_pos fmt;
+      if text.[pos] = '%' then pp_print_char fmt '%'
+      else repl_fun text pos end_pos fmt;
       replace end_pos
   in
   replace 0
 
 let iter_group search_fun iter_fun text =
-  let rec iter start last_was_empty =
-    let startpos = if last_was_empty then start + 1 else start in
-    if startpos < String.length text then
-      match search_fun text startpos with
-      | None -> ()
-      | Some (pos,end_pos) ->
-          iter_fun text pos end_pos;
-          iter end_pos (end_pos = pos)
+  let rec iter start =
+    match search_fun text start with
+    | None -> ()
+    | Some (pos, end_pos) ->
+        if text.[pos] <> '%' then iter_fun text pos end_pos;
+        iter end_pos
   in
-  iter 0 false
+  iter 0
 
 exception BadSyntaxIndex of int
 exception BadSyntaxArity of int * int
+exception BadSyntaxKind of char
 
 let int_of_string s =
   try int_of_string s
@@ -180,35 +182,49 @@ let int_of_string s =
     Format.eprintf "bad argument for int_of_string: %s@." s;
     assert false
 
-let check_syntax s len =
+let iter_on_syntax_parameters s f =
   let arg s b e =
     let i = int_of_string (String.sub s b (e-b)) in
+    f i in
+  iter_group opt_search_forward arg s
+
+(** [check_syntax s len] checks that all %i arguments in [s]
+    satisfy [0 < i <= len]. *)
+let check_syntax s len =
+  iter_on_syntax_parameters s (fun i ->
     if i <= 0 then raise (BadSyntaxIndex i);
     if i > len then raise (BadSyntaxArity (len,i));
-    ()
-  in
-  iter_group opt_search_forward arg s
+    ())
+
+let syntax_arity s =
+  let arity = ref 0 in
+  iter_on_syntax_parameters s (fun i ->
+    if i <= 0 then raise (BadSyntaxIndex i);
+    arity := max i !arity);
+  !arity
 
 let check_syntax_logic ls s =
   let len = List.length ls.ls_args in
   let ret = ls.ls_value <> None in
   let nfv = Stv.cardinal (ls_ty_freevars ls) in
   let arg s b e =
-    if s.[b] = 't' then begin
+    match s.[b] with
+    | 't' ->
       let grp = String.sub s (b+1) (e-b-1) in
       let i = int_of_string grp in
       if i < 0 || (not ret && i = 0) then raise (BadSyntaxIndex i);
       if i > len then raise (BadSyntaxArity (len,i))
-    end else if s.[b] = 'v' then begin
+    | 'v' ->
       let grp = String.sub s (b+1) (e-b-1) in
       let i = int_of_string grp in
       if i < 0 || i >= nfv then raise (BadSyntaxIndex i)
-    end else begin
+    | 'a'..'z' as c ->
+      raise (BadSyntaxKind c)
+    | _ ->
       let grp = String.sub s b (e-b) in
       let i = int_of_string grp in
       if i <= 0 then raise (BadSyntaxIndex i);
       if i > len then raise (BadSyntaxArity (len,i));
-    end
   in
   iter_group opt_search_forward arg s
 
@@ -222,13 +238,6 @@ let check_syntax_literal _ts s =
   (* if !count <> 1 then *)
     (* raise (BadSyntaxArity (1,!count)) *)
 
-let syntax_arguments s print fmt l =
-  let args = Array.of_list l in
-  let repl_fun s b e fmt =
-    let i = int_of_string (String.sub s b (e-b)) in
-    print fmt args.(i-1) in
-  global_substitute_fmt opt_search_forward repl_fun s fmt
-
 (* return the type arguments of a symbol application, sorted according
    to their (formal) names *)
 let get_type_arguments t = match t.t_node with
@@ -242,58 +251,88 @@ let get_type_arguments t = match t.t_node with
   | _ ->
       [||]
 
-let gen_syntax_arguments_typed ty_of tys_of s print_arg print_type t fmt l =
-  let args = Array.of_list l in
+let gen_syntax_arguments_prec fmt s print pl =
+  let precs = Array.of_list pl in
+  let lp = Array.length precs in
+  let num = ref 1 in
   let repl_fun s b e fmt =
-    if s.[b] = 't' then
-      let grp = String.sub s (b+1) (e-b-1) in
-      let i = int_of_string grp in
-      if i = 0
-      then print_type fmt (ty_of t)
-      else print_type fmt (ty_of args.(i-1))
-    else if s.[b] = 'v' then
-      let grp = String.sub s (b+1) (e-b-1) in
-      let m = tys_of t in
-      print_type fmt m.(int_of_string grp)
-    else
-      let grp = String.sub s b (e-b) in
-      let i = int_of_string grp in
-      print_arg fmt args.(i-1) in
+    let (c, i) =
+      match s.[b] with
+      | 'a'..'z' as c -> (Some c, String.sub s (b + 1) (e - b - 1))
+      | _ -> (None, String.sub s b (e - b)) in
+    let i = int_of_string i in
+    let p =
+      if !num < lp then precs.(!num)
+      else if lp = 0 then 0
+      else precs.(0) - 1 in
+    incr num;
+    print fmt p c i in
   global_substitute_fmt opt_search_forward repl_fun s fmt
 
-let syntax_arguments_typed =
-  gen_syntax_arguments_typed t_type get_type_arguments
+let syntax_arguments_prec s print pl fmt l =
+  let args = Array.of_list l in
+  let pr fmt p c i =
+    match c with
+    | None -> print p fmt args.(i - 1)
+    | Some c -> raise (BadSyntaxKind c) in
+  gen_syntax_arguments_prec fmt s pr pl
 
-let syntax_range_literal s fmt c =
+let syntax_arguments s print fmt l =
+  syntax_arguments_prec s (fun _ f a -> print f a) [] fmt l
+
+let syntax_arguments_typed_prec s print_arg print_type t pl fmt l =
+  let args = Array.of_list l in
+  let tys = lazy (get_type_arguments t) in
+  let pr fmt p c i =
+    match c with
+    | None -> print_arg p fmt args.(i - 1)
+    | Some 't' ->
+        let v = if i = 0 then t else args.(i - 1) in
+        print_type fmt (t_type v)
+    | Some 'v' ->
+        print_type fmt (Lazy.force tys).(i)
+    | Some c -> raise (BadSyntaxKind c) in
+  gen_syntax_arguments_prec fmt s pr pl
+
+let syntax_arguments_typed s print_arg print_type t fmt l =
+  syntax_arguments_typed_prec s (fun _ f a -> print_arg f a) print_type t [] fmt l
+
+let syntax_range_literal ?(cb=None) s fmt c =
   let f s b e fmt =
-    let v = Number.compute_int_literal c.Number.ic_abs in
-    let base = match s.[e-1] with
-      | 'x' -> 16
-      | 'd' -> 10
-      | 'o' -> 8
-      | 'b' -> 2
-      | _ -> assert false
-    in
-    let digits =
-      if e > b + 1 then
-        Some (int_of_string (String.sub s b (e-b-1)))
-      else
-        None
-    in
-    if base = 10 then begin
-        if c.Number.ic_negative then fprintf fmt "-";
-        Number.print_in_base base digits fmt v
-      end
-    else
-      let v =
-        if c.Number.ic_negative then
-          match digits with
-            | Some d ->
-               BigInt.sub (BigInt.pow_int_pos base d) v
-            | None -> failwith ("number of digits must be given for printing negative literals in base " ^ string_of_int base)
-        else v
+    try
+      let v = c.Number.il_int in
+      let base = match s.[e-1] with
+        | 'x' -> 16
+        | 'd' -> 10
+        | 'o' -> 8
+        | 'b' -> 2
+        | 'c' -> raise Exit
+        | _ -> assert false
       in
-      Number.print_in_base base digits fmt v
+      let digits =
+        if e > b + 1 then
+          Some (int_of_string (String.sub s b (e-b-1)))
+        else
+          None
+      in
+      if base = 10 then begin
+          if BigInt.lt v BigInt.zero then fprintf fmt "-";
+          Number.print_in_base base digits fmt (BigInt.abs v)
+        end
+      else
+        let v =
+          if BigInt.lt v BigInt.zero then
+            match digits with
+            | Some d ->
+               BigInt.add (BigInt.pow_int_pos base d) v
+            | None -> failwith ("number of digits must be given for printing negative literals in base " ^ string_of_int base)
+          else v
+        in
+        Number.print_in_base base digits fmt v
+    with Exit ->
+      match cb with
+      | Some cb -> cb fmt c
+      | None -> failwith ("custom format string with no callback passed")
   in
   global_substitute_fmt opt_search_forward_literal_format f s fmt
 
@@ -312,8 +351,8 @@ let syntax_float_literal s fp fmt c =
       else
         None
     in
-    let e,m = Number.compute_float c.Number.rc_abs fp in
-    let sg = if c.Number.rc_negative then BigInt.one else BigInt.zero in
+    let e,m = Number.compute_float c fp in
+    let m,sg = if BigInt.lt m BigInt.zero then BigInt.abs m, BigInt.one else m, BigInt.zero in
     match s.[b] with
     | 's' -> Number.print_in_base base digits fmt sg
     | 'e' -> Number.print_in_base base digits fmt e
@@ -550,6 +589,8 @@ let () = Exn_printer.register (fun fmt exn -> match exn with
       fprintf fmt "Bad argument index %d, must start with 1" i
   | BadSyntaxArity (i1,i2) ->
       fprintf fmt "Bad argument index %d, must end with %d" i2 i1
+  | BadSyntaxKind c ->
+      fprintf fmt "Unrecognized argument kind '%c'" c
   | Unsupported s ->
       fprintf fmt "@[<hov 3> Uncaught exception 'Unsupported %s'@]" s
   | UnsupportedType (e,s) ->
