@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -224,7 +224,7 @@ type term = {
 
 and term_node =
   | Tvar of vsymbol
-  | Tconst of Number.constant
+  | Tconst of Constant.constant
   | Tapp of lsymbol * term list
   | Tif of term * term * term
   | Tlet of term * term_bound
@@ -275,7 +275,7 @@ let rec descend vml t = match t.t_node with
       find vs vml
   | _ -> Trm (t, vml)
 
-let t_compare trigger attr t1 t2 =
+let t_compare trigger attr loc t1 t2 =
   let comp_raise c =
     if c < 0 then raise CompLT else if c > 0 then raise CompGT in
   let perv_compare h1 h2 = comp_raise (Pervasives.compare h1 h2) in
@@ -320,7 +320,8 @@ let t_compare trigger attr t1 t2 =
   let rec t_compare bnd vml1 vml2 t1 t2 =
     if t1 != t2 || vml1 <> [] || vml2 <> [] then begin
       comp_raise (oty_compare t1.t_ty t2.t_ty);
-      if attr then comp_raise (Sattr.compare t1.t_attrs t2.t_attrs) else ();
+      if attr then comp_raise (Sattr.compare t1.t_attrs t2.t_attrs);
+      if loc then comp_raise (Opt.compare Loc.compare t1.t_loc t2.t_loc);
       match descend vml1 t1, descend vml2 t2 with
       | Bnd i1, Bnd i2 -> perv_compare i1 i2
       | Bnd _, Trm _ -> raise CompLT
@@ -330,14 +331,7 @@ let t_compare trigger attr t1 t2 =
           | Tvar v1, Tvar v2 ->
               comp_raise (vs_compare v1 v2)
           | Tconst c1, Tconst c2 ->
-              let open Number in
-              begin match c1, c2 with
-              | ConstInt { ic_negative = s1; ic_abs = IConstRaw b1 },
-                ConstInt { ic_negative = s2; ic_abs = IConstRaw b2 } ->
-                  perv_compare s1 s2;
-                  comp_raise (BigInt.compare b1 b2)
-              | _, _ -> perv_compare c1 c2
-              end
+              comp_raise (Constant.compare_const c1 c2)
           | Tapp (s1,l1), Tapp (s2,l2) ->
               comp_raise (ls_compare s1 s2);
               List.iter2 (t_compare bnd vml1 vml2) l1 l2
@@ -380,8 +374,8 @@ let t_compare trigger attr t1 t2 =
               t_compare bnd vml1 vml2 f1 f2
           | Tbinop (op1,f1,g1), Tbinop (op2,f2,g2) ->
               perv_compare op1 op2;
-              t_compare bnd vml1 vml2 f1 f2;
-              t_compare bnd vml1 vml2 g1 g2
+              t_compare bnd vml1 vml2 g1 g2;
+              t_compare bnd vml1 vml2 f1 f2
           | Tnot f1, Tnot f2 ->
               t_compare bnd vml1 vml2 f1 f2
           | Ttrue, Ttrue -> ()
@@ -402,17 +396,11 @@ let t_compare trigger attr t1 t2 =
   try t_compare 0 [] [] t1 t2; 0
   with CompLT -> -1 | CompGT -> 1
 
-let t_equal t1 t2 = (t_compare true true t1 t2 = 0)
-
-let t_equal_nt_na t1 t2 = (t_compare false false t1 t2 = 0)
-
-let t_compare = t_compare true true
-
 let t_similar t1 t2 =
   oty_equal t1.t_ty t2.t_ty &&
   match t1.t_node, t2.t_node with
     | Tvar v1, Tvar v2 -> vs_equal v1 v2
-    | Tconst c1, Tconst c2 -> c1 = c2
+    | Tconst c1, Tconst c2 -> Constant.compare_const c1 c2 = 0
     | Tapp (s1,l1), Tapp (s2,l2) -> ls_equal s1 s2 && Lists.equal (==) l1 l2
     | Tif (f1,t1,e1), Tif (f2,t2,e2) -> f1 == f2 && t1 == t2 && e1 == e2
     | Tlet (t1,bv1), Tlet (t2,bv2) -> t1 == t2 && bv1 == bv2
@@ -510,6 +498,36 @@ let t_hash trigger attr t =
     end in
   t_hash 0 [] t
 
+let t_hash_strict t = t_hash true true t
+let t_equal_strict t1 t2 = t_compare true true true t1 t2 = 0
+let t_compare_strict t1 t2 = t_compare true true true t1 t2
+
+module TermOHT_strict = struct
+  type t = term
+  let hash = t_hash_strict
+  let equal = t_equal_strict
+  let compare = t_compare_strict
+end
+
+module Mterm_strict = Extmap.Make(TermOHT_strict)
+module Sterm_strict = Extset.MakeOfMap(Mterm_strict)
+module Hterm_strict = Exthtbl.Make(TermOHT_strict)
+
+let t_hash t = t_hash false false t
+let t_equal t1 t2 = t_compare false false false t1 t2 = 0
+let t_compare t1 t2 = t_compare false false false t1 t2
+
+module TermOHT = struct
+  type t = term
+  let hash = t_hash
+  let equal = t_equal
+  let compare = t_compare
+end
+
+module Mterm = Extmap.Make(TermOHT)
+module Sterm = Extset.MakeOfMap(Mterm)
+module Hterm = Exthtbl.Make(TermOHT)
+
 (* type checking *)
 
 exception TermExpected of term
@@ -572,27 +590,6 @@ and add_t_vars s t = vars_union s (t_vars t)
 let add_nt_vars _ n t s = vars_union s
   (if n = 1 then t_vars t else Mvs.map (( * ) n) (t_vars t))
 
-module TermOHT = struct
-  type t = term
-  let hash = t_hash true true
-  let equal = t_equal
-  let compare = t_compare
-end
-
-module Mterm = Extmap.Make(TermOHT)
-module Sterm = Extset.MakeOfMap(Mterm)
-module Hterm = Exthtbl.Make(TermOHT)
-
-module TermOHT_nt_na = struct
-  type t = term
-  let hash = t_hash false false
-  let equal = t_equal_nt_na
-end
-
-module Hterm_nt_na = Exthtbl.Make(TermOHT_nt_na)
-
-let t_hash = t_hash true true
-
 (* hash-consing constructors for terms *)
 
 let mk_term n ty = {
@@ -828,6 +825,16 @@ let t_open_quant_cb fq =
   in
   vl, tl, f, close
 
+(* retrieve bound identifiers (useful to detect sharing) *)
+
+let t_peek_bound (v,_,_) = v.vs_name
+
+let t_peek_branch (p,_,_) =
+  Svs.fold (fun v a -> Sid.add v.vs_name a) p.pat_vars Sid.empty
+
+let t_peek_quant (vl,_,_,_) =
+  List.map (fun v -> v.vs_name) vl
+
 (* constructors with type checking *)
 
 let ls_arg_inst ls tl =
@@ -854,33 +861,37 @@ let ps_app ps tl    = t_app ps tl None
 
 let t_nat_const n =
   assert (n >= 0);
-  t_const (Number.const_of_int n) ty_int
+  t_const (Constant.int_const_of_int n) ty_int
 
-let t_bigint_const n = t_const (Number.const_of_big_int n) Ty.ty_int
+let t_int_const n =
+  t_const (Constant.int_const n) Ty.ty_int
+
+let t_real_const ?pow2 ?pow5 s =
+  t_const (Constant.real_const ?pow2 ?pow5 s) Ty.ty_real
+
+let t_string_const s =
+  t_const (Constant.string_const s) Ty.ty_str
 
 exception InvalidIntegerLiteralType of ty
 exception InvalidRealLiteralType of ty
+exception InvalidStringLiteralType of ty
 
 let check_literal c ty =
-  let ts = match ty.ty_node with
-    | Tyapp (ts,[]) -> ts
-    | _ -> match c with
-           | Number.ConstInt _ -> raise (InvalidIntegerLiteralType ty)
-           | Number.ConstReal _ -> raise (InvalidRealLiteralType ty)
-  in
-  match c with
-  | Number.ConstInt _ when ts_equal ts ts_int -> ()
-  | Number.ConstInt n ->
-     begin match ts.ts_def with
-           | Range ir -> Number.(check_range n ir)
-           | _ -> raise (InvalidIntegerLiteralType ty)
-     end
-  | Number.ConstReal _ when ts_equal ts ts_real -> ()
-  | Number.ConstReal x ->
-     begin match ts.ts_def with
-           | Float fp -> Number.(check_float x.Number.rc_abs fp)
-           | _ -> raise (InvalidRealLiteralType ty)
-     end
+  let open Constant in
+  let ts = match ty.ty_node, c with
+    | Tyapp (ts,[]), _ -> ts
+    | _, ConstInt _ -> raise (InvalidIntegerLiteralType ty)
+    | _, ConstReal _ -> raise (InvalidRealLiteralType ty)
+    | _, ConstStr _ -> raise (InvalidStringLiteralType ty) in
+  match c, ts.ts_def with
+  | ConstInt _, _ when ts_equal ts ts_int -> ()
+  | ConstInt n, Range ir -> Number.check_range n ir
+  | ConstInt _, _ -> raise (InvalidIntegerLiteralType ty)
+  | ConstReal _, _ when ts_equal ts ts_real -> ()
+  | ConstReal x, Float fp -> Number.check_float x fp
+  | ConstReal _, _ -> raise (InvalidRealLiteralType ty)
+  | ConstStr _, _ when ts_equal ts ts_str -> ()
+  | ConstStr _, _ -> raise (InvalidStringLiteralType ty)
 
 let t_const c ty = check_literal c ty; t_const c ty
 
