@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -73,6 +73,7 @@ exception PredicateSymbolExpected of lsymbol
 exception ConstructorExpected of lsymbol
 exception InvalidIntegerLiteralType of ty
 exception InvalidRealLiteralType of ty
+exception InvalidStringLiteralType of ty
 
 (** {2 Patterns} *)
 
@@ -125,7 +126,7 @@ type term = private {
 
 and term_node =
   | Tvar of vsymbol
-  | Tconst of Number.constant
+  | Tconst of Constant.constant
   | Tapp of lsymbol * term list
   | Tif of term * term * term
   | Tlet of term * term_bound
@@ -143,19 +144,26 @@ and term_quant
 
 and trigger = term list list
 
+(** {2 Term equality modulo alpha-equivalence} *)
+
+val t_equal_strict : term -> term -> bool
+val t_compare_strict : term -> term -> int
+val t_hash_strict : term -> int
+
+module Mterm_strict : Extmap.S with type key = term
+module Sterm_strict : Extset.S with module M = Mterm_strict
+module Hterm_strict : Exthtbl.S with type key = term
+
+(** {2 Term equality modulo alpha-equivalence, attributes, triggers, and locations} *)
+
+val t_equal : term -> term -> bool
+val t_compare : term -> term -> int
+val t_hash : term -> int
+
 module Mterm : Extmap.S with type key = term
 module Sterm : Extset.S with module M = Mterm
 module Hterm : Exthtbl.S with type key = term
 
-(** {2 Term equality modulo alpha-equivalence and location} *)
-
-val t_compare : term -> term -> int
-val t_equal : term -> term -> bool
-val t_hash : term -> int
-(* Equality modulo attributes and triggers *)
-val t_equal_nt_na : term -> term -> bool
-module Hterm_nt_na : Exthtbl.S with type key = term
-
 (** {2 Bindings} *)
 
 (** close bindings *)
@@ -188,6 +196,12 @@ val t_open_quant_cb :
   term_quant -> vsymbol list * trigger * term *
               (vsymbol list -> trigger -> term -> term_quant)
 
+(** retrieve bound identifiers (useful to detect sharing) *)
+
+val t_peek_bound : term_bound -> ident
+val t_peek_branch : term_branch -> Sid.t
+val t_peek_quant : term_quant -> ident list
+
 (** {2 Type checking} *)
 
 exception TermExpected of term
@@ -212,10 +226,10 @@ val t_app_infer : lsymbol -> term list -> term
 val ls_arg_inst : lsymbol -> term list -> ty Mtv.t
 val ls_app_inst : lsymbol -> term list -> ty option -> ty Mtv.t
 
-val check_literal : Number.constant -> ty -> unit
+val check_literal : Constant.constant -> ty -> unit
 
 val t_var : vsymbol -> term
-val t_const : Number.constant -> ty -> term
+val t_const : Constant.constant -> ty -> term
 val t_if : term -> term -> term -> term
 val t_let : term -> term_bound -> term
 val t_case : term -> term_branch list -> term
@@ -235,7 +249,9 @@ val t_false : term
 val t_nat_const : int -> term
 (** [t_nat_const n] builds the constant integer term [n],
     n must be non-negative *)
-val t_bigint_const : BigInt.t -> term
+val t_int_const : BigInt.t -> term
+val t_real_const : ?pow2:BigInt.t -> ?pow5:BigInt.t -> BigInt.t -> term
+val t_string_const : string -> term
 
 val stop_split : attribute
 val asym_split : attribute
