@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -220,6 +220,7 @@ type naming_table = {
     coercion : Coercion.t;
     printer : Ident.ident_printer;
     aprinter : Ident.ident_printer;
+    meta_id_args : Ident.ident Mstr.t;
  }
 (** In order to interpret, that is type, string arguments as symbols or
    terms, a transformation may need a [naming_table]. Typing arguments
@@ -236,8 +237,10 @@ type naming_table = {
 
 exception Bad_name_table of string
 
-type trans_with_args = string list -> Env.env -> naming_table -> task trans
-type trans_with_args_l = string list -> Env.env -> naming_table -> task tlist
+type trans_with_args =
+  string list -> Env.env -> naming_table -> Env.fformat -> task trans
+type trans_with_args_l =
+  string list -> Env.env -> naming_table -> Env.fformat -> task tlist
 
 val list_transforms_with_args   : unit -> (string * Pp.formatted) list
 val list_transforms_with_args_l : unit -> (string * Pp.formatted) list
@@ -261,8 +264,11 @@ val lookup_trans_desc: string -> Pp.formatted
 
 val list_trans : unit -> string list
 
+exception Unnecessary_arguments of string list
+
 val apply_transform : string -> Env.env -> task -> task list
 (** apply a registered 1-to-1 or a 1-to-n, directly.*)
 
-val apply_transform_args : string -> Env.env -> string list -> naming_table -> task -> task list
+val apply_transform_args :
+  string -> Env.env -> string list -> naming_table -> Env.fformat -> task -> task list
 (** apply a registered 1-to-1 or a 1-to-n or a trans with args, directly *)
