@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -77,12 +77,15 @@ let print_float fmt f =
 
 let print_value fmt v =
   match v with
+  | String s ->
+     Format.fprintf fmt "String: %a" Constant.print_string_def s
   | Integer s -> Format.fprintf fmt "Integer: %s" s
   | Decimal (s1, s2) -> Format.fprintf fmt "Decimal: %s . %s" s1 s2
   | Fraction (s1, s2) -> Format.fprintf fmt "Fraction: %s / %s" s1 s2
   | Float f -> Format.fprintf fmt "Float: %a" print_float f
   | Other s -> Format.fprintf fmt "Other: %s" s
-  | Bitvector bv -> Format.fprintf fmt "Bv: %s" bv
+  | Bitvector (Bv_int bv) -> Format.fprintf fmt "Bv: %s" bv
+  | Bitvector (Bv_sharp bv) -> Format.fprintf fmt "Bv: %s" bv
   | Boolean b -> Format.fprintf fmt "Boolean: %b " b
 
 let rec print_array fmt a =
@@ -106,7 +109,9 @@ and print_term fmt t =
   | TCvc4_Variable _ -> Format.fprintf fmt "CVC4VAR: TREE"
   | TFunction_Local_Variable v -> Format.fprintf fmt "LOCAL: %s" v
   | TVariable v -> Format.fprintf fmt "VAR: %s" v
-  | TIte _ -> Format.fprintf fmt "ITE"
+  | TIte (teq1, teq2, tthen, telse) ->
+      Format.fprintf fmt "ITE (%a = %a) then %a else %a"
+        print_term teq1 print_term teq2 print_term tthen print_term telse
   | TRecord (n, l) ->
       Format.fprintf fmt "record_type: %s; list_fields: %a" n
         (Pp.print_list Pp.semi
@@ -128,6 +133,33 @@ let rec print_tree fmt t =
 and print_mpt fmt t =
   Mpr.iter (fun key e -> Format.fprintf fmt "P: %s; T: %a" key print_tree e) t
 
+let subst_local_var var value t =
+  let rec aux t =
+    match t with
+    | TFunction_Local_Variable var' when var' = var ->
+        value
+    | TFunction_Local_Variable _ | TVariable _ | TCvc4_Variable _ | TSval _ ->
+        t
+    | TApply (s, args) ->
+        TApply (s, List.map aux args)
+    | TIte (t1, t2, t3, t4) ->
+        TIte (aux t1, aux t2, aux t3, aux t4)
+    | TArray tarray ->
+        TArray (aux_array tarray)
+    | TRecord (s, fields) ->
+        let aux_field (s, t) = (s, aux t) in
+        TRecord (s, List.map aux_field fields)
+    | TTo_array t ->
+        TTo_array (aux t)
+  and aux_array a =
+    match a with
+    | TArray_var _ ->
+        a
+    | TConst t ->
+        TConst (aux t)
+    | TStore (a, t1, t2) ->
+        TStore (aux_array a, aux t1, aux t2) in
+  aux t
 
 (* Printing function for debugging *)
 let print_table (t: correspondence_table) =
@@ -205,6 +237,72 @@ let remove_end_num s =
 (* Used to handle case of badly formed table *)
 exception Incorrect_table
 
+(* Simplify if-then-else in value so that it can be read by
+   add_vars_to_table. *)
+let rec simplify_value table v =
+  match v with
+  | TApply (s, args') ->
+      let vars, body = (* Function binding for s *)
+        match Mstr.find s table with
+        | Leaf (TFunction (vars, body)) -> vars, body
+        | _ -> raise Incorrect_table
+        | exception Not_found -> raise Incorrect_table in
+      let vars = List.map fst vars in
+      let args = List.map (simplify_value table) args' in
+      List.fold_right2 subst_local_var vars args body |>
+      simplify_value table
+  | TIte (
+      TIte (TFunction_Local_Variable x,
+            TCvc4_Variable cvc,
+            TCvc4_Variable cvc1,
+            _),
+      TCvc4_Variable cvc2,
+      tth,
+      tel) when cvc = cvc1 && cvc = cvc2 ->
+      (* Here we chose what we keep from the model. This case is not complete
+         but good enough. *)
+      let t = TIte (TFunction_Local_Variable x, TCvc4_Variable cvc, tth, tel) in
+      simplify_value table t
+  | TIte (
+      TIte (TCvc4_Variable cvc,
+            TFunction_Local_Variable x,
+            TCvc4_Variable cvc1,
+            _),
+      TCvc4_Variable cvc2,
+      tth,
+      tel) when cvc = cvc1 && cvc = cvc2 (* Same as above *) ->
+      (* Here we chose what we keep from the model. This case is not complete
+         but good enough. *)
+      let t = TIte (TFunction_Local_Variable x, TCvc4_Variable cvc, tth, tel) in
+      simplify_value table t
+  | TIte (
+      TIte (TFunction_Local_Variable x,
+            TCvc4_Variable cvc,
+            TCvc4_Variable cvc1,
+            TCvc4_Variable cvc3),
+      TCvc4_Variable cvc2,
+      tth,
+      tel) when cvc = cvc1 && cvc <> cvc2 && cvc3 = cvc2 ->
+      (* Here we chose what we keep from the model. This case is not complete
+         but good enough. *)
+      let t = TIte (TFunction_Local_Variable x, TCvc4_Variable cvc3, tth, tel) in
+      simplify_value table t
+  | TIte (
+      TIte (TCvc4_Variable cvc,
+            TFunction_Local_Variable x,
+            TCvc4_Variable cvc1,
+            TCvc4_Variable cvc3),
+      TCvc4_Variable cvc2,
+      tth,
+      tel) when cvc = cvc1 && cvc <> cvc2 && cvc3 = cvc2 (* Same as above *) ->
+      (* Here we chose what we keep from the model. This case is not complete
+         but good enough. *)
+      let t = TIte (TFunction_Local_Variable x, TCvc4_Variable cvc3, tth, tel) in
+      simplify_value table t
+  | TIte (eq1, eq2, tthen, telse) ->
+      TIte (eq1, eq2, simplify_value table tthen, simplify_value table telse)
+  | _ -> v
+
 (* Add the variables that can be deduced from ITE to the table of variables *)
 let add_vars_to_table (table: correspondence_table) key value : correspondence_table =
 
@@ -227,6 +325,7 @@ let add_vars_to_table (table: correspondence_table) key value : correspondence_t
           Mstr.add cvc (Node (Mpr.add key t1 Mpr.empty)) table
     in
 
+    let value = simplify_value table value in
     match value with
     | TIte (TCvc4_Variable (Var cvc), TFunction_Local_Variable _x, t1, t2) ->
         let table = add_var_ite cvc t1 table in
@@ -240,7 +339,7 @@ let add_vars_to_table (table: correspondence_table) key value : correspondence_t
     | TIte (TFunction_Local_Variable _x, t, TCvc4_Variable (Var cvc), t2) ->
         let table = add_var_ite cvc t table in
         add_vars_to_table ~type_value table t2
-    | TIte (_, _, _, _) -> table
+    | TIte _ -> table
     | _ ->
       begin
         match type_value with
@@ -249,8 +348,8 @@ let add_vars_to_table (table: correspondence_table) key value : correspondence_t
             Mstr.fold (fun key_val l_elt acc ->
               let match_str_z3 = type_value ^ "!" in
               let match_str_cvc4 = "_" ^ type_value ^ "_" in
-              let match_str = Str.regexp ("\\(" ^ match_str_z3 ^ "\\|" ^ match_str_cvc4 ^ "\\)") in
-              match Str.search_forward match_str (remove_end_num key_val) 0 with
+              let match_str = Re.Str.regexp ("\\(" ^ match_str_z3 ^ "\\|" ^ match_str_cvc4 ^ "\\)") in
+              match Re.Str.search_forward match_str (remove_end_num key_val) 0 with
               | exception Not_found -> acc
               | _ ->
                   if l_elt = Leaf TNoelement then
@@ -371,22 +470,23 @@ let refine_variable_value table key t =
   let encountered_key = Hstr.create 16 in
   refine_variable_value ~enc:encountered_key table key t
 
-(* Conversion to value referenced as defined in model_parser.
-   We assume that array indices fit into an integer *)
-let convert_to_indice t =
-  match t with
-  | Sval (Integer i)    -> i
-  | Sval (Bitvector bv) -> bv
-  | Sval (Boolean b)    -> string_of_bool b
-  | _ -> raise Not_value
-
 let convert_simple_to_model_value (v: simple_value) =
   match v with
+  | String s -> Model_parser.String s
   | Integer i -> Model_parser.Integer i
   | Decimal (d1, d2) -> Model_parser.Decimal (d1, d2)
   | Fraction (s1, s2) -> Model_parser.Fraction (s1, s2)
   | Float f -> Model_parser.Float f
-  | Bitvector bv -> Model_parser.Bitvector bv
+  | Bitvector (Bv_int bv) -> Model_parser.Integer bv
+  | Bitvector (Bv_sharp bv) ->
+      (* Convert "#x0032" to "0x0032" then to "50" for uniformity with Bv_int *)
+      begin try
+        let a = ref 0 in
+        let bv = "0" ^ String.sub bv 1 (String.length bv - 1) in
+        Scanf.sscanf bv "%i" (fun x -> a := x);
+        let bv = Format.asprintf "%d" !a in
+        Model_parser.Integer bv
+      with _ -> Model_parser.Bitvector bv end
   | Boolean b -> Model_parser.Boolean b
   | Other _s -> raise Not_value
 
@@ -403,7 +503,7 @@ let rec convert_array_value lf (a: array) : Model_parser.model_array =
                    Model_parser.arr_others = convert_to_model_value lf t}
     | Store (a, t1, t2) ->
         let new_index =
-          { Model_parser.arr_index_key = convert_to_indice t1;
+          { Model_parser.arr_index_key = convert_to_model_value lf t1;
             Model_parser.arr_index_value = convert_to_model_value lf t2} in
         array_indices := new_index :: !array_indices;
         create_array_value a in
@@ -680,8 +780,10 @@ let create_list (projections_list: Ident.ident Mstr.t)
   Mstr.fold
     (fun key value list_acc ->
       try (convert_to_model_element ~set_str field_list key value :: list_acc)
-      with Not_value when not (Debug.test_flag Debug.stack_trace) ->
-        Debug.dprintf debug_cntex "Element creation failed: %s@." key; list_acc
+      with Not_value when not (Debug.test_flag debug_cntex &&
+                               Debug.test_flag Debug.stack_trace) ->
+        Debug.dprintf debug_cntex "Element creation failed: %s@." key;
+        list_acc
       | e -> raise e)
     table
     []
