@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -69,12 +69,12 @@ let do_parsing list_proj list_fields list_records noarg_constructors set_str mod
 let parse : raw_model_parser =
   fun list_proj list_fields list_records noarg_constructors set_str input ->
   try
-(*    let r = Str.regexp "unknown\\|sat\\|\\(I don't know.*\\)" in
-    ignore (Str.search_forward r input 0);
-    let match_end = Str.match_end () in*)
-    let nr = Str.regexp "^)+" in
-    let res = Str.search_backward nr input (String.length input) in
-    let model_string = String.sub input 0 (res + String.length (Str.matched_string input)) in
+(*    let r = Re.Str.regexp "unknown\\|sat\\|\\(I don't know.*\\)" in
+    ignore (Re.Str.search_forward r input 0);
+    let match_end = Re.Str.match_end () in*)
+    let nr = Re.Str.regexp "^)+" in
+    let res = Re.Str.search_backward nr input (String.length input) in
+    let model_string = String.sub input 0 (res + String.length (Re.Str.matched_string input)) in
     do_parsing list_proj list_fields list_records noarg_constructors set_str model_string
   with
   | Not_found -> []
