@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -93,12 +93,12 @@ module PE = struct
 
 end
 
-let readBody (xhr: XmlHttpRequest.xmlHttpRequest Js.t) =
+let readBody (xhr: XmlHttpRequest.xmlHttpRequest Js.t) : string =
   let data = ref None in
   data := Some (xhr ##. responseText);
   match !data with
   | None -> raise Not_found
-  | Some data -> Js.to_string data
+  | Some data -> Js.Opt.case data (fun () -> "") (fun x -> Js.to_string x)
 
 module Tabs =
   struct
@@ -553,13 +553,15 @@ let interpNotif (n: notification) =
       TaskList.attach_new_node nid parent ntype name detached;
       TaskList.onclick_do_something (string_of_int nid);
       sendRequest (Get_task (string_of_int nid))
-  | File_contents (_f,_s) ->
+  | File_contents (_f,_s,_, _) ->
      PE.error_print_msg "Notification File_contents not handled yet"
   | Source_and_ce _ ->
      PE.error_print_msg "Notification Source_and_ce not handled yet"
+  | Ident_notif_loc _ ->
+     PE.error_print_msg "Notification Ident_notif_loc not handled yet"
   | Next_Unproven_Node_Id (_nid1,_nid2) ->
      PE.error_print_msg "Notification Next_Unproven_Node_Id not handled yet"
-  | Task (nid, task, _list_loc) -> (* TODO add color on sources *)
+  | Task (nid, task, _list_loc, _goal_loc, _lang) -> (* TODO add color on sources *)
       Hashtbl.add TaskList.printed_task_list (string_of_int nid) task
   | Remove nid ->
       TaskList.remove_node (string_of_int nid)
