@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -26,6 +26,7 @@ val dity_of_ity : ity -> dity
 
 val dity_int  : dity
 val dity_real : dity
+val dity_str  : dity
 val dity_bool : dity
 val dity_unit : dity
 
@@ -55,6 +56,10 @@ type ghost = bool
 
 type dbinder = preid option * ghost * dity
 
+val dummy_var_attr : Ident.attribute
+(* pvsymbols created from dbinders with a None ident are dummy variables,
+   they are decorated with this attribute. *)
+
 (** Specifications *)
 
 exception UnboundLabel of string
@@ -104,7 +109,7 @@ type dexpr = private {
 and dexpr_node =
   | DEvar of string * dvty * dref
   | DEsym of prog_symbol
-  | DEconst of Number.constant * dity
+  | DEconst of Constant.constant * dity
   | DEapp of dexpr * dexpr
   | DEfun of dbinder list * dity * mask * dspec later * dexpr
   | DEany of dbinder list * dity * mask * dspec later
