@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -319,7 +319,7 @@ type expr = {
 
 and expr_node =
   | Evar    of pvsymbol
-  | Econst  of Number.constant
+  | Econst  of Constant.constant
   | Eexec   of cexp * cty
   | Eassign of assign list
   | Elet    of let_defn * expr
@@ -458,7 +458,9 @@ let localize_divergence el =
     Loc.error ?loc GhostDivergence) el;
   raise GhostDivergence
 
-let try_effect el fn x y = try fn x y with
+let try_effect el fn x y =
+  if Debug.test_flag Debug.stack_trace then fn x y else
+  try fn x y with
   | BadGhostWrite (v,r) -> localize_ghost_write v r el
   | IllegalUpdate (v,r) -> localize_immut_write v r el
   | StaleVariable (v,r) -> localize_reset_stale v r el
@@ -490,7 +492,7 @@ let e_const c ity =
 
 let e_nat_const n =
   assert (n >= 0);
-  e_const (Number.const_of_int n) ity_int
+  e_const (Constant.int_const_of_int n) ity_int
 
 let e_ghostify gh ({e_effect = eff} as e) =
   if not gh then e else
@@ -631,10 +633,12 @@ let rec post_of_expr res e = match e.e_node with
   | _ when ity_equal e.e_ity ity_unit -> t_true
   | Eassign _ | Ewhile _ | Efor _ | Eassert _ -> assert false
   | Evar v -> post_of_term res (t_var v.pv_vs)
-  | Econst (Number.ConstInt _ as c)->
+  | Econst (Constant.ConstInt _ as c)->
       post_of_term res (t_const c ty_int)
-  | Econst (Number.ConstReal _ as c)->
+  | Econst (Constant.ConstReal _ as c)->
       post_of_term res (t_const c ty_real)
+  | Econst (Constant.ConstStr _ as c) ->
+      post_of_term res (t_const c ty_str)
   | Epure t -> post_of_term res t
   | Eghost e | Eexn (_,e) -> post_of_expr res e
   | Eexec (_,c) ->
@@ -757,6 +761,7 @@ let e_exec c =
           eff_catch eff xs) x_lost e.e_effect in
         let eff = if cty.cty_effect.eff_ghost then
           try_effect [e] eff_ghostify true eff else eff in
+        let eff = eff_bind (Mpv.domain cty.cty_oldies) eff in
         eff_union_par cty.cty_effect eff
     | _ -> cty.cty_effect in
   mk_expr (Eexec (c, cty)) cty.cty_result cty.cty_mask eff
@@ -1378,7 +1383,7 @@ and print_cexp exec pri fmt {c_node = n; c_cty = c} = match n with
 
 and print_enode pri fmt e = match e.e_node with
   | Evar v -> print_pv fmt v
-  | Econst c -> Number.print_constant fmt c
+  | Econst c -> Constant.print_def fmt c
   | Eexec (c,_) -> print_cexp true pri fmt c
   | Elet (LDvar (v,e1), e2)
     when v.pv_vs.vs_name.id_string = "_" && ity_equal v.pv_ity ity_unit ->
