@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -77,7 +77,7 @@ let create_semi_constructor id s fdl pjl invl =
 
 let create_plain_record_decl ~priv ~mut id args fdl invl witn =
   let exn = Invalid_argument "Pdecl.create_plain_record_decl" in
-  let cid = id_fresh ?loc:id.pre_loc ("mk " ^ id.pre_name) in
+  let cid = id_fresh ?loc:id.pre_loc (id.pre_name ^ "'mk") in
   let add_fd fds (mut, fd) = Mpv.add_new exn fd mut fds in
   let fds = List.fold_left add_fd Mpv.empty fdl in
   let fdl = List.map snd fdl in
@@ -103,7 +103,7 @@ let create_rec_record_decl s fdl =
   let exn = Invalid_argument "Pdecl.create_rec_record_decl" in
   if not (its_recursive s) then raise exn;
   let id = s.its_ts.ts_name in
-  let cid = id_fresh ?loc:id.id_loc ("mk " ^ id.id_string) in
+  let cid = id_fresh ?loc:id.id_loc (id.id_string ^ "'mk") in
   List.iter (check_field (Stv.of_list s.its_ts.ts_args)) fdl;
   let cs = create_constructor ~constr:1 cid s fdl in
   let pjl = List.map (create_projection s) fdl in
@@ -187,7 +187,7 @@ let get_syms node pure =
   let syms_ty syms ty = ty_s_fold syms_ts syms ty in
   let syms_term syms t = t_s_fold syms_ty syms_ls syms t in
   let syms_tl syms tl = List.fold_left syms_term syms tl in
-  let syms_pure syms d = Sid.union syms d.d_syms in
+  let syms_pure syms d = Sid.union syms (get_decl_syms d) in
   let syms_vari syms (t,r) = Opt.fold syms_ls (syms_term syms t) r in
   let syms_varl syms varl = List.fold_left syms_vari syms varl in
   let syms = List.fold_left syms_pure Sid.empty pure in
@@ -343,41 +343,57 @@ let create_type_decl dl =
     | _, _, Alias _ ->
         mk_decl (PDtype [itd]) [create_ty_decl ts]
     | _, _, Range ir ->
-        let pj_id = id_derive (nm ^ "'int") id in
+        let pj_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'int" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'int") in
         let pj_ls = create_fsymbol pj_id [ty_app ts []] ty_int in
         let pj_decl = create_param_decl pj_ls in
         (* Create the projection meta for pj_decl *)
         let meta_proj_pj = (Theory.meta_projection, [Theory.MAls pj_ls]) in
         (* create max attribute *)
-        let max_id = id_derive (nm ^ "'maxInt") id in
+        let max_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'maxInt" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'maxInt") in
         let max_ls = create_fsymbol max_id [] ty_int  in
-        let max_defn = t_const Number.(const_of_big_int ir.ir_upper) ty_int in
+        let const = Constant.int_const ir.Number.ir_upper in
+        let max_defn = t_const const ty_int in
         let max_decl = create_logic_decl [make_ls_defn max_ls [] max_defn] in
         (* create min attribute *)
-        let min_id = id_derive (nm ^ "'minInt") id in
+        let min_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'minInt" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'minInt") in
         let min_ls = create_fsymbol min_id [] ty_int  in
-        let min_defn = t_const Number.(const_of_big_int ir.ir_lower) ty_int in
+        let const = Constant.int_const ir.Number.ir_lower in
+        let min_defn = t_const const ty_int in
         let min_decl = create_logic_decl [make_ls_defn min_ls [] min_defn] in
         let pure = [create_ty_decl ts; pj_decl; max_decl; min_decl] in
         let meta = Theory.(meta_range, [MAts ts; MAls pj_ls]) in
         mk_decl_meta [meta; meta_proj_pj] (PDtype [itd]) pure
     | _, _, Float ff ->
-        let pj_id = id_derive (nm ^ "'real") id in
+        let pj_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'real" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'real") in
         let pj_ls = create_fsymbol pj_id [ty_app ts []] ty_real in
         let pj_decl = create_param_decl pj_ls in
         (* Create the projection meta for pj_decl *)
         let meta_proj_pj = (Theory.meta_projection, [Theory.MAls pj_ls]) in
         (* create finiteness predicate *)
-        let iF_id = id_derive (nm ^ "'isFinite") id in
+        let iF_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'isFinite" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'isFinite") in
         let iF_ls = create_psymbol iF_id [ty_app ts []] in
         let iF_decl = create_param_decl iF_ls in
         (* create exponent digits attribute *)
-        let eb_id = id_derive (nm ^ "'eb") id in
+        let eb_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'eb" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'eb") in
         let eb_ls = create_fsymbol eb_id [] ty_int in
         let eb_defn = t_nat_const ff.Number.fp_exponent_digits in
         let eb_decl = create_logic_decl [make_ls_defn eb_ls [] eb_defn] in
         (* create significand digits attribute *)
-        let sb_id = id_derive (nm ^ "'sb") id in
+        let sb_id =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'sb" in
+          id_fresh ~attrs ?loc:id.id_loc (nm ^ "'sb") in
         let sb_ls = create_fsymbol sb_id [] ty_int  in
         let sb_defn = t_nat_const ff.Number.fp_significand_digits in
         let sb_decl = create_logic_decl [make_ls_defn sb_ls [] sb_defn] in
@@ -386,7 +402,9 @@ let create_type_decl dl =
         mk_decl_meta [meta; meta_proj_pj] (PDtype [itd]) pure
     | fl, _, NoDef when itd.itd_invariant <> [] ->
         let inv = axiom_of_invariant itd in
-        let pr = create_prsymbol (id_derive (nm ^ "'invariant") id) in
+        let pr =
+          let attrs = suffix_attr_name ~attrs:id.id_attrs "'invariant" in
+          create_prsymbol (id_fresh ~attrs ?loc:id.id_loc (nm ^ "'invariant")) in
         let ax = create_prop_decl Paxiom pr inv in
         let add_fd s dl = create_param_decl (ls_of_rs s) :: dl in
         let pure = create_ty_decl ts :: List.fold_right add_fd fl [ax] in
@@ -496,13 +514,13 @@ let create_let_decl ld =
         let vl = List.map (fun v -> v.pv_vs) cty.cty_args in
         let hd = t_app ls (List.map t_var vl) ls.ls_value in
         let f = t_and_simp_l (conv_post hd cty.cty_post) in
-        let nm = id.id_string ^ "_spec" in
+        let nm = id.id_string ^ "'spec" in
         let axms = cty_axiom (id_derive ~attrs nm id) cty f axms in
         let c = if Mrs.is_empty sm then c else c_rs_subst sm c in
         begin match c.c_node with
         | Cany | Capp _ | Cpur _ ->
             (* the full spec of c is inherited by the rsymbol and
-               so appears in the "_spec" axiom above. Even if this
+               so appears in the "'spec" axiom above. Even if this
                spec contains "result = ...", we do not try to extract
                a definition from it. We only produce definitions via
                term_of_expr from a Cfun, and the user must eta-expand
@@ -513,13 +531,13 @@ let create_let_decl ld =
             | Some f when cty.cty_pre = [] ->
                 abst, (ls, vl, f) :: defn, axms
             | Some f ->
-                let f = t_insert hd f and nm = id.id_string ^ "_def" in
+                let f = t_insert hd f and nm = id.id_string ^ "'def" in
                 let axms = cty_axiom (id_derive ~attrs nm id) cty f axms in
                 create_param_decl ls :: abst, defn, axms
             | None when cty.cty_post = [] ->
                 let axms = match post_of_expr hd e with
                   | Some f ->
-                      let nm = id.id_string ^ "_def" in
+                      let nm = id.id_string ^ "'def" in
                       cty_axiom (id_derive ~attrs nm id) cty f axms
                   | None -> axms in
                 create_param_decl ls :: abst, defn, axms
@@ -544,7 +562,7 @@ let create_let_decl ld =
     try [create_logic_decl dl] with Decl.NoTerminationProof _ ->
     let abst = List.map (fun (s,_) -> create_param_decl s) dl in
     let mk_ax ({ls_name = id} as s, vl, t) =
-      let nm = id.id_string ^ "_def" in
+      let nm = id.id_string ^ "'def" in
       let pr = create_prsymbol (id_derive ~attrs nm id) in
       let hd = t_app s (List.map t_var vl) t.t_ty in
       let ax = t_forall_close vl [] (t_insert hd t) in
@@ -571,10 +589,12 @@ open Theory
    Therefore we match the exact contents of th_decls, and crash if it
    is not what we expect. *)
 
-let pd_int, pd_real, pd_equ = match builtin_theory.th_decls with
-  | [{td_node = Decl di}; {td_node = Decl dr}; {td_node = Decl de}] ->
+let pd_int, pd_real, pd_str, pd_equ = match builtin_theory.th_decls with
+  | [{td_node = Decl di}; {td_node = Decl dr};
+     {td_node = Decl ds}; {td_node = Decl de}] ->
       mk_decl (PDtype [mk_itd its_int  [] [] [] []]) [di],
       mk_decl (PDtype [mk_itd its_real [] [] [] []]) [dr],
+      mk_decl (PDtype [mk_itd its_str  [] [] [] []]) [ds],
       mk_decl PDpure [de]
   | _ -> assert false
 
@@ -669,7 +689,7 @@ let print_its_defn fst fmt itd =
         (if s.its_mutable && s.its_mfields = [] then " mutable" else "")
         (Pp.print_list Pp.semi print_field) fl
     | NoDef, fl, [{rs_name = {id_string = n}}]
-      when n = "mk " ^ s.its_ts.ts_name.id_string -> fprintf fmt " =%s { %a }"
+      when n = s.its_ts.ts_name.id_string ^ "'mk" -> fprintf fmt " =%s { %a }"
         (if s.its_mutable && s.its_mfields = [] then " mutable" else "")
         (Pp.print_list Pp.semi print_field) fl
     | NoDef, fl, cl ->
