@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -287,19 +287,27 @@ let add_meta uc m al = { uc with
   muc_theory = Theory.add_meta uc.muc_theory m al;
   muc_units  = Umeta (m,al) :: uc.muc_units; }
 
-let store_path, store_module, restore_path =
+let store_path, store_module, restore_path, restore_module_id =
   let id_to_path = Wid.create 17 in
+  let id_to_pmod = Wid.create 17 in
   let store_path {muc_theory = uc} path id =
     (* this symbol already belongs to some theory *)
     if Wid.mem id_to_path id then () else
     let prefix = List.rev (id.id_string :: path @ uc.uc_prefix) in
     Wid.set id_to_path id (uc.uc_path, uc.uc_name.id_string, prefix) in
-  let store_module {mod_theory = {th_name = id} as th} =
+  let store_module pmod  =
+    let th = pmod.mod_theory in
+    let id = th.th_name in
     (* this symbol is already a module *)
-    if Wid.mem id_to_path id then () else
-    Wid.set id_to_path id (th.th_path, id.id_string, []) in
+    if Wid.mem id_to_path id then () else begin
+        Wid.set id_to_path id (th.th_path, id.id_string, []);
+        Sid.iter (fun id -> Wid.set id_to_pmod id pmod) pmod.mod_local;
+        Wid.set id_to_pmod id pmod;
+      end
+  in
   let restore_path id = Wid.find id_to_path id in
-  store_path, store_module, restore_path
+  let restore_module_id id = Wid.find id_to_pmod id in
+  store_path, store_module, restore_path, restore_module_id
 
 let close_module uc =
   let m = close_module uc in
@@ -375,6 +383,7 @@ let builtin_module =
   let uc = empty_module dummy_env (id_fresh "BuiltIn") ["why3";"BuiltIn"] in
   let uc = add_pdecl_no_logic uc pd_int in
   let uc = add_pdecl_no_logic uc pd_real in
+  let uc = add_pdecl_no_logic uc pd_str in
   let uc = add_pdecl_no_logic uc pd_equ in
   let m = close_module uc in
   { m with mod_theory = builtin_theory }
@@ -487,8 +496,6 @@ let empty_clones m = {
   xs_table = Mxs.empty;
 }
 
-exception CloneDivergence of ident * ident
-
 (* populate the clone structure *)
 
 let rec sm_trans_ty tym tsm ty = match ty.ty_node with
@@ -516,45 +523,47 @@ let rec clone_ity cl ity = match ity.ity_node with
   | Ityreg r ->
       ity_reg (clone_reg cl r)
   | Ityapp (s, tl, rl) ->
-      let tl = List.map (clone_ity cl) tl in
-      let rl = List.map (clone_ity cl) rl in
+      let tl' = List.map (clone_ity cl) tl in
       begin match Mts.find_opt s.its_ts cl.ts_table with
-      | Some its -> ity_app_pure its tl rl
+      | Some s' -> ity_app_pure s' tl' (clone_regs cl s tl rl s')
       | None -> (* creative indentation *)
       begin match Mts.find_opt s.its_ts cl.ty_table with
-      | Some ity -> ity_full_inst (its_match_regs s tl rl) ity
-      | None -> ity_app_pure s tl rl
+      | Some ity -> ity_full_inst (its_match_regs s tl' []) ity
+      | None -> ity_app_pure s tl' (List.map (clone_ity cl) rl)
       end end
   | Ityvar _ -> ity
 
 and clone_reg cl reg =
-  (* FIXME: what about top-level non-instantiated regions?
-     We cannot check in cl.cl_local to see if they are there.
-     Instead, we should prefill cl.pv_table and cl.rn_table
-     with all top-level pvsymbols (local or external) before
-     descending into a let_defn.
-     TODO: add to module/uc a list of locally-defined toplevel
-     variables, as well as a set of imported toplevel variables. *)
   try Mreg.find reg cl.rn_table with Not_found ->
-  let tl = List.map (clone_ity cl) reg.reg_args in
-  let rl = List.map (clone_ity cl) reg.reg_regs in
-  let r = match Mts.find_opt reg.reg_its.its_ts cl.ts_table with
-    | Some its ->
-        create_region (id_clone reg.reg_name) its tl rl
-    | None -> (* creative indentation *)
-    begin match Mts.find_opt reg.reg_its.its_ts cl.ty_table with
-    | Some {ity_node = Ityreg r} ->
-        let sbs = its_match_regs reg.reg_its tl rl in
-        let tl = List.map (ity_full_inst sbs) r.reg_args in
-        let rl = List.map (ity_full_inst sbs) r.reg_regs in
-        create_region (id_clone reg.reg_name) r.reg_its tl rl
-    | Some _ -> assert false
-    | None ->
-        create_region (id_clone reg.reg_name) reg.reg_its tl rl
-    end in
+  let tl' = List.map (clone_ity cl) reg.reg_args in
+  let s',rl' = match Mts.find_opt reg.reg_its.its_ts cl.ts_table with
+    | Some s' -> s', clone_regs cl reg.reg_its reg.reg_args reg.reg_regs s'
+    | None -> reg.reg_its, List.map (clone_ity cl) reg.reg_regs in
+  let r = create_region (id_clone reg.reg_name) s' tl' rl' in
   cl.rn_table <- Mreg.add reg r cl.rn_table;
   r
 
+and clone_regs cl s tl rl s' =
+  if rl = [] then [] else
+  let base = its_match_regs s tl rl in
+  let add sbs pj =
+    let ity = clone_ity cl (ity_full_inst base pj.pv_ity) in
+    ity_match sbs (Mpv.find pj cl.fd_table).pv_ity ity in
+  let sbs = List.fold_left add isb_empty s.its_mfields in
+  let sbs = List.fold_left add sbs s.its_ofields in
+  let rec inst ity =
+    if ity.ity_pure then ity_full_inst sbs ity else
+    match ity.ity_node with
+    | Ityreg r when Mreg.mem r sbs.isb_reg ->
+        Mreg.find r sbs.isb_reg
+    | Ityreg {reg_its = s; reg_args = tl; reg_regs = rl} ->
+        ity_app s (List.map inst tl) (List.map inst rl)
+    | Ityapp (s,tl,rl) ->
+        ity_app_pure s (List.map inst tl) (List.map inst rl)
+    | Ityvar v ->
+        Mtv.find v sbs.isb_var in
+  List.map (fun reg -> inst (ity_reg reg)) s'.its_regions
+
 let cl_find_ls cl ls =
   if not (Sid.mem ls.ls_name cl.cl_local) then ls
   else Mls.find ls cl.ls_table
@@ -587,13 +596,14 @@ let cl_init m inst =
   Mts.iter (fun ts _ -> non_local ts.ts_name) inst.mi_ty;
   let check_ls ls _ =
     non_local ls.ls_name;
-    try  ignore (restore_rs ls); raise (BadInstance ls.ls_name)
+    try ignore (restore_rs ls);
+        raise (BadInstance (BadI_ls_rs ls))
     with Not_found -> () in
   Mls.iter check_ls inst.mi_ls;
   let check_rs rs _ =
     non_local rs.rs_name;
     match (Mid.find rs.rs_name m.mod_known).pd_node with
-    | PDtype _ -> raise (BadInstance rs.rs_name)
+    | PDtype _ -> raise (CannotInstantiate rs.rs_name)
     | PDlet  _ | PDexn  _ | PDpure -> () in
   Mrs.iter check_rs inst.mi_rs;
   Mvs.iter (fun vs _ -> non_local vs.vs_name) inst.mi_pv;
@@ -601,10 +611,12 @@ let cl_init m inst =
   let check_pk pr _ =
     non_local pr.pr_name;
     match (Mid.find pr.pr_name m.mod_known).pd_node with
-    | PDtype _ | PDlet  _ | PDexn  _ -> raise (BadInstance pr.pr_name)
+    | PDtype _ | PDlet  _ | PDexn  _ ->
+        raise (CannotInstantiate pr.pr_name)
     | PDpure -> () in
   Mpr.iter check_pk inst.mi_pk;
-  Mpr.iter (fun pr _ -> raise (BadInstance pr.pr_name)) inst.mi_pr;
+  Mpr.iter (fun pr _ -> (* only through bad API use *)
+    raise (BadInstance (BadI pr.pr_name))) inst.mi_pr;
   cl
 
 (* clone declarations *)
@@ -620,16 +632,17 @@ let clone_ls cl ls =
 
 let clone_decl inst cl uc d = match d.d_node with
   | Dtype _ | Ddata _ -> assert false (* impossible *)
-  | Dparam ({ls_name = id} as ls) when Mls.mem ls inst.mi_ls ->
+  | Dparam ls when Mls.mem ls inst.mi_ls ->
       let ls' = Mls.find ls inst.mi_ls in
-      let mtch sb ty ty' = try ty_match sb ty' (clone_ty cl ty)
-        with TypeMismatch _ -> raise (BadInstance id) in
+      let mtch sb ty ty' = try ty_match sb ty' (clone_ty cl ty) with
+        | Ty.TypeMismatch _ -> raise (BadInstance (BadI_ls_type ls)) in
       let sbs = match ls.ls_value,ls'.ls_value with
         | Some ty, Some ty' -> mtch Mtv.empty ty ty'
         | None, None -> Mtv.empty
-        | _ -> raise (BadInstance id) in
+        | _ -> raise (BadInstance (BadI_ls_kind ls)) in
       ignore (try List.fold_left2 mtch sbs ls.ls_args ls'.ls_args
-        with Invalid_argument _ -> raise (BadInstance id));
+        with Invalid_argument _ ->
+          raise (BadInstance (BadI_ls_arity ls)));
       cl.ls_table <- Mls.add ls ls' cl.ls_table;
       uc
   | Dparam ls ->
@@ -663,7 +676,8 @@ let clone_decl inst cl uc d = match d.d_node with
         | Paxiom, Some k -> false, k
         | Paxiom, None -> false, inst.mi_df in
       if skip then uc else
-      let pr' = create_prsymbol (id_clone pr.pr_name) in
+      let attr = Sattr.remove Ident.useraxiom_attr pr.pr_name.id_attrs in
+      let pr' = create_prsymbol (id_attr pr.pr_name attr) in
       cl.pr_table <- Mpr.add pr pr' cl.pr_table;
       let d = create_prop_decl k' pr' (clone_fmla cl f) in
       add_pdecl ~warn:false ~vc:false uc (create_pure_decl d)
@@ -676,13 +690,13 @@ let cl_save_rs cl s s' =
   begin match s.rs_field, s'.rs_field with
   | Some v, Some v' -> cl.fd_table <- Mpv.add v v' cl.fd_table
   | None, _ -> () (* can instantiate a non-field with a field *)
-  | _ -> assert false (* but not vice versa *)
+  | _ -> raise (CannotInstantiate s.rs_name) (* but not vice versa *)
   end;
   match s.rs_logic, s'.rs_logic with
   | RLls s, RLls s' -> cl_save_ls cl s s'
-  | RLlemma, RLlemma -> () (* TODO: update cl.pr_table? *)
-  | RLnone, RLnone -> ()
-  | _ -> assert false
+  | RLnone, (RLnone | RLls _ | RLlemma) -> ()
+  | RLlemma, RLlemma -> ()
+  | _ -> raise (BadInstance (BadI_rs_kind s.rs_name))
 
 type smap = {
   sm_vs : vsymbol Mvs.t;
@@ -753,24 +767,40 @@ let clone_cty cl sm ?(drop_decr=false) cty =
   let reads = Spv.fold add_read (cty_reads cty) Spv.empty in
   let reads = List.fold_right add_read cty.cty_args reads in
   let reads = Spv.union reads (Mpv.domain olds) in
-  let add_write reg fs m = (* add new mutable fields to functions effect *)
+  let add_frz v frz = ity_frz_regs frz v.pv_ity in
+  let frz = Spv.fold add_frz reads Sreg.empty in
+  let add_write reg fs m =
     let add_fd fd s = Spv.add (Mpv.find_def fd fd cl.fd_table) s in
-    let reg' = clone_reg cl reg in
-    let smf_reg' = Spv.of_list reg'.reg_its.its_mfields in
+    (* add old mutable fields to the effect *)
+    let fs = Spv.fold add_fd fs Spv.empty in
+    (* add new mutable fields to the effect *)
+    let ({reg_its = s'} as reg') = clone_reg cl reg in
+    let smf_reg' = Spv.of_list s'.its_mfields in
     let smf_reg = Spv.of_list reg.reg_its.its_mfields in
     let smf_ref = Spv.fold add_fd smf_reg Spv.empty in
-    let smf_new = Spv.diff smf_reg' smf_ref in
-    let fs = Spv.fold add_fd fs Spv.empty in
-    Mreg.add reg' (Spv.union fs smf_new) m in
+    let fs = Spv.union fs (Spv.diff smf_reg' smf_ref) in
+    let m = if Spv.is_empty fs && not s'.its_private
+            then m else Mreg.add reg' fs m in
+    (* add new non-mutable fields to the effect *)
+    let sof_reg' = Spv.of_list s'.its_ofields in
+    let sof_reg = Spv.of_list reg.reg_its.its_ofields in
+    let sof_ref = Spv.fold add_fd sof_reg Spv.empty in
+    let sbs = its_match_regs s' reg'.reg_args reg'.reg_regs in
+    let add_reg m r = if Sreg.mem r frz then m else
+      Mreg.add r (Spv.of_list r.reg_its.its_mfields) m in
+    let add_fd f m = ity_exp_fold add_reg m (ity_full_inst sbs f.pv_ity) in
+    Spv.fold add_fd (Spv.diff sof_reg' sof_ref) m in
   let writes = Mreg.fold add_write cty.cty_effect.eff_writes Mreg.empty in
   let add_reset reg s = Sreg.add (clone_reg cl reg) s in
   let resets = Sreg.fold add_reset cty.cty_effect.eff_resets Sreg.empty in
-  let eff = eff_reset (eff_write reads writes) resets in
+  let eff = eff_reset (eff_reset_overwritten (eff_write reads writes)) resets in
+  let add_spoil tv eff = eff_spoil eff (ity_var tv) in
+  let eff = Stv.fold add_spoil cty.cty_effect.eff_spoils eff in
   let add_raise xs eff = eff_raise eff (sm_find_xs sm xs) in
   let eff = Sxs.fold add_raise cty.cty_effect.eff_raises eff in
   let eff = if partial cty.cty_effect.eff_oneway then eff_partial eff else eff in
   let eff = if diverges cty.cty_effect.eff_oneway then eff_diverge eff else eff in
-  let cty = create_cty ~mask:cty.cty_mask args pre post xpost olds eff res in
+  let cty = create_cty_defensive ~mask:cty.cty_mask args pre post xpost olds eff res in
   cty_ghostify (cty_ghost cty) cty
 
 let sm_save_args sm c c' =
@@ -896,28 +926,62 @@ and clone_let_defn cl sm ld = match ld with
       sm, ld
 
 let clone_type_record cl s d s' d' =
-  let id = s.its_ts.ts_name in
   let fields' = Hstr.create 16 in
+  let matched_fields' = Hpv.create 16 in
   let add_field' rs' = let pj' = fd_of_rs rs' in
     Hstr.add fields' pj'.pv_vs.vs_name.id_string rs' in
   List.iter add_field' d'.itd_fields;
-  (* check if fields from former type are also declared in the new type *)
-  let match_pj rs = let pj = fd_of_rs rs in
+  (* refinement preserves (im)mutability *)
+  if not s.its_mutable && s'.its_mutable then
+    raise (BadInstance (BadI_ty_mut_rhs s.its_ts));
+  if s.its_mutable && not s'.its_mutable then
+    raise (BadInstance (BadI_ty_mut_lhs s.its_ts));
+  (* the fields from the old type must appear in the new type *)
+  let match_pj (bsb,fsb) rs =
+    let pj = fd_of_rs rs in
     let pj_str = pj.pv_vs.vs_name.id_string in
     let pj_ity = clone_ity cl pj.pv_ity in
     let pj_ght = pj.pv_ghost in
+    let pj_mut = List.exists (pv_equal pj) s.its_mfields in
     let rs' = try Hstr.find fields' pj_str with Not_found ->
-      raise (BadInstance id) in
+        raise (BadInstance (BadI_field (s.its_ts, pj.pv_vs))) in
     let pj' = fd_of_rs rs' in
     let pj'_ity = pj'.pv_ity in
     let pj'_ght = pj'.pv_ghost in
-    if not (ity_equal pj_ity pj'_ity && (pj_ght || not pj'_ght)) then
-      raise (BadInstance id);
+    let pj'_mut = List.exists (pv_equal pj') s'.its_mfields in
+    let bsb = try ity_match bsb pj'_ity pj_ity with TypeMismatch _ ->
+      raise (BadInstance (BadI_field_type (s.its_ts, pj.pv_vs))) in
+    let fsb = try ity_match fsb pj_ity pj'_ity with TypeMismatch _ ->
+      raise (BadInstance (BadI_field_type (s.its_ts, pj.pv_vs))) in
+    if not (pj_ght || not pj'_ght) then
+      raise (BadInstance (BadI_field_ghost (s.its_ts, pj.pv_vs)));
+    if not (pj_mut = pj'_mut) then
+      raise (BadInstance (BadI_field_mut (s.its_ts, pj.pv_vs)));
     let ls, ls' = ls_of_rs rs, ls_of_rs rs' in
     cl.ls_table <- Mls.add ls ls' cl.ls_table;
     cl.rs_table <- Mrs.add rs rs' cl.rs_table;
-    cl.fd_table <- Mpv.add pj pj' cl.fd_table in
-  List.iter match_pj d.itd_fields;
+    cl.fd_table <- Mpv.add pj pj' cl.fd_table;
+    Hpv.add matched_fields' pj' pj;
+    bsb, fsb in
+  ignore (List.fold_left match_pj (isb_empty,isb_empty) d.itd_fields);
+  (* the new fields must be separated from the old ones *)
+  let check_npj' (op,np) rs' =
+    let pj' = fd_of_rs rs' in
+    if Hpv.mem matched_fields' pj' then ity_freeze op pj'.pv_ity, np
+                                   else op, ity_freeze np pj'.pv_ity in
+  let op,np = List.fold_left check_npj' (isb_empty,isb_empty) d'.itd_fields in
+  if not (Mreg.set_disjoint op.isb_reg np.isb_reg) then
+    raise (BadInstance (BadI_ty_alias s.its_ts));
+  (* if we refine a mutable type, then all new regions in the invariant
+     come from the new fields (no invariant strengthening on the old ones) *)
+  let pj_occurs pj f = t_v_occurs pj.pv_vs f > 0 in
+  let check_opj' pj' pj =
+    if not pj'.pv_ity.ity_pure &&
+       List.exists (pj_occurs pj') d'.itd_invariant &&
+       not (List.exists (pj_occurs pj) d.itd_invariant) then
+    raise (BadInstance (BadI_field_inv (s.its_ts, pj.pv_vs))) in
+  if s.its_mutable then Hpv.iter check_opj' matched_fields';
+  (* validate the refinement *)
   cl.ts_table <- Mts.add s.its_ts s' cl.ts_table
 
 let clone_type_decl inst cl tdl kn =
@@ -952,13 +1016,13 @@ let clone_type_decl inst cl tdl kn =
       begin match Mts.find_opt ts inst.mi_ts with
       | Some s' ->
           if not (List.length ts.ts_args = List.length s'.its_ts.ts_args) then
-            raise (BadInstance id);
+            raise (BadInstance (BadI_ty_arity ts));
           let pd' = Mid.find s'.its_ts.ts_name kn in
           let d' = match pd'.pd_node with
             | PDtype [d'] -> d'
-            (* FIXME: we could refine with mutual types *)
-            | PDtype _ -> raise (BadInstance id)
-            | PDlet _ | PDexn _ | PDpure -> raise (BadInstance id) in
+            (* FIXME? we could refine with mutual types *)
+            | PDtype _ -> raise (BadInstance (BadI_ty_rec ts))
+            | PDlet _ | PDexn _ | PDpure -> assert false in
           clone_type_record cl s d s' d'; (* clone record fields *)
           (* generate and add VC for type invariant implication *)
           if d.itd_invariant <> [] then begin
@@ -968,16 +1032,16 @@ let clone_type_decl inst cl tdl kn =
             List.iter add_vc invl end
       | None -> begin match Mts.find_opt ts inst.mi_ty with
       | Some ity -> (* creative indentation *)
-          (* TODO: clone_type_record, axiom_of_invariant *)
-          (* TODO: should we only allow cloning into ity for
-             private types with no fields and no invariant? *)
           let stv = Stv.of_list ts.ts_args in
-          if not (Stv.subset (ity_freevars Stv.empty ity) stv &&
-                  its_pure s && ity.ity_pure) then raise (BadInstance id);
+          if not (Stv.subset (ity_freevars Stv.empty ity) stv) then
+            raise (BadInstance (BadI_ty_vars ts));
+          if not (d.itd_fields = [] && d.itd_invariant = []) then
+            raise (BadInstance (BadI_ty_ner ts));
+          if not (its_pure s && ity.ity_pure) then
+            raise (BadInstance (BadI_ty_impure ts));
           cl.ty_table <- Mts.add ts ity cl.ty_table
       | None -> assert false end end;
       Hits.add htd s None;
-      (* TODO: check typing conditions for refined record type *)
     end else
     (* variant *)
     if not s.its_mutable && d.itd_constructors <> [] &&
@@ -1040,7 +1104,7 @@ let clone_type_decl inst cl tdl kn =
 let add_vc uc (its, f) =
   let {id_string = nm; id_loc = loc} = its.its_ts.ts_name in
   let attrs = Sattr.singleton (Ident.create_attribute ("expl:VC for " ^ nm)) in
-  let pr = create_prsymbol (id_fresh ~attrs ?loc ("VC " ^ nm)) in
+  let pr = create_prsymbol (id_fresh ~attrs ?loc (nm ^ "'vc")) in
   let d = create_pure_decl (create_prop_decl Pgoal pr f) in
   add_pdecl ~warn:false ~vc:false uc d
 
@@ -1072,48 +1136,83 @@ let clone_pdecl inst cl uc d = match d.pd_node with
       let add uc d = add_pdecl ~warn:false ~vc:false uc d in
       List.fold_left add uc dl
   | PDlet (LDsym (rs, c)) when Mrs.mem rs inst.mi_rs ->
+      freeze_foreign cl (cty_reads c.c_cty);
       (* refine only [val] symbols *)
       let cty = match c.c_node with (* cty for [val constant] is not c.c_cty *)
         | Cany -> c.c_cty
         | Cfun {e_node = Eexec ({c_node = Cany}, cty)} -> cty
-        | _ -> raise (BadInstance rs.rs_name) in
-      let kind = match rs.rs_logic with
-        | RLnone -> RKnone
-        | RLpv _ -> raise (BadInstance rs.rs_name)
-        | RLls ls when ls.ls_value = None -> RKpred
-        | RLls _ -> RKfunc
-        | RLlemma -> RKlemma in
+        | _ -> raise (CannotInstantiate rs.rs_name) in
       let cty = clone_cty cl (sm_of_cl cl) cty in
       let rs' = Mrs.find rs inst.mi_rs in
-      (* arity and types will be checked when refinement VC is generated *)
-      begin match rs.rs_logic, rs'.rs_logic with
-      | RLnone, (RLnone | RLls _ | RLlemma) | RLlemma, RLlemma -> ()
-      | RLls ls, RLls ls' -> cl.ls_table <- Mls.add ls ls' cl.ls_table
-      | _ -> raise (BadInstance rs.rs_name)
-      end;
-      begin
-        match cty.cty_effect.eff_oneway, rs'.rs_cty.cty_effect.eff_oneway with
-        | _, Total | Diverges, _ | Partial, Partial -> ()
-        | _ -> raise (CloneDivergence (rs.rs_name, rs'.rs_name))
-      end;
-      cl.rs_table <- Mrs.add rs rs' cl.rs_table;
-      let e = e_exec (c_app rs' cty.cty_args [] cty.cty_result) in
+      cl_save_rs cl rs rs';
+      if List.length cty.cty_args <> List.length rs'.rs_cty.cty_args then
+        raise (BadInstance (BadI_rs_arity rs.rs_name));
+      let e = try e_exec (c_app rs' cty.cty_args [] cty.cty_result) with
+        | TypeMismatch _ -> raise (BadInstance (BadI_rs_type rs.rs_name)) in
       let cexp = c_fun ~mask:cty.cty_mask cty.cty_args cty.cty_pre
         cty.cty_post cty.cty_xpost cty.cty_oldies e in
-      let id = id_clone rs.rs_name in (* FIXME better name *)
-      let ld, _ = let_sym id ~ghost:(rs_ghost rs) ~kind cexp in
-      (* FIXME check ghost status and mask of cexp/ld wrt rs *)
-      (* FIXME check effects of cexp/ld wrt rs *)
+      let id = id_derive (rs.rs_name.id_string ^ "'refn") rs.rs_name in
+      let ld, ss = let_sym id ~ghost:(rs_ghost rs) ~kind:(rs_kind rs) cexp in
+      if cty_ghost ss.rs_cty <> cty_ghost rs.rs_cty then
+        raise (BadInstance (BadI_rs_ghost rs.rs_name));
+      if mask_spill ss.rs_cty.cty_mask rs.rs_cty.cty_mask then
+        raise (BadInstance (BadI_rs_mask rs.rs_name));
+      let eff = eff_ghostify (rs_ghost rs) cty.cty_effect in
+      let eff' = eff_union_par eff ss.rs_cty.cty_effect in
+      if not (eff_equal eff eff') then begin
+        (* Format.eprintf "@[%a@]@\n" print_cty cty; *)
+        (* Format.eprintf "@[%a@]@\n" print_cty ss.rs_cty; *)
+        let find_vars regs =
+          let reads = Spv.union eff.eff_reads (Spv.of_list cty.cty_args) in
+          Mreg.fold (fun r _ svs -> Spv.fold (fun v svs ->
+            if ity_r_reachable r v.pv_ity
+            then Svs.add v.pv_vs svs else svs) reads svs) regs Svs.empty in
+        if not (Spv.equal eff.eff_reads eff'.eff_reads) then
+          raise (BadInstance (BadI_rs_reads (rs.rs_name,
+            Spv.fold (fun v svs -> Svs.add v.pv_vs svs)
+              (Spv.diff eff'.eff_reads eff.eff_reads) Svs.empty)));
+        (* extra resets may lead to fewer writes, so we check them first *)
+        if not (Sreg.subset eff.eff_covers eff'.eff_covers) then
+          raise (BadInstance (BadI_rs_resets (rs.rs_name,
+            find_vars (Sreg.diff eff.eff_covers eff'.eff_covers))));
+        if not (Mreg.equal Spv.equal eff.eff_writes eff'.eff_writes) then
+          raise (BadInstance (BadI_rs_writes (rs.rs_name,
+            find_vars (Mreg.filter (fun r fd ->
+              try not (Spv.equal fd (Mreg.find r eff.eff_writes))
+              with Not_found -> true) eff'.eff_writes))));
+        if not (Sreg.equal eff.eff_taints eff'.eff_taints) then
+          raise (BadInstance (BadI_rs_taints (rs.rs_name,
+            find_vars (Sreg.diff eff'.eff_taints eff.eff_taints))));
+        if not (Sreg.equal eff.eff_covers eff'.eff_covers) then
+          raise (BadInstance (BadI_rs_covers (rs.rs_name,
+            find_vars (Sreg.diff eff'.eff_covers eff.eff_covers))));
+        if not (Sreg.equal eff.eff_resets eff'.eff_resets) then
+          raise (BadInstance (BadI_rs_resets (rs.rs_name,
+            find_vars (Sreg.diff eff'.eff_resets eff.eff_resets))));
+        if not (Sxs.equal eff.eff_raises eff'.eff_raises) then
+          raise (BadInstance (BadI_rs_raises (rs.rs_name,
+            Sxs.fold (fun xs sid -> Sid.add xs.xs_name sid)
+              (Sxs.diff eff'.eff_raises eff.eff_raises) Sid.empty)));
+        if not (Stv.equal eff.eff_spoils eff'.eff_spoils) then
+          raise (BadInstance (BadI_rs_spoils (rs.rs_name,
+              Stv.diff eff'.eff_spoils eff.eff_spoils)));
+        if eff.eff_oneway <> eff'.eff_oneway then
+          raise (BadInstance (BadI_rs_oneway rs.rs_name));
+        if eff.eff_ghost <> eff'.eff_ghost then
+          raise (BadInstance (BadI_rs_ghost rs.rs_name));
+        raise (BadInstance (BadI rs.rs_name));
+      end;
       (* FIXME add correspondance for "let lemma" to cl.pr_table *)
       let dl = mk_vc uc (create_let_decl ld) in
       List.fold_left (add_pdecl_raw ~warn:false) uc dl
   | PDlet ld ->
       begin match ld with
         | LDvar ({pv_vs=vs}, _) when Mvs.mem vs inst.mi_pv ->
-            raise (BadInstance vs.vs_name)
+            raise (CannotInstantiate vs.vs_name)
         | LDrec rdl ->
-            let no_inst { rec_sym = rs } =
-              if Mrs.mem rs inst.mi_rs then raise (BadInstance rs.rs_name) in
+            let no_inst {rec_sym = rs} =
+              if Mrs.mem rs inst.mi_rs then
+                raise (CannotInstantiate rs.rs_name) in
             List.iter no_inst rdl
         | _ -> () end;
       let reads = match ld with
@@ -1129,8 +1228,9 @@ let clone_pdecl inst cl uc d = match d.pd_node with
       let xs' = Mxs.find xs inst.mi_xs in
       begin try let ity = clone_ity cl xs.xs_ity in
                 ignore (ity_match isb_empty xs'.xs_ity ity)
-        with TypeMismatch _ -> raise (BadInstance id) end;
-      if mask_spill xs'.xs_mask xs.xs_mask then raise (BadInstance id);
+        with TypeMismatch _ -> raise (BadInstance (BadI_xs_type id)) end;
+      if mask_spill xs'.xs_mask xs.xs_mask then
+        raise (BadInstance (BadI_xs_mask id));
       cl.xs_table <- Mxs.add xs xs' cl.xs_table;
       uc
   | PDexn xs ->
@@ -1273,15 +1373,9 @@ let print_module fmt m = Format.fprintf fmt
   "@[<hov 2>module %s@\n%a@]@\nend" m.mod_theory.th_name.id_string
   (Pp.print_list Pp.newline2 print_unit) m.mod_units
 
-let print_id fmt id = Ident.print_decoded fmt id.id_string
-
 let () = Exn_printer.register (fun fmt e -> match e with
   | IncompatibleNotation nm -> Format.fprintf fmt
       "Incombatible type signatures for notation '%a'" Ident.print_decoded nm
   | ModuleNotFound (sl,s) -> Format.fprintf fmt
       "Module %s not found in library %a" s print_path sl
-  | CloneDivergence (iv, il) -> Format.fprintf fmt
-      "Cannot instantiate symbol %a with symbol %a \
-       that has worse termination status"
-      print_id iv print_id il
   | _ -> raise e)
