@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -153,9 +153,12 @@ let attrs_has_expl attrs =
 let annot_attrs = Sattr.add stop_split (Sattr.singleton annot_attr)
 
 let vc_expl loc attrs expl f =
+  let rec relocate g =
+    t_attr_set ?loc g.t_attrs (TermTF.t_map (fun t -> t) relocate g) in
   let attrs = Sattr.union annot_attrs (Sattr.union attrs f.t_attrs) in
   let attrs = if attrs_has_expl attrs then attrs else Sattr.add expl attrs in
-  t_attr_set ?loc:(if loc = None then f.t_loc else loc) attrs f
+  if loc = None then t_attr_set ?loc:f.t_loc attrs f
+                else t_attr_set ?loc attrs (relocate f)
 
 (* propositional connectives with limited simplification *)
 
@@ -511,8 +514,7 @@ let inv_of_pure {known_map = kn} loc fl k =
    [res] names the result of the normal execution of [e]
    [xmap] maps every raised exception to a pair [i,xres]:
    - [i] is a positive int assigned at the catching site
-   - [xres] names the value carried by the exception
-   [case_xmap] is used for match-with-exceptions *)
+   - [xres] names the value carried by the exception *)
 let rec k_expr env lps e res xmap =
   let loc = e.e_loc and eff = e.e_effect in
   let attrs = Sattr.diff e.e_attrs vc_attrs in
@@ -824,7 +826,11 @@ let rec k_expr env lps e res xmap =
         let prev = sp_of_inv None attrs expl_loop_init invl in
         let keep = wp_of_inv None attrs expl_loop_keep invl in
         let oldies, ovarl = oldify_variant varl in
-        let decr = decrease env loc attrs expl_loop_vari ovarl varl in
+        let variant_loc = (* Compute the location of the variant. *)
+          match varl with
+          | (var_t, _) :: _ when var_t.t_loc <> None -> var_t.t_loc
+          | _ -> loc in
+        let decr = decrease env variant_loc attrs expl_loop_vari ovarl varl in
         let keep = wp_and decr keep in
         let iinv = inv_of_loop env e.e_loc invl varl in
         let j = List.fold_right assert_inv iinv (Kstop init) in
@@ -1294,7 +1300,8 @@ let wrt_exists vl f = wrt_rename sp_exists vl f
    variable in the range of [wr_i] is free in [sp_i]. *)
 
 let rec sp_expr kn k rdm dst = match k with
-  | Kseq (Klet (v, t, f), 0, k2) ->
+  | Kseq (Klet (v, t, f), 0, k2)
+    when not (has_a_model_attr v.pv_vs.vs_name) ->
       let wp2, sp2, rd2 = sp_expr kn k2 rdm dst in
       let rd1 = t_freepvs (t_freepvs rd2 t) f in
       let wp = wp_let v t (sp_implies f wp2) in
@@ -1513,6 +1520,16 @@ and wp_expr kn k q = match k with
       let wp, _, _ = sp_expr kn k Mint.empty Mpv.empty in wp
   | Ktag ((Push _|WP), _) -> assert false (* cannot happen *)
 
+let rec simp_cast_projections env t = match t.t_node with
+  | Tapp (ls, [{t_node = Tconst (Constant.ConstInt _ as c);
+                t_ty = Some {ty_node = Tyapp (ts,_)}}])
+    when is_range_type_def ts.ts_def
+      && ls_equal ls (int_of_range env ts) ->
+      (* t'int (c:t) -> c *)
+      t_const c ty_int
+  | _ ->
+      t_map (simp_cast_projections env) t
+
 (** VCgen *)
 
 let vc_kode {known_map = kn} vc_wp k =
@@ -1529,15 +1546,16 @@ let vc_fun env vc_wp cty e =
 let vc_rec env vc_wp rdl =
   List.map (vc_kode env vc_wp) (k_rec env Mls.empty rdl)
 
-let mk_vc_decl kn id f =
+let mk_vc_decl ({known_map = kn} as env) id f =
   let {id_string = nm; id_attrs = attrs; id_loc = loc} = id in
   let attrs = if attrs_has_expl attrs then attrs else
     Sattr.add (Ident.create_attribute ("expl:VC for " ^ nm)) attrs in
-  let pr = create_prsymbol (id_fresh ~attrs ?loc ("VC " ^ nm)) in
+  let pr = create_prsymbol (id_fresh ~attrs ?loc (nm ^ "'vc")) in
   let f = wp_forall (Mvs.keys (t_freevars Mvs.empty f)) f in
   let f = Typeinv.inject kn f in
   let f = if Debug.test_flag debug_no_eval then f else
-    Eval_match.eval_match kn f in
+          Eval_match.eval_match kn f in
+  let f = simp_cast_projections env f in
   create_pure_decl (create_prop_decl Pgoal pr f)
 
 let add_vc_decl kn id f vcl =
@@ -1552,18 +1570,20 @@ let vc env kn tuc d = match d.pd_node with
         | Eexec ({c_node = Cfun e} as c, _) -> c, e
         | _ -> c_fun [] [] [] Mxs.empty Mpv.empty e, e in
       let f = vc_fun env (Debug.test_noflag debug_sp) c.c_cty e in
-      add_vc_decl kn v.pv_vs.vs_name f []
+      add_vc_decl env v.pv_vs.vs_name f []
   | PDlet (LDsym (s, {c_node = Cfun e; c_cty = cty})) ->
       let env = mk_env env kn tuc in
       let f = vc_fun env (Debug.test_noflag debug_sp) cty e in
-      add_vc_decl kn s.rs_name f []
+      add_vc_decl env s.rs_name f []
   | PDlet (LDrec rdl) ->
       let env = mk_env env kn tuc in
       let fl = vc_rec env (Debug.test_noflag debug_sp) rdl in
-      let add rd f vcl = add_vc_decl kn rd.rec_sym.rs_name f vcl in
+      let add rd f vcl = add_vc_decl env rd.rec_sym.rs_name f vcl in
       List.fold_right2 add rdl fl []
   | PDtype tdl ->
+      let env = lazy (mk_env env kn tuc) in
       let add_witness d vcl =
+        let env = Lazy.force env in
         let add_fd (mv,ldl) fd e =
           let fd = fd_of_rs fd in
           let id = id_clone fd.pv_vs.vs_name in
@@ -1577,16 +1597,16 @@ let vc env kn tuc d = match d.pd_node with
           e_let ld e) d.itd_invariant e_void in
         let e = List.fold_right e_let ldl e in
         let c = c_fun [] [] [] Mxs.empty Mpv.empty e in
-        let f = vc_fun (mk_env env kn tuc)
-          (Debug.test_noflag debug_sp) c.c_cty e in
-        add_vc_decl kn d.itd_its.its_ts.ts_name f vcl in
+        let f = vc_fun env (Debug.test_noflag debug_sp) c.c_cty e in
+        add_vc_decl env d.itd_its.its_ts.ts_name f vcl in
       let add_invariant d vcl =
+        let env = Lazy.force env in
         let vs_of_rs fd = (fd_of_rs fd).pv_vs in
         let vl = List.map vs_of_rs d.itd_fields in
         let expl f = vc_expl None Sattr.empty expl_type_inv f in
         let f = t_and_asym_l (List.map expl d.itd_invariant) in
         let f = t_exists_close_simp vl [] f in
-        add_vc_decl kn d.itd_its.its_ts.ts_name f vcl in
+        add_vc_decl env d.itd_its.its_ts.ts_name f vcl in
       let add_itd d vcl =
         if d.itd_witness <> [] then add_witness d vcl else
         if d.itd_invariant <> [] then add_invariant d vcl else
