@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -19,6 +19,9 @@ val def: kind:string -> ident -> unit
 val use: kind:string -> Loc.position -> ident -> unit
   (** [use loc id] registers that [id] is used at position [loc] *)
 
+val clear: string -> unit
+  (** [clear file] clears the locations association for said [file] *)
+
 type def_use = Def | Use
 
 val find: Loc.position -> ident * def_use * string
