@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -30,11 +30,16 @@ let debug_parse_only = Debug.register_flag "parse_only"
 let debug_type_only  = Debug.register_flag "type_only"
   ~desc:"Stop@ after@ type-checking."
 
-let debug_useless_at = Debug.register_flag "ignore_useless_at"
+let debug_ignore_useless_at = Debug.register_flag "ignore_useless_at"
   ~desc:"Remove@ warning@ for@ useless@ at/old."
 
 (** symbol lookup *)
 
+
+let qualid_last = function Qident x | Qdot (_, x) -> x
+
+let use_as q = function Some x -> x | None -> qualid_last q
+
 let rec qloc = function
   | Qdot (p, id) -> Loc.join (qloc p) id.id_loc
   | Qident id    -> id.id_loc
@@ -53,26 +58,51 @@ let string_list_of_qualid q =
     | Qident id -> id.id_str :: acc in
   sloq [] q
 
-exception UnboundSymbol of qualid
+(* Type of symbol queried *)
+type symbol_kind =
+  | Prop
+  | Type
+  | Fun_pre
+  | Fun
+  | Predicate
+  | Exc
+  | Prog
 
-let find_qualid get_id find ns q =
+let symbol_kind_to_string qs =
+  match qs with
+  | Prop      -> "proposition"
+  | Type      -> "type"
+  | Fun_pre   -> "function or predicate"
+  | Fun       -> "function"
+  | Predicate -> "predicate"
+  | Exc       -> "exception"
+  | Prog      -> "program function or variable"
+
+(* [UnboundSymbol (s, q)]: s is the type of the qualid searched. It depends on
+   the namespace queried: find_*_ns. *)
+exception UnboundSymbol of (symbol_kind * qualid)
+
+let find_qualid ~ty get_id find ns q =
   let sl = string_list_of_qualid q in
   let r = try find ns sl with Not_found ->
-    Loc.error ~loc:(qloc q) (UnboundSymbol q) in
+    Loc.error ~loc:(qloc q) (UnboundSymbol (ty, q)) in
   if Debug.test_flag Glob.flag then Glob.use ~kind:"" (qloc_last q) (get_id r);
   r
 
-let find_prop_ns     ns q = find_qualid (fun pr -> pr.pr_name) ns_find_pr ns q
-let find_tysymbol_ns ns q = find_qualid (fun ts -> ts.ts_name) ns_find_ts ns q
-let find_lsymbol_ns  ns q = find_qualid (fun ls -> ls.ls_name) ns_find_ls ns q
+let find_prop_ns     ns q =
+  find_qualid ~ty:Prop (fun pr -> pr.pr_name) ns_find_pr ns q
+let find_tysymbol_ns ns q =
+  find_qualid ~ty:Type (fun ts -> ts.ts_name) ns_find_ts ns q
+let find_lsymbol_ns ?ty ns q =
+  find_qualid ~ty:(Opt.get_def Fun_pre ty) (fun ls -> ls.ls_name) ns_find_ls ns q
 
 let find_fsymbol_ns ns q =
-  let ls = find_lsymbol_ns ns q in
+  let ls = find_lsymbol_ns ~ty:Fun ns q in
   if ls.ls_value <> None then ls else
     Loc.error ~loc:(qloc q) (FunctionSymbolExpected ls)
 
 let find_psymbol_ns ns q =
-  let ls = find_lsymbol_ns ns q in
+  let ls = find_lsymbol_ns ~ty:Predicate ns q in
   if ls.ls_value = None then ls else
     Loc.error ~loc:(qloc q) (PredicateSymbolExpected ls)
 
@@ -92,10 +122,10 @@ let find_prop_of_kind k tuc q =
         | Plemma -> "a lemma" | Paxiom -> "an axiom" | Pgoal -> "a goal")
 
 let find_itysymbol_ns ns q =
-  find_qualid (fun s -> s.its_ts.ts_name) Pmodule.ns_find_its ns q
+  find_qualid ~ty:Type (fun s -> s.its_ts.ts_name) Pmodule.ns_find_its ns q
 
 let find_xsymbol_ns ns q =
-  find_qualid (fun s -> s.xs_name) Pmodule.ns_find_xs ns q
+  find_qualid ~ty:Exc (fun s -> s.xs_name) Pmodule.ns_find_xs ns q
 
 let find_prog_symbol_ns ns p =
   let get_id_ps = function
@@ -104,7 +134,13 @@ let find_prog_symbol_ns ns p =
       (* FIXME: this is incorrect, but we cannot
         know the correct symbol at this stage *)
     | OO ss -> (Srs.choose ss).rs_name in
-  find_qualid get_id_ps ns_find_prog_symbol ns p
+  find_qualid ~ty:Prog get_id_ps ns_find_prog_symbol ns p
+
+let import_namespace ~loc ns sl =
+  Loc.try2 ~loc import_namespace ns sl
+
+let import_scope ~loc ns sl =
+  Loc.try2 ~loc import_scope ns sl
 
 (** Parsing types *)
 
@@ -123,7 +159,8 @@ let rec ty_of_pty ns = function
   | PTarrow (ty1, ty2) ->
       ty_func (ty_of_pty ns ty1) (ty_of_pty ns ty2)
   | PTscope (q, ty) ->
-      let ns = import_namespace ns (string_list_of_qualid q) in
+      let loc = qloc q in
+      let ns = import_namespace ~loc ns (string_list_of_qualid q) in
       ty_of_pty ns ty
   | PTpure ty | PTparen ty ->
       ty_of_pty ns ty
@@ -168,7 +205,8 @@ let rec dpattern ns km { pat_desc = desc; pat_loc = loc } =
   match desc with
     | Ptree.Pparen p -> dpattern ns km p
     | Ptree.Pscope (q,p) ->
-        let ns = import_namespace ns (string_list_of_qualid q) in
+        let loc = qloc q in
+        let ns = import_namespace ~loc ns (string_list_of_qualid q) in
         dpattern ns km p
     | _ -> (* creative indentation ahead *)
   Dterm.dpattern ~loc (match desc with
@@ -356,10 +394,12 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
         | e23 ->
             apply loc de1 op1 (dterm ns km crcmap gvars at denv e23) in
       chain loc (dterm ns km crcmap gvars at denv e1) op1 e23
-  | Ptree.Tconst (Number.ConstInt _ as c) ->
+  | Ptree.Tconst (Constant.ConstInt _ as c) ->
       DTconst (c, dty_int)
-  | Ptree.Tconst (Number.ConstReal _ as c) ->
+  | Ptree.Tconst (Constant.ConstReal _ as c) ->
       DTconst (c, dty_real)
+  | Ptree.Tconst (Constant.ConstStr _ as c) ->
+      DTconst (c, dty_str)
   | Ptree.Tlet (x, e1, e2) ->
       let id = create_user_id x in
       let e1 = dterm ns km crcmap gvars at denv e1 in
@@ -433,12 +473,13 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
       (* check if the label has actually been defined *)
       ignore (Loc.try2 ~loc gvars (Some l) (Qident id));
       let e1 = dterm ns km crcmap gvars (Some l) denv e1 in
-      if not (Hstr.find at_uses l) && Debug.test_noflag debug_useless_at then
+      if not (Hstr.find at_uses l) && Debug.test_noflag debug_ignore_useless_at then
         Warning.emit ~loc "this `at'/`old' operator is never used";
       Hstr.remove at_uses l;
       DTattr (e1, Sattr.empty)
   | Ptree.Tscope (q, e1) ->
-      let ns = import_namespace ns (string_list_of_qualid q) in
+      let loc = qloc q in
+      let ns = import_namespace ~loc ns (string_list_of_qualid q) in
       DTattr (dterm ns km crcmap gvars at denv e1, Sattr.empty)
   | Ptree.Tasref q ->
       let e1 = { term_desc = Tident q; term_loc = loc } in
@@ -475,6 +516,7 @@ open Dexpr
 let ty_of_pty tuc = ty_of_pty (get_namespace tuc)
 
 let get_namespace muc = List.hd muc.Pmodule.muc_import
+let get_namespace_export muc = List.hd muc.Pmodule.muc_export
 
 let dterm muc =
   let uc = muc.muc_theory in
@@ -484,6 +526,10 @@ let find_xsymbol     muc q = find_xsymbol_ns     (get_namespace muc) q
 let find_itysymbol   muc q = find_itysymbol_ns   (get_namespace muc) q
 let find_prog_symbol muc q = find_prog_symbol_ns (get_namespace muc) q
 
+let find_rsymbol muc q =
+  let ns = get_namespace_export muc in
+  find_qualid ~ty:Prog (fun rs -> rs.rs_name) ns_find_rs ns q
+
 let find_special muc test nm q =
   match find_prog_symbol muc q with
   | RS s when test s -> s
@@ -512,8 +558,9 @@ let rec ity_of_pty muc = function
   | PTpure ty ->
       ity_purify (ity_of_pty muc ty)
   | PTscope (q, ty) ->
+      let loc = qloc q in
       let muc = open_scope muc "dummy" in
-      let muc = import_scope muc (string_list_of_qualid q) in
+      let muc = import_scope ~loc muc (string_list_of_qualid q) in
       ity_of_pty muc ty
   | PTparen ty ->
       ity_of_pty muc ty
@@ -547,10 +594,11 @@ let parse_record muc fll =
   let itd = find_its_defn muc.muc_known its in
   let check v s = match s.rs_field with
     | Some u -> pv_equal v u
-    | _ -> false in
+    | _ -> raise (BadRecordUnnamed (ls_of_rs rs, its.its_ts)) in
   let cs = match itd.itd_constructors with
     | [cs] when Lists.equal check cs.rs_cty.cty_args itd.itd_fields -> cs
-    | _ -> raise (BadRecordField (ls_of_rs rs)) in
+    | _ :: _ :: _ -> raise (BadRecordCons (ls_of_rs rs, its.its_ts))
+    | _ -> raise (BadRecordType (ls_of_rs rs, its.its_ts)) in
   let pjs = Srs.of_list itd.itd_fields in
   let flm = List.fold_left (fun m (pj,v) -> if Srs.mem pj pjs then
     Mrs.add_new (DuplicateRecordField (ls_of_rs pj)) pj v m
@@ -576,8 +624,9 @@ let rec dpattern muc gh { pat_desc = desc; pat_loc = loc } =
     | Ptree.Pparen p -> dpattern muc gh p
     | Ptree.Pghost p -> dpattern muc true p
     | Ptree.Pscope (q,p) ->
+        let loc = qloc q in
         let muc = open_scope muc "dummy" in
-        let muc = import_scope muc (string_list_of_qualid q) in
+        let muc = import_scope ~loc muc (string_list_of_qualid q) in
         dpattern muc gh p
     | _ -> (* creative indentation ahead *)
   Dexpr.dpattern ~loc (match desc with
@@ -671,7 +720,23 @@ let dpost muc ql lvm old ity =
     | _ -> mk_case loc pfl in
   List.map dpost ql
 
-let dxpost muc ql lvm xsm old =
+let dpost muc ids ql lvm old ity =
+  let check_id id =
+    if Sstr.mem id.id_str ids then Loc.errorm ~loc:id.id_loc
+      "Result %s has the same name as a formal parameter" id.id_str in
+  let rec check p = match p.pat_desc with
+    | Ptree.Pwild -> ()
+    | Ptree.Pvar id -> check_id id
+    | Ptree.Papp (_, pl) | Ptree.Ptuple pl -> List.iter check pl
+    | Ptree.Prec fl -> List.iter (fun (_, p) -> check p) fl
+    | Ptree.Pas (p, id, _) -> check_id id; check p
+    | Ptree.Por (p, _) | Ptree.Pcast (p, _)
+    | Ptree.Pscope (_, p) | Ptree.Pparen p | Ptree.Pghost p -> check p in
+  let check (p, _) = check p in
+  List.iter (fun (_,pfl) -> List.iter check pfl) ql;
+  dpost muc ql lvm old ity
+
+let dxpost muc ids ql lvm xsm old =
   let add_exn (q,pf) m =
     let xs = match q with
       | Qident i ->
@@ -685,7 +750,7 @@ let dxpost muc ql lvm xsm old =
       | None,    Some _ -> l) xs m in
   let mk_xpost loc xs pfl =
     if pfl = [] then [] else
-    dpost muc [loc,pfl] lvm old xs.xs_ity in
+    dpost muc ids [loc,pfl] lvm old xs.xs_ity in
   let exn_map (loc,xpfl) =
     let m = List.fold_right add_exn xpfl Mxs.empty in
     Mxs.mapi (fun xs pfl -> mk_xpost loc xs pfl) m in
@@ -713,20 +778,29 @@ let dvariant muc varl lvm _xsm old =
   let dvar (t,q) = dvar t, Opt.map (find_variant_ls muc) q in
   List.map dvar varl
 
-let dspec muc sp lvm xsm old ity = {
+let dspec muc ids sp lvm xsm old ity = {
   ds_pre     = dpre muc sp.sp_pre lvm old;
-  ds_post    = dpost muc sp.sp_post lvm old ity;
-  ds_xpost   = dxpost muc sp.sp_xpost lvm xsm old;
+  ds_post    = dpost muc ids sp.sp_post lvm old ity;
+  ds_xpost   = dxpost muc ids sp.sp_xpost lvm xsm old;
   ds_reads   = dreads muc sp.sp_reads lvm;
   ds_writes  = dwrites muc sp.sp_writes lvm;
   ds_checkrw = sp.sp_checkrw;
   ds_diverge = sp.sp_diverge;
   ds_partial = sp.sp_partial; }
 
-let dspec_no_variant muc sp = match sp.sp_variant with
+let dspec muc pl sp =
+  let add_ident acc (a,_,_) = match a with
+    | None -> acc
+    | Some { pre_name = s ; pre_loc = loc } ->
+        if String.equal s "result" then Loc.errorm ?loc
+          "Formal parameters of program functions cannot be named 'result'";
+        Sstr.add s acc in
+  dspec muc (List.fold_left add_ident Sstr.empty pl) sp
+
+let dspec_no_variant muc pl sp = match sp.sp_variant with
   | ({term_loc = loc},_)::_ ->
       Loc.errorm ~loc "unexpected 'variant' clause"
-  | _ -> dspec muc sp
+  | _ -> dspec muc pl sp
 
 let dassert muc f lvm _xsm old = type_fmla muc lvm old f
 
@@ -754,8 +828,8 @@ let mk_let ~loc n de node =
   DElet ((id_user n loc, false, RKnone, de), de1)
 
 let update_any kind e = match e.expr_desc with
-  | Ptree.Eany (pl, _, pty, msk, sp) ->
-      { e with expr_desc = Ptree.Eany (pl, kind, pty, msk, sp) }
+  | Ptree.Eany (pl, _, pty, pat, msk, sp) ->
+      { e with expr_desc = Ptree.Eany (pl, kind, pty, pat, msk, sp) }
   | _ -> e
 
 let local_kind = function
@@ -791,8 +865,9 @@ let rec eff_dterm muc denv {term_desc = desc; term_loc = loc} =
   | Ptree.Tapply (e1, e2) ->
       DEapp (eff_dterm muc denv e1, eff_dterm muc denv e2)
   | Ptree.Tscope (q, e1) ->
+      let loc = qloc q in
       let muc = open_scope muc "dummy" in
-      let muc = import_scope muc (string_list_of_qualid q) in
+      let muc = import_scope ~loc muc (string_list_of_qualid q) in
       DEattr (eff_dterm muc denv e1, Sattr.empty)
   | Ptree.Tasref q ->
       let e1 = { term_desc = Tident q; term_loc = loc } in
@@ -848,6 +923,20 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
         -> DEgexn (find_xsymbol muc q))
     | _ -> DEgexn (find_xsymbol muc q)
   in
+  let handle_alias denv alias pat dity =
+    let pp = dpattern muc pat in
+    let denv = denv_add_pat denv pp dity in
+    let res = Some (id_fresh "result"), false, dity in
+    let denv = denv_add_args denv [res] in
+    let add_alias (t1, t2) =
+      let bty = dity_fresh () in
+      let dt1 = eff_dterm muc denv t1 in
+      let dt2 = eff_dterm muc denv t2 in
+      ignore (Dexpr.dexpr ~loc:t1.term_loc (DEcast (dt1, bty)));
+      ignore (Dexpr.dexpr ~loc:t2.term_loc (DEcast (dt2, bty)));
+    in
+    List.iter add_alias alias
+  in
   Dexpr.dexpr ~loc begin match desc with
   | Ptree.Eident q ->
       qualid_app loc q []
@@ -882,14 +971,16 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
         | e23 ->
             apply loc de1 op1 (dexpr muc denv e23) in
       chain "q1 " "q2 " loc (dexpr muc denv e1) op1 e23
-  | Ptree.Econst (Number.ConstInt _ as c) ->
+  | Ptree.Econst (Constant.ConstInt _ as c) ->
       let dty = if Mts.is_empty muc.muc_theory.uc_ranges
                 then dity_int else dity_fresh () in
       DEconst(c, dty)
-  | Ptree.Econst (Number.ConstReal _ as c) ->
+  | Ptree.Econst (Constant.ConstReal _ as c) ->
       let dty = if Mts.is_empty muc.muc_theory.uc_floats
                 then dity_real else dity_fresh () in
       DEconst(c, dty)
+  | Ptree.Econst (Constant.ConstStr _ as c) ->
+      DEconst(c, dity_str)
   | Ptree.Eref ->
       DEsym (RS rs_ref)
   | Ptree.Erecord fl ->
@@ -918,36 +1009,29 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
       let ld = create_user_prog_id id, gh, kind, dexpr muc denv e1 in
       DElet (ld, dexpr muc (denv_add_let denv ld) e2)
   | Ptree.Erec (fdl, e1) ->
-      let update_kind (id, gh, k, bl, pty, msk, sp, e) =
-        id, gh, local_kind k, bl, pty, msk, sp, e in
+      let update_kind (id, gh, k, bl, pty, pat, msk, sp, e) =
+        id, gh, local_kind k, bl, pty, pat, msk, sp, e in
       let fdl = List.map update_kind fdl in
       let denv, rd = drec_defn muc denv fdl in
       DErec (rd, dexpr muc denv e1)
-  | Ptree.Efun (bl, pty, msk, sp, e) ->
+  | Ptree.Efun (bl, pty, pat, msk, sp, e) ->
       let bl = List.map (dbinder muc) bl in
-      let ds = dspec_no_variant muc sp in
+      let ds = dspec_no_variant muc bl sp in
       let dity = dity_of_opt muc pty in
       let denv = denv_add_args denv bl in
+      handle_alias denv sp.sp_alias pat dity;
       DEfun (bl, dity, msk, ds, dexpr muc denv e)
-  | Ptree.Eany (pl, kind, pty, msk, sp) ->
+  | Ptree.Eany (pl, kind, pty, pat, msk, sp) ->
       let pl = List.map (dparam muc) pl in
-      let ds = dspec_no_variant muc sp in
+      let ds = dspec_no_variant muc pl sp in
       let ity = match kind, pty with
         | _, Some pty -> ity_of_pty muc pty
         | RKlemma, None -> ity_unit
         | RKpred, None -> ity_bool
         | _ -> Loc.errorm ~loc "cannot determine the type of the result" in
       let dity = dity_of_ity ity in
-      let res = Some (id_fresh "result"), false, dity in
-      let denv = denv_add_args denv (res::pl) in
-      let add_alias (t1, t2) =
-        let bty = dity_fresh () in
-        let dt1 = eff_dterm muc denv t1 in
-        let dt2 = eff_dterm muc denv t2 in
-        ignore (Dexpr.dexpr ~loc:t1.term_loc (DEcast (dt1, bty)));
-        ignore (Dexpr.dexpr ~loc:t2.term_loc (DEcast (dt2, bty)));
-      in
-      List.iter add_alias sp.sp_alias;
+      let denv = denv_add_args denv pl in
+      handle_alias denv sp.sp_alias pat dity;
       DEany (pl, dity, msk, ds)
   | Ptree.Ematch (e1, bl, xl) ->
       let e1 = dexpr muc denv e1 in
@@ -1030,7 +1114,10 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
       let gvars _at q = try match find_prog_symbol muc q with
         | PV v -> Some v | _ -> None with _ -> None in
       let get_dty pure_denv =
+        let nw = Debug.test_noflag debug_ignore_useless_at in
+        if nw then Debug.set_flag debug_ignore_useless_at;
         let dt = dterm muc gvars None pure_denv t in
+        if nw then Debug.unset_flag debug_ignore_useless_at;
         match dt.dt_dty with Some dty -> dty | None -> dty_bool in
       DEpure (get_term, denv_pure denv get_dty)
   | Ptree.Eassert (ak, f) ->
@@ -1043,8 +1130,9 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
   | Ptree.Elabel (id, e1) ->
       DElabel (create_user_id id, dexpr muc denv e1)
   | Ptree.Escope (q, e1) ->
+      let loc = qloc q in
       let muc = open_scope muc "dummy" in
-      let muc = import_scope muc (string_list_of_qualid q) in
+      let muc = import_scope ~loc muc (string_list_of_qualid q) in
       DEattr (dexpr muc denv e1, Sattr.empty)
   | Ptree.Easref q ->
       let e1 = { expr_desc = Eident q; expr_loc = loc } in
@@ -1062,13 +1150,16 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
   end
 
 and drec_defn muc denv fdl =
-  let prep (id, gh, kind, bl, pty, msk, sp, e) =
+  let prep (id, gh, kind, bl, pty, _pat, msk, sp, e) =
+    if sp.sp_alias <> []
+    then Loc.errorm ~loc:id.id_loc
+           "alias is forbidden in recursive functions";
     let bl = List.map (dbinder muc) bl in
     let dity = dity_of_opt muc pty in
     let pre denv =
       let denv = denv_add_args denv bl in
       let dv = dvariant muc sp.sp_variant in
-      dspec muc sp, dv, dexpr muc denv e in
+      dspec muc bl sp, dv, dexpr muc denv e in
     create_user_prog_id id, gh, kind, bl, dity, msk, pre in
   Dexpr.drec_defn denv (List.map prep fdl)
 
@@ -1266,6 +1357,14 @@ let tyl_of_params {muc_theory = tuc} pl =
     ty_of_pty tuc ty in
   List.map ty_of_param pl
 
+(* Used to check unused variables in logic declarations *)
+let check_unused_vars ldl =
+  List.iter
+    (fun (_name, ld) ->
+       let (vsl, t) = open_ls_defn ld in
+       List.iter (Dterm.check_used_var t) vsl)
+    ldl
+
 let add_logics muc dl =
   let lsymbols = Hstr.create 17 in
   (* 1. create all symbols and make an environment with these symbols *)
@@ -1301,7 +1400,10 @@ let add_logics muc dl =
         abst, (make_ls_defn ls vl t) :: defn in
   let abst,defn = List.fold_right type_decl dl ([],[]) in
   let add_param muc s = add_decl muc (create_param_decl s) in
-  let add_logic muc l = add_decl muc (create_logic_decl l) in
+  let add_logic muc l =
+    (* Check for unused vars in logic declaration *)
+    check_unused_vars l;
+    add_decl muc (create_logic_decl l) in
   let muc = List.fold_left add_param muc abst in
   if defn = [] then muc else add_logic muc defn
 
@@ -1404,8 +1506,10 @@ let type_inst ({muc_theory = tuc} as muc) ({mod_theory = t} as m) s =
             Loc.errorm ~loc:(qloc q) "program constant expected"
         | RS _, PV _ ->
             Loc.errorm ~loc:(qloc q) "program function expected"
-        | OO _, _ | _, OO _ ->
+        | _, OO _ ->
             Loc.errorm ~loc:(qloc q) "ambiguous notation"
+        | OO _, _  ->
+            assert false (* should never happen *)
         end
     | CSxsym (p,q) ->
         let xs1 = find_xsymbol_ns m.mod_export p in
@@ -1430,7 +1534,7 @@ let type_inst ({muc_theory = tuc} as muc) ({mod_theory = t} as m) s =
   in
   List.fold_left add_inst (empty_mod_inst m) s
 
-let add_decl muc env file d =
+let rec add_decl muc env file d =
   let vc = muc.muc_theory.uc_path = [] &&
     Debug.test_noflag debug_type_only in
   match d with
@@ -1453,6 +1557,7 @@ let add_decl muc env file d =
         | Ptree.Max q  -> MApr (find_prop_of_kind Paxiom tuc q)
         | Ptree.Mlm q  -> MApr (find_prop_of_kind Plemma tuc q)
         | Ptree.Mgl q  -> MApr (find_prop_of_kind Pgoal  tuc q)
+        | Ptree.Mval q -> MAid (find_rsymbol muc q).rs_name
         | Ptree.Mstr s -> MAstr s
         | Ptree.Mint i -> MAint i in
       add_meta muc (lookup_meta id.id_str) (List.map convert al)
@@ -1467,12 +1572,58 @@ let add_decl muc env file d =
       let ity = ity_of_pty muc pty in
       let xs = create_xsymbol (create_user_id id) ~mask ity in
       add_pdecl ~vc muc (create_exn_decl xs)
-  | Ptree.Duse use ->
+  | Ptree.Duseexport use ->
       use_export muc (find_module env file use)
-  | Ptree.Dclone (use, inst) ->
+  | Ptree.Dcloneexport (use, inst) ->
       let m = find_module env file use in
       warn_clone_not_abstract (qloc use) m.mod_theory;
       clone_export muc m (type_inst muc m inst)
+  | Ptree.Duseimport (_loc,import,uses) ->
+      let add_import muc (m, q) =
+        let import = import || q = None in
+        let muc = open_scope muc (use_as m q).id_str in
+        let m = find_module env file m in
+        let muc = use_export muc m in
+        close_scope muc ~import in
+      List.fold_left add_import muc uses
+  | Ptree.Dcloneimport (_loc,import,qid,as_opt,inst) ->
+      let import = import || as_opt = None in
+      let muc = open_scope muc (use_as qid as_opt).id_str in
+      let m = find_module env file qid in
+      warn_clone_not_abstract (qloc qid) m.mod_theory;
+      let muc = clone_export muc m (type_inst muc m inst) in
+      let muc = close_scope muc ~import in
+      muc
+  | Ptree.Dimport q ->
+      import_scope muc (string_list_of_qualid q)
+  | Ptree.Dscope (_loc,import,qid,decls) ->
+      let muc = open_scope muc qid.id_str in
+      let add_decl_env_file muc d = add_decl muc env file d in
+      let muc = List.fold_left add_decl_env_file muc decls in
+      let muc = close_scope muc ~import in
+      muc
+
+
+let type_module file env loc path (id,dl) =
+  let muc = create_module env ~path (create_user_id id) in
+  let add_decl_env_file muc d = add_decl muc env file d in
+  let muc = List.fold_left add_decl_env_file muc dl in
+  let m = Loc.try1 ~loc close_module muc in
+  let file = Mstr.add m.mod_theory.th_name.id_string m file in
+  file
+
+let type_mlw_file env path filename mlw_file =
+  if Debug.test_flag Glob.flag then Glob.clear filename;
+  let file = Mstr.empty in
+  let loc = Loc.user_position filename 0 0 0 in
+  let file =
+    match mlw_file with
+    | Ptree.Decls decls -> type_module file env loc path ({id_str=""; id_ats=[]; id_loc=loc},decls)
+    | Ptree.Modules m_or_t ->
+        let type_module_env_loc_path file (id,dl) = type_module file env loc path (id,dl) in
+        List.fold_left type_module_env_loc_path file m_or_t
+  in
+  file
 
 (* incremental parsing *)
 
@@ -1540,14 +1691,6 @@ let close_scope loc ~import =
     let muc = Loc.try1 ~loc (close_scope ~import) (Opt.get slice.muc) in
     slice.muc <- Some muc
 
-let import_scope loc q =
-  assert (not (Stack.is_empty state));
-  let slice = Stack.top state in
-  let muc = top_muc_on_demand loc slice in
-  if Debug.test_noflag debug_parse_only then
-    let muc = Loc.try2 ~loc import_scope muc (string_list_of_qualid q) in
-    slice.muc <- Some muc
-
 let add_decl loc d =
   assert (not (Stack.is_empty state));
   let slice = Stack.top state in
@@ -1559,6 +1702,7 @@ let add_decl loc d =
 (** Exception printing *)
 
 let () = Exn_printer.register (fun fmt e -> match e with
-  | UnboundSymbol q ->
-      Format.fprintf fmt "unbound symbol '%a'" print_qualid q
+  | UnboundSymbol (ty, q) ->
+      Format.fprintf fmt "unbound %s symbol '%a'" (symbol_kind_to_string ty)
+        print_qualid q
   | _ -> raise e)
