@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,10 +9,18 @@
 (*                                                                  *)
 (********************************************************************)
 
+(** Typing parse trees *)
+
+(** {1 Typing parse trees} *)
+
 val debug_parse_only : Debug.flag
 
 val debug_type_only : Debug.flag
 
+val type_mlw_file : Env.env -> string list -> string -> Ptree.mlw_file -> Pmodule.pmodule Wstdlib.Mstr.t
+
+(** {2 Incremental typing of parsed modules} *)
+
 val open_file : Env.env -> Env.pathname -> unit
 
 val close_file : unit -> Pmodule.pmodule Wstdlib.Mstr.t
@@ -27,11 +35,9 @@ val open_scope : Loc.position -> Ptree.ident -> unit
 
 val close_scope : Loc.position -> import:bool -> unit
 
-val import_scope : Loc.position -> Ptree.qualid -> unit
-
 val add_decl : Loc.position -> Ptree.decl -> unit
 
-
+(** {2 Typing terms and formulas in isolation} *)
 
 val string_list_of_qualid : Ptree.qualid -> string list
 
