@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -151,6 +151,19 @@ let unambig_fs fs =
   in
   inspect (Opt.get fs.ls_value)
 
+let number_format = {
+    Number.long_int_support = `Default;
+    Number.negative_int_support = `Default;
+    Number.dec_int_support = `Default;
+    Number.hex_int_support = `Unsupported;
+    Number.oct_int_support = `Unsupported;
+    Number.bin_int_support = `Unsupported;
+    Number.negative_real_support = `Default;
+    Number.dec_real_support = `Default;
+    Number.hex_real_support = `Default;
+    Number.frac_real_support = `Unsupported (fun _ _ -> assert false);
+  }
+
 let rec print_term info fmt t =
   if check_for_counterexample t then
     info.info_model <- add_model_element t info.info_model;
@@ -159,22 +172,7 @@ let rec print_term info fmt t =
 
   let () = match t.t_node with
   | Tconst c ->
-      let number_format = {
-          Number.long_int_support = true;
-          Number.extra_leading_zeros_support = true;
-          Number.negative_int_support = Number.Number_default;
-          Number.dec_int_support = Number.Number_default;
-          Number.hex_int_support = Number.Number_unsupported;
-          Number.oct_int_support = Number.Number_unsupported;
-          Number.bin_int_support = Number.Number_unsupported;
-          Number.def_int_support = Number.Number_unsupported;
-          Number.negative_real_support = Number.Number_default;
-          Number.dec_real_support = Number.Number_default;
-          Number.hex_real_support = Number.Number_default;
-          Number.frac_real_support = Number.Number_unsupported;
-          Number.def_real_support = Number.Number_unsupported;
-        } in
-      Number.print number_format fmt c
+      Constant.(print number_format unsupported_escape) fmt c
   | Tvar { vs_name = id } ->
       print_ident info fmt id
   | Tapp (ls, tl) ->
@@ -227,8 +225,9 @@ let rec print_term info fmt t =
      end
   | Tlet _ -> unsupportedTerm t
       "alt-ergo: you must eliminate let in term"
-  | Tif _ -> unsupportedTerm t
-      "alt-ergo: you must eliminate if_then_else"
+  | Tif(t1,t2,t3) ->
+     fprintf fmt "(if %a then %a else %a)"
+       (print_fmla info) t1 (print_term info) t2 (print_term info) t3
   | Tcase _ -> unsupportedTerm t
       "alt-ergo: you must eliminate match"
   | Teps _ -> unsupportedTerm t
@@ -242,7 +241,7 @@ and print_tapp info fmt = function
   | [] -> ()
   | tl -> fprintf fmt "(%a)" (print_list comma (print_term info)) tl
 
-let rec print_fmla info fmt f =
+and print_fmla info fmt f =
   if check_for_counterexample f then
     info.info_model <- add_model_element f info.info_model;
 
@@ -295,12 +294,18 @@ and print_fmla_node info fmt f = match f.t_node with
       fprintf fmt "true"
   | Tfalse ->
       fprintf fmt "false"
-  | Tif (f1, f2, f3) ->
-      fprintf fmt "((%a and@ %a)@ or@ (not@ %a and@ %a))"
-        (print_fmla info) f1 (print_fmla info) f2 (print_fmla info)
-        f1 (print_fmla info) f3
-  | Tlet _ -> unsupportedTerm f
-      "alt-ergo: you must eliminate let in formula"
+  | Tif(t1,t2,t3) ->
+     fprintf fmt "(if %a then %a else %a)"
+       (print_fmla info) t1 (print_fmla info) t2 (print_fmla info) t3
+  | Tlet (t1, tb) ->
+      let v, f2 = t_open_bound tb in
+      fprintf fmt "(let %a =@ %a@ : %a in@ %a)"
+        (print_ident info) v.vs_name
+        (print_term info) t1
+         (** some version of alt-ergo have an inefficient typing of let *)
+        (print_type info) v.vs_ty
+        (print_fmla info) f2;
+      forget_var info v
   | Tcase _ -> unsupportedTerm f
       "alt-ergo: you must eliminate match"
   | Tvar _ | Tconst _ | Teps _ -> raise (FmlaExpected f)
@@ -470,7 +475,7 @@ let print_task args ?old:_ fmt task =
   let inv_trig = Task.on_tagged_ls meta_invalid_trigger task in
   let show,cast = Task.on_meta meta_printer_option
     check_options (false,true) task in
-  let cntexample = Prepare_for_counterexmp.get_counterexmp task in
+  let cntexample = Inlining.get_counterexmp task in
   let vc_loc = Intro_vc_vars_counterexmp.get_location_of_vc task in
   let vc_info = {vc_inside = false; vc_loc = None; vc_func_name = None} in
   let info = {
