@@ -1,7 +1,7 @@
 /********************************************************************/
 /*                                                                  */
 /*  The Why3 Verification Platform   /   The Why3 Development Team  */
-/*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  */
+/*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  */
 /*                                                                  */
 /*  This software is distributed under the terms of the GNU Lesser  */
 /*  General Public License version 2.1, with the special exception  */
@@ -131,7 +131,7 @@ void print_request(prequest r) {
   if (r) {
     switch (r->req_type) {
     case REQ_RUN:
-      printf("req_type=REQ_RUN, timeout=%d, memlimit=%d, cmd=%s", r->timeout, r->memlimit, r->cmd);
+      printf("req_type=REQ_RUN, timeout=%d, memlimit=%u, cmd=%s", r->timeout, r->memlimit, r->cmd);
       for (int i = 0; i < r->numargs; i++) {
         printf(" %s", r->args[i]);
       }
