@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -12,14 +12,11 @@
 open Format
 open Wstdlib
 open Session_itp
-open Generic_arg_trans_utils
-open Args_wrapper
 
 let debug_sched = Debug.register_info_flag "scheduler"
   ~desc:"Print@ debugging@ messages@ about@ scheduling@ of@ prover@ calls@ \
          and@ transformation@ applications."
 
-let debug_call_prover = Debug.lookup_flag "call_prover"
 let default_delay_ms = 100 (* 0.1 seconds *)
 
 (** State of a proof *)
@@ -41,7 +38,8 @@ let print_status fmt st =
   | Scheduled         -> fprintf fmt "Scheduled"
   | Running           -> fprintf fmt "Running"
   | Done r            ->
-      fprintf fmt "Done(%a)" Call_provers.print_prover_result r
+      fprintf fmt "Done(%a)"
+        (Call_provers.print_prover_result ~json_model:false) r
   | Interrupted       -> fprintf fmt "Interrupted"
   | Detached          -> fprintf fmt "Detached"
   | InternalFailure e ->
@@ -85,12 +83,32 @@ type controller =
     controller_running_proof_attempts : unit Hpan.t;
   }
 
+let set_partial_config cont c =
+  let open Whyconf in
+  let config = set_editors cont.controller_config (get_editors c) in
+  let config = set_provers config (get_provers c) in
+  cont.controller_config <- config
+
 let session_max_tasks = ref 1
 
 let set_session_max_tasks n =
   session_max_tasks := n;
   Prove_client.set_max_running_provers n
 
+let set_session_memlimit c n =
+  let main = Whyconf.get_main c.controller_config in
+  let timelimit = Whyconf.timelimit main in
+  let run_max = Whyconf.running_provers_max main in
+  let main = Whyconf.set_limits main timelimit n run_max in
+  c.controller_config <- Whyconf.set_main c.controller_config main
+
+let set_session_timelimit c n =
+  let main = Whyconf.get_main c.controller_config in
+  let memlimit = Whyconf.memlimit main in
+  let run_max = Whyconf.running_provers_max main in
+  let main = Whyconf.set_limits main n memlimit run_max in
+  c.controller_config <- Whyconf.set_main c.controller_config main
+
 let set_session_prover_upgrade_policy c p u =
   c.controller_config <- Whyconf.set_prover_upgrade_policy c.controller_config p u
 
@@ -98,16 +116,10 @@ let load_drivers c =
   let env = c.controller_env in
   let config = c.controller_config in
   let provers = Whyconf.get_provers config in
-  Whyconf.Mprover.iter
-    (fun _ p ->
-     try
-       let d = Whyconf.load_driver (Whyconf.get_main config) env p.Whyconf.driver [] in
-       Whyconf.Hprover.add c.controller_provers p.Whyconf.prover (p,d)
-     with e ->
-       Debug.dprintf debug_call_prover
-         "[Controller_itp] error while loading driver for prover %a: %a@."
-         Whyconf.print_prover p.Whyconf.prover
-         Exn_printer.exn_printer e)
+  let main = Whyconf.get_main config in
+  Whyconf.Mprover.iter (fun _ p ->
+      let d = Whyconf.load_driver main env p.Whyconf.driver [] in
+      Whyconf.Hprover.add c.controller_provers p.Whyconf.prover (p,d))
     provers
 
 let create_controller config env ses =
@@ -178,7 +190,7 @@ module PSession = struct
     match x.tkind with
     | Session -> "", Hfile.fold (fun _ f -> n (File f)) (get_files s) []
     | File f ->
-       string_of_file_path (file_path f),
+       Pp.sprintf "%a" Sysutil.print_file_path (file_path f),
        List.fold_right (fun th -> n (Theory th)) (file_theories f) []
     | Theory th ->
        let id = theory_name th in
@@ -227,28 +239,34 @@ let print_session fmt c =
 (** reload files, associating old proof attempts and transformations
     to the new goals.  old theories and old goals for which we cannot
     find a corresponding new theory resp. old goal are kept, with
-    tasks associated to them *)
+    tasks associated to them.
+    When [hard_reload] option is true, dependencies and drivers are also
+    reloaded.
+ *)
 
-let reload_files (c : controller) ~shape_version =
+let reload_files ?(hard_reload=false) (c : controller) ~shape_version  =
   let old_ses = c.controller_session in
-  c.controller_env <- Env.create_env (Env.get_loadpath c.controller_env);
-  Whyconf.Hprover.reset c.controller_provers;
-  load_drivers c;
-  c.controller_session <- empty_session ~from:old_ses (get_dir old_ses);
+  if hard_reload then begin
+    c.controller_env <- Env.create_env (Env.get_loadpath c.controller_env);
+    Whyconf.Hprover.reset c.controller_provers;
+    load_drivers c;
+  end;
+  c.controller_session <- empty_session ~shape_version ~from:old_ses (get_dir old_ses);
   merge_files ~shape_version c.controller_env c.controller_session old_ses
 
 exception Errors_list of exn list
 
-let reload_files (c: controller) ~shape_version =
-  let errors, b1, b2 = reload_files c ~shape_version in
+let reload_files ?(hard_reload=false) (c: controller) ~shape_version =
+  let errors, b1, b2 = reload_files c ~shape_version ~hard_reload in
   match errors with
   | [] -> b1, b2
   | _ -> raise (Errors_list errors)
 
 let add_file c ?format fname =
-  let file_is_detached,theories,errors =
-    try false,(Session_itp.read_file c.controller_env ?format fname), None
-    with e -> true,[], Some e
+  let format = Env.get_format ?format fname in
+  let file_is_detached,(theories,format),errors =
+    try false,(Session_itp.read_file c.controller_env ~format fname), None
+    with e -> true,([], format), Some e
   in
   let (_ : file) = add_file_section c.controller_session fname ~file_is_detached theories format in
   errors
@@ -302,6 +320,7 @@ let prover_tasks_in_progress :
    rely on a loop on why3server's results. *)
 let prover_tasks_edited = Queue.create ()
 
+let idle_handler_running = ref false
 let timeout_handler_running = ref false
 
 
@@ -485,23 +504,29 @@ let timeout_handler () =
     done;
     Queue.transfer q prover_tasks_edited;
   end;
+  update_observer ();
+  true
+
+
+let idle_handler () =
 
   (* if the number of prover tasks is less than S.multiplier times the maximum
      number of running provers, then we heuristically decide to add
      more tasks *)
   begin
     try
-      for _i = Hashtbl.length prover_tasks_in_progress
-          to S.multiplier * !session_max_tasks do
+      if S.blocking ||
+         (Hashtbl.length prover_tasks_in_progress <
+          S.multiplier * !session_max_tasks)
+      then
         let spa = Queue.pop scheduled_proof_attempts in
         try build_prover_call spa
         with e when not (Debug.test_flag Debug.stack_trace) ->
           spa.spa_callback (InternalFailure e)
-      done
-  with Queue.Empty -> ()
+    with Queue.Empty -> idle_handler_running := false
   end;
   update_observer ();
-  true
+  !idle_handler_running
 
 let interrupt () =
   (* Interrupt provers *)
@@ -525,7 +550,14 @@ let interrupt () =
   done;
   !observer 0 0 0
 
-let run_timeout_handler () =
+let run_idle_handler () =
+  if not !idle_handler_running then
+    begin
+      idle_handler_running := true;
+      (* The prio should be at least 100. From testing, it seems that the GTK
+         tooling is using a prio of 100 or higher for the display of IDE. *)
+      S.idle ~prio:300 idle_handler;
+    end;
   if not !timeout_handler_running then
     begin
       timeout_handler_running := true;
@@ -572,8 +604,9 @@ let schedule_proof_attempt c id pr ?save_to ~limit ~callback ~notification =
       let script =
         if save_to = None then
           Opt.map (fun s ->
-                            Debug.dprintf debug_sched "Script file = %s@." s;
-                            Filename.concat (get_dir ses) s) a.proof_script
+              let s = Pp.sprintf "%a" Sysutil.print_file_path s in
+              Debug.dprintf debug_sched "Script file = %s@." s;
+              Filename.concat (get_dir ses) s) a.proof_script
         else
           save_to
       in
@@ -591,7 +624,7 @@ let schedule_proof_attempt c id pr ?save_to ~limit ~callback ~notification =
       spa_ores     = ores } in
   Queue.add spa scheduled_proof_attempts;
   callback panid Scheduled;
-  run_timeout_handler ()
+  run_idle_handler ()
 
 
 
@@ -607,14 +640,11 @@ let create_file_rel_path c pr pn =
   let th = get_encapsulating_theory session (APn pn) in
   let th_name = (Session_itp.theory_name th).Ident.id_string in
   let f = get_encapsulating_file session (ATh th) in
-  let fn = Filename.chop_extension (Session_itp.basename (file_path f)) in
+  let fn = Filename.chop_extension (Sysutil.basename (file_path f)) in
   let file = Driver.file_of_task driver fn th_name task in
   let file = Filename.concat session_dir file in
   let file = Sysutil.uniquify file in
-  let file = Sysutil.relativize_filename session_dir file in
-  match file with
-  | [f] -> f
-  | _ -> assert false
+  Sysutil.relativize_filename session_dir file
 
 let prepare_edition c ?file pn pr ~notification =
   let session = c.controller_session in
@@ -648,7 +678,7 @@ let prepare_edition c ?file pn pr ~notification =
   let file = Opt.get pa.proof_script in
   let old_res = pa.proof_state in
   let session_dir = Session_itp.get_dir session in
-  let file = Filename.concat session_dir file in
+  let file = Sysutil.system_dependent_absolute_path session_dir file in
   let old =
     if Sys.file_exists file
     then
@@ -716,7 +746,7 @@ let schedule_edition c id pr ~callback ~notification =
   let call = Call_provers.call_editor ~command:editor file in
   callback panid Running;
   Queue.add (callback panid,call,old_res) prover_tasks_edited;
-  run_timeout_handler ()
+  run_idle_handler ()
 
 exception TransAlreadyExists of string * string
 exception GoalNodeDetached of proofNodeID
@@ -743,21 +773,14 @@ let schedule_transformation c id name args ~callback ~notification =
         callback (TSdone tid)
       with
       | NoProgress ->
-         (* if result is same as input task, consider it as a failure *)
-         callback (TSfailed (id, NoProgress))
-      | (Arg_trans _ | Arg_trans_decl _ | Arg_trans_missing _
-        | Arg_trans_term _ | Arg_trans_term2 _ | Arg_trans_term3 _
-        | Arg_trans_pattern _ | Arg_trans_type _ | Arg_bad_hypothesis _
-        | Cannot_infer_type _ | Unnecessary_terms _ | Parse_error _
-        | Arg_expected _ | Arg_theory_not_found _ | Arg_expected_none _
-        | Arg_qid_not_found _ | Arg_pr_not_found _ | Arg_error _
-        | Arg_parse_type_error _ | Unnecessary_arguments _
-        | Reflection.NoReification ) as e ->
+          (* if result is same as input task, consider it as a failure *)
+          callback (TSfailed (id, NoProgress))
+      | e when not (is_fatal e) ->
           callback (TSfailed (id, e))
       | e when not (Debug.test_flag Debug.stack_trace) ->
           (* "@[Exception raised in Session_itp.apply_trans_to_goal %s:@ %a@.@]"
           name Exn_printer.exn_printer e; TODO *)
-        callback (TSfatal (id, e))
+          callback (TSfatal (id, e))
     end;
     false
   in
@@ -779,6 +802,9 @@ let run_strategy_on_goal
     else
       match Array.get strat pc with
       | Icall_prover(p,timelimit,memlimit) ->
+         let main = Whyconf.get_main c.controller_config in
+         let timelimit = Opt.get_def (Whyconf.timelimit main) timelimit in
+         let memlimit = Opt.get_def (Whyconf.memlimit main) memlimit in
          let callback panid res =
            callback_pa panid res;
            match res with
@@ -875,7 +901,26 @@ let clean c ~removed nid =
     if do_remove then
       remove_subtree ~notification ~removed c any
   in
+  match nid with
+  | Some nid ->
+      Session_itp.fold_all_any s clean_aux () nid
+  | None ->
+      Session_itp.fold_all_session s clean_aux ()
 
+let reset_proofs c ~removed ~notification nid =
+  let s = c.controller_session in
+  (* This function is applied on leafs first for the case of removes *)
+  let clean_aux () any =
+    let do_remove =
+      Session_itp.is_detached s any ||
+      match any with
+      | APa _ -> true
+      | ATn _ -> true
+      | _ -> false
+    in
+    if do_remove then
+      remove_subtree ~notification ~removed c any
+  in
   match nid with
   | Some nid ->
       Session_itp.fold_all_any s clean_aux () nid
@@ -1035,8 +1080,8 @@ let print_report fmt (r: report) =
   match r with
   | Result (new_r, old_r) ->
     Format.fprintf fmt "new_result = %a, old_result = %a@."
-      Call_provers.print_prover_result new_r
-      Call_provers.print_prover_result old_r
+      (Call_provers.print_prover_result ~json_model:false) new_r
+      (Call_provers.print_prover_result ~json_model:false) old_r
   | CallFailed e ->
     Format.fprintf fmt "Callfailed %a@." Exn_printer.exn_printer e
   | Replay_interrupted ->
@@ -1047,7 +1092,7 @@ let print_report fmt (r: report) =
     Format.fprintf fmt "No edited file@."
   | No_former_result new_r ->
     Format.fprintf fmt "new_result = %a, no former result@."
-      Call_provers.print_prover_result new_r
+      (Call_provers.print_prover_result ~json_model:false) new_r
 
 (* TODO to be removed when we have a better way to print *)
 let replay_print fmt (lr: (proofNodeID * Whyconf.prover * Call_provers.resource_limit * report) list) =
@@ -1167,9 +1212,15 @@ let create_rem_list =
     if Buffer.length b > 0 then Buffer.add_char b ',';
     Buffer.add_string b (Pp.string_of pr id)
   in
-  let remove_ts ts = add Pretty.print_ts ts in
-  let remove_ls ls = add Pretty.print_ls ls in
-  let remove_pr pr = add Pretty.print_pr pr in
+  let module P = (val Pretty.create
+      rem.Eliminate_definition.rem_nt.Trans.printer
+      rem.Eliminate_definition.rem_nt.Trans.aprinter
+      rem.Eliminate_definition.rem_nt.Trans.printer
+      rem.Eliminate_definition.rem_nt.Trans.printer
+      false) in
+  let remove_ts ts = add P.print_ts ts in
+  let remove_ls ls = add P.print_ls ls in
+  let remove_pr pr = add P.print_pr pr in
   Ty.Sts.iter remove_ts rem.Eliminate_definition.rem_ts;
   Term.Sls.iter remove_ls rem.Eliminate_definition.rem_ls;
   Decl.Spr.iter remove_pr rem.Eliminate_definition.rem_pr;
@@ -1227,7 +1278,7 @@ let bisect_proof_attempt ~callback_tr ~callback_pa ~notification ~removed c pa_i
                   | Done res ->
                      assert (res.Call_provers.pr_answer = Call_provers.Valid);
                      Debug.dprintf debug "Bisecting: %a.@."
-                                   Call_provers.print_prover_result res
+                       (Call_provers.print_prover_result ~json_model:false) res
                   end
                 in
                 schedule_proof_attempt ?save_to:None c pn prover ~limit ~callback ~notification
@@ -1244,7 +1295,7 @@ let bisect_proof_attempt ~callback_tr ~callback_pa ~notification ~removed c pa_i
        2) on the generated sub-goal, run the prover with some callback
        3) the callback should :
           compute (next_iter success_value)
-          if result is done, do nothing more
+          if result is done, remove the transformation and go to bisect_end
           if result is some new rem, remove the previous transformation
            and recursively call bisect_step
      *)
@@ -1302,9 +1353,9 @@ later on. We do has if proof fails. *)
                    Call_provers.print_prover_answer res.Call_provers.pr_answer;
                  let b = res.Call_provers.pr_answer = Call_provers.Valid in
                  if b then set_timelimit res;
+                 Session_itp.remove_subtree ~notification ~removed ses (Session_itp.ATn trid);
                  match kont b with
                  | Eliminate_definition.BSstep (rem,kont) ->
-                    Session_itp.remove_subtree ~notification ~removed ses (Session_itp.ATn trid);
                     bisect_step rem kont
                  | Eliminate_definition.BSdone rem ->
                     bisect_end rem
