@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -67,6 +67,7 @@ type color =
   | Error_color
   | Error_line_color
   | Error_font_color
+  | Search_color
 
 type update_info =
   | Proved of bool
@@ -98,14 +99,26 @@ type notification =
   (* an informative message, can be an error message *)
   | Dead         of string
   (* server exited *)
-  | Task         of node_ID * string * (Loc.position * color) list
-  (* the node_ID's task together with information that allows to color the
-     source code corresponding to different part of the task (premise, goal,
-     etc) *)
-  | File_contents of string * string
-  (* File_contents (filename, contents) *)
-  | Source_and_ce of string * (Loc.position * color) list
-  (* Source interleaved with counterexamples: contents and list color loc *)
+  | Task         of node_ID * string * (Loc.position * color) list * Loc.position option * Env.fformat
+  (* [n, s, list_loc, goal_loc, lang] with
+     - [n] the node_ID's task,
+     - [s] the task to be displayed
+     - [list_loc] a list of location to color the source,
+     - [goal_loc] the location of the goal,
+     - [lang] the language to load in Why3ide for syntax coloring
+  *)
+  | File_contents of string * string * Env.fformat * bool
+  (* File_contents (filename, contents, format, read_only) *)
+  | Source_and_ce of string * (Loc.position * color) list * Loc.position option * Env.fformat
+  (* Source interleaved with counterexamples: contents, list color loc,
+     loc of the goal, format of the source *)
+  | Ident_notif_loc of Loc.position
+  (* Answer the position where an ident is defined *)
+
+type next_unproved_node_strat =
+  | Prev
+  | Next
+  | Clever
 
 type ide_request =
   | Command_req             of node_ID * string
@@ -117,13 +130,15 @@ type ide_request =
   | Remove_subtree          of node_ID
   | Copy_paste              of node_ID * node_ID
   | Save_file_req           of string * string
-  | Get_first_unproven_node of node_ID
+  | Get_first_unproven_node of next_unproved_node_strat * node_ID
+  | Find_ident_req          of Loc.position
   | Unfocus_req
   | Save_req
   | Reload_req
   | Check_need_saving_req
   | Exit_req
   | Interrupt_req
+  | Reset_proofs_req
   | Get_global_infos
 
 
@@ -140,7 +155,8 @@ let print_request fmt r =
      fprintf fmt "set prover policy %a -> %a" Whyconf.print_prover p1
              Whyconf.print_prover_upgrade_policy p2
   | Get_file_contents _f            -> fprintf fmt "get file contents"
-  | Get_first_unproven_node _nid    -> fprintf fmt "get first unproven node"
+  | Get_first_unproven_node (_,_nid)-> fprintf fmt "get first unproven node"
+  | Find_ident_req _                -> fprintf fmt "find ident"
   | Get_task(nid,b,loc)             -> fprintf fmt "get task(%d,%b,%b)" nid b loc
   | Remove_subtree _nid             -> fprintf fmt "remove subtree"
   | Copy_paste _                    -> fprintf fmt "copy paste"
@@ -151,6 +167,7 @@ let print_request fmt r =
   | Check_need_saving_req           -> fprintf fmt "check need saving"
   | Exit_req                        -> fprintf fmt "exit"
   | Interrupt_req                   -> fprintf fmt "interrupt"
+  | Reset_proofs_req                -> fprintf fmt "reset proofs"
   | Get_global_infos                -> fprintf fmt "get_global_infos"
 
 let print_msg fmt m =
@@ -201,8 +218,11 @@ let print_notify fmt n =
   | Message msg                       ->
       print_msg fmt msg
   | Dead s                            -> fprintf fmt "dead :%s" s
-  | File_contents (f, _s)             -> fprintf fmt "file contents %s" f
-  | Source_and_ce (_s, _list_loc)     -> fprintf fmt "source and ce"
-  | Task (ni, _s, list_loc)           ->
-      fprintf fmt "task for node_ID %d which contains a list of %d locations"
-              ni (List.length list_loc) (* print_list_loc list_loc *)
+  | File_contents (f, _s, _, _)       -> fprintf fmt "file contents %s" f
+  | Source_and_ce (_, _list_loc, _gl, _) -> fprintf fmt "source and ce"
+  | Task (ni, _s, list_loc, g_loc, _lang) ->
+      fprintf fmt "task for node_ID %d which contains a list of %d locations. Goal_location = %a"
+        ni (List.length list_loc)
+        (Pp.print_option print_loc) g_loc (* print_list_loc list_loc *)
+  | Ident_notif_loc loc               ->
+      fprintf fmt "ident notification %a" Pretty.print_loc loc
