@@ -1475,8 +1475,11 @@ end
     | Some (APa panid) ->
       let pan = Session_itp.get_proof_attempt_node session panid in
       let filter_prover =
+        let altern =
+          if pan.prover.Whyconf.prover_altern = "" then "counterexamples"
+          else pan.prover.Whyconf.prover_altern ^ "+" ^ "counterexamples" in
         Whyconf.mk_filter_prover ~version:pan.prover.Whyconf.prover_version
-          ~altern:"counterexamples" pan.prover.Whyconf.prover_name
+            ~altern pan.prover.Whyconf.prover_name
       in
       begin match Whyconf.filter_one_prover config filter_prover with
       | config_prover ->
