@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -21,6 +21,9 @@ let has_extension f =
 let get_session_dir ~allow_mkdir files =
   if Queue.is_empty files then invalid_arg "no files given";
   let first = Queue.pop files in
+  (* The session should always return an absolute path. It will be used for
+     relative calculus of every other paths *)
+  let first = Sysutil.concat (Sys.getcwd ()) first in
   let dir =
     if Sys.file_exists first then
       if Sys.is_directory first then
@@ -54,7 +57,6 @@ let get_session_dir ~allow_mkdir files =
     end;
   dir
 
-
 (******************)
 (* Simple queries *)
 (******************)
@@ -160,8 +162,8 @@ let print_id s tables =
   (* Different constructs are printed differently *)
   match d.Decl.d_node, table_id with
   | Decl.Dind (_, il), Args_wrapper.Tsprsymbol pr ->
-      print_constr_string ~print_term:P.print_term ~print_pr:P.print_pr il pr
-  | _ -> Pp.string_of P.print_decl d
+      (d, print_constr_string ~print_term:P.print_term ~print_pr:P.print_pr il pr)
+  | _ -> (d, Pp.string_of P.print_decl d)
 
 (* searching ids in declarations *)
 
@@ -230,9 +232,16 @@ let search ~search_both s tables =
       let module P = (val Pretty.create pr apr pr pr false) in
       Pp.string_of (Pp.print_list Pp.newline2 P.print_decl) l
 
+let locate_id _cont task args =
+  match args with
+  | [s] -> let (d, _) = print_id s task in
+      let id = Ident.Sid.choose d.Decl.d_news in
+      id.Ident.id_loc
+  | _ -> raise Number_of_arguments
+
 let print_id _cont task args =
   match args with
-  | [s] -> print_id s task
+  | [s] -> let (_, s) = print_id s task in s
   | _ -> raise Number_of_arguments
 
 let search_id ~search_both _cont task args =
@@ -244,7 +253,6 @@ type query =
   | Qnotask of (Controller_itp.controller -> string list -> string)
   | Qtask of (Controller_itp.controller -> Trans.naming_table -> string list -> string)
 
-
 let help_on_queries fmt commands =
   let l = Hstr.fold (fun c (h,_) acc -> (c,h)::acc) commands [] in
   let l = List.sort sort_pair l in
@@ -266,12 +274,6 @@ let return_prover name config =
   end else
     Some (snd (Whyconf.Mprover.choose provers))
 
-let session_timelimit = ref 2
-let session_memlimit = ref 1000
-let set_session_timelimit n = session_timelimit := n
-let set_session_memlimit n = session_memlimit := n
-
-
 type command_prover =
   | Bad_Arguments of Whyconf.prover
   | Not_Prover
@@ -289,14 +291,18 @@ let parse_prover_name config name args : command_prover =
         if (List.length args > 2) then Bad_Arguments prover else
         match args with
         | [] ->
+            let limit_time = Whyconf.timelimit (Whyconf.get_main config) in
+            let limit_mem = Whyconf.memlimit (Whyconf.get_main config) in
             let default_limit = Call_provers.{empty_limit with
-                                              limit_time = !session_timelimit;
-                                              limit_mem = !session_memlimit} in
+                                              limit_time = limit_time;
+                                              limit_mem = limit_mem} in
             Prover (prover_config, default_limit)
-        | [timeout] -> Prover (prover_config,
-                               Call_provers.{empty_limit with
-                                             limit_time = int_of_string timeout;
-                                             limit_mem = !session_memlimit})
+        | [timeout] ->
+            let limit_mem = Whyconf.memlimit (Whyconf.get_main config) in
+            Prover (prover_config,
+                    Call_provers.{empty_limit with
+                                  limit_time = int_of_string timeout;
+                                  limit_mem = limit_mem})
         | [timeout; oom ] ->
             Prover (prover_config, Call_provers.{empty_limit with
                                                  limit_time = int_of_string timeout;
@@ -325,17 +331,25 @@ let split_args s =
   in
   let push_char c = Buffer.add_char b c in
   let state = ref 0 in
+  (* state 0 : normal mode (no '"' and no ",")
+     state 1 : encountered one '"' waiting for another one to go back to 0
+     state 2 : encountered one ',': immediately followed whitespace are not
+               argument separations
+     par_depth : number of nested opened parenthesis *)
   for i = 0 to String.length s - 1 do
     let c = s.[i] in
     match !state, c with
     | 0,' ' -> if !par_depth > 0 then push_char c else push_arg ()
+    | 0,',' -> push_char c; if !par_depth > 0 then () else state := 2
     | 0,'(' -> incr par_depth; push_char c
     | 0,')' -> decr par_depth; push_char c
     | 0,'"' -> state := 1; if !par_depth > 0 then push_char c
-    | 0,_ -> push_char c
+    | 0,_   -> push_char c
     | 1,'"' -> state := 0; if !par_depth > 0 then push_char c
-    | 1,_ -> push_char c
-    | _ -> assert false
+    | 1,_   -> push_char c
+    | 2,' ' -> push_char c
+    | 2,_   -> push_char c; state := 0
+    | _     -> assert false
   done;
   push_arg ();
   match List.rev !args with
@@ -354,6 +368,7 @@ type command =
   | Mark_Obsolete
   | Focus_req
   | Unfocus_req
+  | Locate       of string
   | Help_message of string
   | Query        of string
   | QError       of string
@@ -445,6 +460,11 @@ let interp commands_table cont id s =
                | Some _ -> Strategies cmd
              else
                match cmd, args with
+               | "locate", args ->
+                   begin match args with
+                     | [id] -> Locate id
+                     | _ -> QError "locate expects only one argument"
+                   end
                | "edit", _ ->
                   begin
                     match id with
@@ -559,3 +579,56 @@ let get_first_unproven_goal_around
       | hd :: _tl -> Some hd
     end
   | hd :: _tl  -> Some hd
+
+let find_next_unproved_goal ~proved ~children ~get_parent ~is_pa node =
+  let rec find_next_in_above node =
+    match get_parent node with
+    | None -> None
+    | Some parent when proved parent ->
+        find_next_in_above parent
+    | Some parent ->
+        let brothers_node = children parent in
+        let _before_nodes, after_nodes = split_list brothers_node node in
+        let after_nodes = List.filter
+            (fun x -> not (proved x) && not (is_pa x)) after_nodes in
+        if after_nodes = [] then
+          find_next_in_above parent
+        else
+          Some (List.hd after_nodes)
+  in
+  let childrens = children node in
+  let l = List.filter (fun x -> not (proved x) && not (is_pa x)) childrens in
+  if l <> [] then
+    Some (List.hd l)
+  else
+    find_next_in_above node
+
+let find_previous_unproved_goal ~proved ~children ~get_parent node =
+  let original_node = node in
+  let rec find_previous_in_above node =
+    match get_parent node with
+    | None -> if node = original_node then None else Some node
+    | Some parent when proved parent ->
+        find_previous_in_above parent
+    | Some parent ->
+        if not (proved node) && node <> original_node then
+          Some node
+        else
+          let brothers_node = children parent in
+          let before_nodes, _after_nodes = split_list brothers_node node in
+          let before_nodes =
+            Lists.rev_filter (fun x -> not (proved x)) before_nodes in
+          begin match before_nodes with
+            | [] -> find_previous_in_above parent
+            | t :: _ -> Some t
+          end
+  in
+  find_previous_in_above node
+
+let get_next_with_strategy ~st
+    ~always_send ~proved ~children ~get_parent ~is_goal ~is_pa node =
+  let open Itp_communication in
+  match st with
+  | Next -> find_next_unproved_goal ~proved ~children ~get_parent ~is_pa node
+  | Prev -> find_previous_unproved_goal ~proved ~children ~get_parent node
+  | Clever -> get_first_unproven_goal_around ~always_send ~proved ~children ~get_parent ~is_goal ~is_pa node
