@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,7 +9,10 @@
 (*                                                                  *)
 (********************************************************************)
 
+open Wstdlib
 open Term
+open Number
+open Constant
 
 (*******************************)
 (*          explanations       *)
@@ -168,67 +171,120 @@ let store_id s i =
           id
 *)
 
+module Common = struct
+
+let sign fmt n =
+  if n then Format.pp_print_char fmt '-'
+
+let const_v1 fmt c =
+  match c with
+  | ConstInt { il_kind=k; il_int=i } ->
+      sign fmt (BigInt.lt i BigInt.zero);
+      let i = BigInt.abs i in
+      begin match k with
+      | ILitUnk | ILitDec -> Format.pp_print_string fmt (BigInt.to_string i)
+      | ILitHex -> Format.fprintf fmt "0x%a" (Number.print_in_base 16 None) i
+      | ILitOct -> Format.fprintf fmt "0o%a" (Number.print_in_base 8 None) i
+      | ILitBin -> Format.fprintf fmt "0b%a" (Number.print_in_base 2 None) i
+      end
+  | ConstReal { rl_kind=k; rl_real={ rv_sig=i; rv_pow2=p2; rv_pow5=p5 } } ->
+      (* not the actual encoding *)
+      sign fmt (BigInt.lt i BigInt.zero);
+      let i = BigInt.abs i in
+      begin match k with
+      | RLitUnk | RLitDec _ ->
+          Format.fprintf fmt "%s.0e%sp%s"
+            (BigInt.to_string i) (BigInt.to_string p5) (BigInt.to_string p2)
+      | RLitHex _ ->
+          Format.fprintf fmt "%a.0p%se%s"
+            (Number.print_in_base 16 None) i (BigInt.to_string p2) (BigInt.to_string p5)
+      end
+  | ConstStr c ->
+      print_string_def fmt c
+
+let const_v2 fmt c =
+  match c with
+  | ConstInt { il_kind=_; il_int=i } ->
+      Format.pp_print_string fmt (BigInt.to_string i)
+  | ConstReal { rl_kind=_; rl_real={ rv_sig=i; rv_pow2=p2; rv_pow5=p5 } } ->
+      Format.fprintf fmt "%s.p%se%s"
+        (BigInt.to_string i) (BigInt.to_string p2) (BigInt.to_string p5)
+  | ConstStr c ->
+      print_string_def fmt c
+
+end
+
 (* {2 Shapes} *)
 
 type shape = string
 
-let string_of_shape s = s
-(*
-  try
-  let l = String.length s in
-  let r = Buffer.create l in
-  let i = ref 0 in
-  while !i < l do
-  match String.get s !i with
-    | '(' ->
-      Buffer.add_char r '(';
-      incr i;
-      Buffer.add_string r (store_id s i);
-      Buffer.add_char r ')'
-    | c -> Buffer.add_char r c; incr i
-  done;
-  Buffer.contents r
-  with e ->
-    Format.eprintf "Error while reading shape [%s]@." s;
-    raise e
-*)
+type bound_shape = int list
 
-let shape_of_string s = s
-(*
-  try
-  let l = String.length s in
-  let r = Buffer.create l in
-  let i = ref 0 in
-  while !i < l do
-  match String.get s !i with
-    | '(' -> incr i; Buffer.add_string r (get_id s i)
-    | c -> Buffer.add_char r c; incr i
-  done;
-  Buffer.contents r
-  with e ->
-    Format.eprintf "Error while reading shape [%s]@." s;
-    raise e
- *)
+let current_shape_version = 6
 
-(* tests
-let _ = reset_dict () ; shape_of_string "a(b)cde(0)"
-let _ = reset_dict () ; shape_of_string "a(bc)d(e)f(1)g(0)h"
-let _ = reset_dict () ; shape_of_string "(abc)(def)(1)(0)(1)"
-let _ = reset_dict () ; shape_of_string "(abcde)(fghij)(1)(0)(1)"
-*)
+type shape_version = SV1 | SV2 | SV3 | SV4 | SV5
 
-let equal_shape (x:string) y = x = y
-(* unused
-let print_shape fmt s = Format.pp_print_string fmt (string_of_shape s)
-*)
+let int_to_shape_version n =
+  match n with
+  | 1 -> SV1 | 2 -> SV2 | 3 | 4 -> SV3 | 5 -> SV4 | 6 -> SV5
+  | _ -> assert false
+
+let is_bound_shape_version v =
+  match int_to_shape_version v with
+  | SV1 | SV2 | SV3 | SV4 -> false
+  | SV5 -> true
+
+type shape_v =
+  | Old_shape of shape
+  | Bound_shape of bound_shape
+
+let string_of_shape sv =
+  match sv with
+  | Old_shape s -> s
+  | Bound_shape l ->
+      Format.asprintf "%a" (Pp.print_list (Pp.constant_string "H") Pp.int) l
+
+let shape_of_string =
+  let cache = ref [] in
+  fun ~version s ->
+  if is_bound_shape_version version then
+    let s = Strings.rev_split 'H' s in
+    (* most consecutive shapes have very long common suffixes,
+       so we look for some sharing with the previously computed shape
+       in order to reduce the memory footprint of large sessions *)
+    let c, lc =
+      let lc = List.length !cache in
+      let ls = List.length s in
+      (* this avoids reverting a uselessly long list below *)
+      if lc > ls then Lists.chop (lc - ls) !cache, ls
+      else !cache, lc in
+    let rec aux c s n =
+      match c, s with
+      | hc :: tc, hs :: ts ->
+          begin match int_of_string hs with
+          | x -> if x = hc then aux tc ts (n + 1) else (n, s)
+          | exception _ -> aux c ts n
+          end
+      | _, _ -> (n, s) in
+    let (common, remaining) = aux (List.rev c) s 0 in
+    let common = Lists.chop (lc - common) c in
+    let s = List.fold_left (fun acc x ->
+        try int_of_string x :: acc with
+        | _ -> acc) common remaining in
+    cache := s;
+    Bound_shape s
+  else
+    Old_shape s
+
+let empty_shape = ""
+
+let empty_bound_shape = []
 
 let debug = Debug.register_info_flag "session_pairing"
   ~desc:"Print@ debugging@ messages@ about@ reconstruction@ of@ \
          session@ trees@ after@ modification@ of@ source@ files."
 
-let current_shape_version = 5
-
-type shape_version = SV1 | SV2 | SV3 | SV4
+module Shape = struct
 
 (* similarity code of terms, or of "shapes"
 
@@ -263,15 +319,25 @@ exception ShapeTooLong
 
 let shape_buffer = Buffer.create 17
 
+let check () =
+  if Buffer.length shape_buffer >= 1024 then raise ShapeTooLong
+
 let push s =
   Buffer.add_string shape_buffer s;
-  if Buffer.length shape_buffer >= 1024 then raise ShapeTooLong
+  check ()
 
 let pushc c =
   Buffer.add_char shape_buffer c;
-  if Buffer.length shape_buffer >= 1024 then raise ShapeTooLong
+  check ()
 
-let ident h id =
+(* Remove infix and mixfix. (prefix should be rare enough) *)
+let decoded pr id =
+  Ident.(match sn_decode id.id_string with
+  | SNword _ -> Ident.id_unique pr id
+  | SNinfix s | SNtight s | SNprefix s | SNget s | SNset s
+  | SNupdate s | SNcut s | SNlcut s | SNrcut s -> s)
+
+let ident_v4 h id =
   let x =
     try Ident.Mid.find id !h
     with Not_found ->
@@ -280,6 +346,15 @@ let ident h id =
   in
   push x
 
+let ident_v5 pr h id =
+  let x =
+    try Ident.Mid.find id !h
+    with Not_found ->
+      let s = decoded pr id in
+      h := Ident.Mid.add id s !h; s
+  in
+  push x
+
 let vs_rename_alpha c h vs =
   incr c;
   let s = string_of_int !c in
@@ -299,75 +374,65 @@ let id_string_shape id = push id
 let ident_shape id = id_string_shape id.Ident.id_string
 *)
 
-open Number
+let const_shape v c =
+  let fmt = Format.formatter_of_buffer shape_buffer in
+  begin match v with
+  | SV1 | SV2 | SV3 | SV4 -> Common.const_v1 fmt c
+  | SV5 -> Common.const_v2 fmt c
+  end;
+  Format.pp_print_flush fmt ();
+  check ()
 
-let integer_const_shape = function
-  | IConstRaw i -> push (BigInt.to_string i)
-  | IConstDec s -> push s
-  | IConstHex s -> push "0x"; push s
-  | IConstOct s -> push "0o"; push s
-  | IConstBin s -> push "0b"; push s
+let ident_shape v pr m id =
+  match v with
+  | SV1 | SV2 | SV3 | SV4 -> ident_v4 m id
+  | SV5 -> ident_v5 pr m id
 
-let real_const_shape = function
-  | RConstDec (i,f,None)   -> push i; push "."; push f
-  | RConstDec (i,f,Some e) -> push i; push "."; push f; push "e"; push e
-  | RConstHex (i,f,Some e) -> push "0x"; push i; push "."; push f; push "p"; push e
-  | RConstHex (i,f,None)   -> push "0x"; push i; push "."; push f
-
-let sign_shape is_negative =
-  if is_negative then pushc '-'
-
-let const_shape c =
-  match c with
-  | ConstInt c -> sign_shape c.ic_negative; integer_const_shape c.ic_abs
-  | ConstReal c -> sign_shape c.rc_negative; real_const_shape c.rc_abs
-
-
-let rec pat_shape c m p : 'a =
+let rec pat_shape ~version pr c m p : 'a =
   match p.pat_node with
     | Pwild -> pushc tag_wild
     | Pvar _ -> pushc tag_var
     | Papp (f, l) ->
-       pushc tag_app;
-       ident m f.ls_name;
-       List.iter (pat_shape c m) l
-    | Pas (p, _) -> pat_shape c m p; pushc tag_as
+        pushc tag_app;
+        ident_shape version pr m f.ls_name;
+        List.iter (pat_shape ~version pr c m) l
+    | Pas (p, _) -> pat_shape ~version pr c m p; pushc tag_as
     | Por (p, q) ->
-       pat_shape c m q; pushc tag_or; pat_shape c m p
+       pat_shape ~version pr c m q; pushc tag_or; pat_shape ~version pr c m p
 
-let rec t_shape ~version c m t =
-  let fn = t_shape ~version c m in
+let rec t_shape ~version pr c m t =
+  let fn = t_shape ~version pr c m in
   match t.t_node with
-    | Tconst c -> pushc tag_const; const_shape c
-    | Tvar v -> pushc tag_var; ident m v.vs_name
+    | Tconst c -> pushc tag_const; const_shape version c
+    | Tvar v -> pushc tag_var; ident_shape version pr m v.vs_name
     | Tapp (s,l) ->
-       pushc tag_app;
-       ident m s.ls_name;
-       List.iter fn l
+        pushc tag_app;
+        ident_shape version pr m s.ls_name;
+        List.iter fn l
     | Tif (f,t1,t2) ->
       begin match version with
       | SV1 | SV2 -> pushc tag_if; fn f; fn t1; fn t2
-      | SV3 | SV4 -> pushc tag_if; fn t2; fn t1; fn f
+      | SV3 | SV4 | SV5 -> pushc tag_if; fn t2; fn t1; fn f
       end
     | Tcase (t1,bl) ->
         let br_shape b =
           let p,t2 = t_open_branch b in
           match version with
           | SV1 | SV2 ->
-            pat_shape c m p;
+            pat_shape ~version pr c m p;
             pat_rename_alpha c m p;
-            t_shape ~version c m t2
-          | SV3 | SV4 ->
+            fn t2
+          | SV3 | SV4 | SV5 ->
             pat_rename_alpha c m p;
-            t_shape ~version c m t2;
-            pat_shape c m p
+            fn t2;
+            pat_shape ~version pr c m p
         in
         begin match version with
         | SV1 | SV2 ->
                  pushc tag_case;
                  fn t1;
                  List.iter br_shape bl
-        | SV3 | SV4 ->
+        | SV3 | SV4 | SV5 ->
            pushc tag_case;
            List.iter br_shape bl;
            fn t1
@@ -376,21 +441,16 @@ let rec t_shape ~version c m t =
         pushc tag_eps;
         let u,f = t_open_bound b in
         vs_rename_alpha c m u;
-        t_shape ~version c m f
+        fn f
     | Tquant (q,b) ->
         let vl,triggers,f1 = t_open_quant b in
         vl_rename_alpha c m vl;
         (* argument first, intentionally, to give more weight on A in
            forall x,A *)
-        t_shape ~version c m f1;
+        fn f1;
         let hq = match q with Tforall -> tag_forall | Texists -> tag_exists in
         pushc hq;
-        List.iter
-          (fun trigger ->
-             List.iter
-               (fun t -> t_shape ~version c m t)
-               trigger)
-          triggers
+        List.iter (fun trigger -> List.iter fn trigger) triggers
     | Tbinop (o,f,g) ->
        (* g first, intentionally, to give more weight on B in A->B *)
        fn g;
@@ -408,21 +468,22 @@ let rec t_shape ~version c m t =
         begin
           match version with
             | SV1 ->
-               pushc tag_let; fn t1; t_shape ~version c m t2
-            | SV2 | SV3 | SV4 ->
-                     (* t2 first, intentionally *)
-                     t_shape ~version c m t2; pushc tag_let; fn t1
+                pushc tag_let; fn t1; fn t2
+            | SV2 | SV3 | SV4 | SV5 ->
+                (* t2 first, intentionally *)
+                fn t2; pushc tag_let; fn t1
         end
     | Tnot f ->
       begin match version with
       | SV1 | SV2 -> fn f; pushc tag_not
-      | SV3 | SV4 -> pushc tag_not; fn f
+      | SV3 | SV4 | SV5 -> pushc tag_not; fn f
       end
     | Ttrue -> pushc tag_true
     | Tfalse -> pushc tag_false
 
 let t_shape_task ~version ~expl t =
   Buffer.clear shape_buffer;
+  let pr = Ident.create_ident_printer [] in
   let f = Task.task_goal_fmla t in
   let c = ref (-1) in
   let m = ref Ident.Mid.empty in
@@ -431,23 +492,20 @@ let t_shape_task ~version ~expl t =
       (* expl *)
       begin match version with
       | SV1 | SV2 -> ()
-      | SV3 | SV4 -> push expl end;
+      | SV3 | SV4 | SV5 -> push expl end;
       (* goal shape *)
-      t_shape ~version c m f;
-      (* introduced premises shape *)
+      t_shape ~version pr c m f;
+      (* All declarations shape *)
       begin match version with
       | SV1 | SV2 | SV3 -> ()
-      | SV4 ->
+      | SV4 | SV5 ->
          let open Decl in
-         let introduced id = Ident.Sattr.mem
-                               Introduction.intro_attr
-                               id.Ident.id_attrs in
          let do_td td = match td.Theory.td_node with
            | Theory.Decl d ->
               begin match d.d_node with
               | Dparam _ls -> ()
-              | Dprop (_, pr, f) when introduced pr.pr_name ->
-                 t_shape ~version c m f
+              | Dprop (_, _pr, f) ->
+                  t_shape ~version pr c m f
               | _ -> ()
               end
            | _ -> () in
@@ -458,19 +516,116 @@ let t_shape_task ~version ~expl t =
   in
   Buffer.contents shape_buffer
 
+end
+
+module Gshape = struct
+
+  open Wstdlib
+
+  (* [hyp_shape] associate a shape to its index in the file;
+     [index_sh] associate an index to a shape;
+     [n] is the last shape variable *)
+  type gshape = { mutable hyp_shape: int Mstr.t;
+                  mutable index_sh: string Mint.t;
+                  mutable n: int }
+
+  let create () =
+    { hyp_shape = Mstr.empty;
+      index_sh = Mint.empty;
+      n = 0 }
+
+  let copy from_s to_s =
+    to_s.hyp_shape <- from_s.hyp_shape;
+    to_s.index_sh <- from_s.index_sh;
+    to_s.n <- from_s.n
+
+  let clear_gs gs =
+    gs.hyp_shape <- Mstr.empty;
+    gs.index_sh <- Mint.empty;
+    gs.n <- 0
+
+  let add_shape gs s =
+    try Mstr.find s gs.hyp_shape with
+    | Not_found ->
+        gs.hyp_shape <- Mstr.add s gs.n gs.hyp_shape;
+        gs.index_sh  <- Mint.add gs.n s gs.index_sh;
+        gs.n <- gs.n + 1;
+        gs.n - 1
+
+  let add_and_prepend gs current_shape s : unit =
+    (* Do not save empty: could be confused with end of assingments (TODO to be improved)  *)
+    if s = "" then () else
+      current_shape := add_shape gs s :: !current_shape
+
+  let add_shape_g gs s =
+    let (_: int) = add_shape gs s in ()
+
+  let write_shape_to_file s csh =
+    Mint.iter (fun _ s -> Compress.Compress_z.output_string csh s;
+                Compress.Compress_z.output_string csh "\n")
+      s.index_sh
+
+  let goal_and_expl_shapes (gs: gshape) li =
+    try
+      match li with
+      | expl :: goal :: _ ->
+          Mint.find expl gs.index_sh ^ Mint.find goal gs.index_sh
+      | _ ->
+          ""
+    with Not_found -> ""
+
+  let t_shape_task_v5 ~expl gs t =
+    let current_shape = ref [] in
+    Buffer.clear Shape.shape_buffer;
+    let pr = Ident.create_ident_printer [] in
+    let c = ref (-1) in
+    let m = ref Ident.Mid.empty in
+    (* All declarations shape *)
+    let local_decls =
+      let ut = Task.used_symbols (Task.used_theories t) in
+      Task.local_decls t ut in
+    let open Decl in
+    let do_d d =
+      match d.d_node with
+      | Dparam _ls -> ()
+      | Dprop (_, _pr, f) ->
+          let sh =
+            (try Shape.t_shape ~version:SV5 pr c m f with Shape.ShapeTooLong -> ());
+            Buffer.contents Shape.shape_buffer in
+          Buffer.clear Shape.shape_buffer;
+          add_and_prepend gs current_shape sh
+      | _ -> () in
+    List.iter do_d local_decls;
+    (* TODO don't save empty shapes as \n as it could be confused with end of assignments *)
+    let expl = if expl = "" then "empty_shape" else expl in
+    add_and_prepend gs current_shape expl;
+    (* The order should be [|shape|] :: [|goal|] :: [|rest of decl|] *)
+    !current_shape
+
+  let t_bound_shape_task gs ~version ~expl t =
+    if is_bound_shape_version version then
+      t_shape_task_v5 gs ~expl t
+    else
+      assert false
+
+  let empty_bshape = []
+
+end
+
 (*
 let time = ref 0.0
  *)
 
-let t_shape_task ?(version=current_shape_version) ~expl t =
-  let version = match version with
-    | 1 -> SV1 | 2 -> SV2 | 3 | 4 -> SV3 | 5 -> SV4
-    | _ -> assert false
-  in
+let t_shape_task ~version ~expl t =
+  let version = int_to_shape_version version in
 (*
   let tim = Unix.gettimeofday () in
  *)
-  let s = t_shape_task ~version ~expl t in
+  let s =
+    match version with
+    | SV1 | SV2 | SV3 | SV4 -> Shape.t_shape_task ~version ~expl t
+    | SV5 -> assert false
+  in
 (*
   let tim = Unix.gettimeofday () -. tim in
   time := !time +. tim;
@@ -491,14 +646,13 @@ let buffer_checksum b =
   let s = Buffer.contents b in
   Digest.to_hex (Digest.string s)
 
-type checksum_version = CV1 | CV2
+type checksum_version = CV1 | CV2 | CV3
 
 module Checksum = struct
 
   let char (_,_,_,buf) c = Buffer.add_char buf c
   let int (_,_,_,buf as b) i =
     char b 'i'; Buffer.add_string buf (string_of_int i)
-  let raw_string (_,_,_,buf) s = Buffer.add_string buf s
   let string (_,_,_,buf as b) s =
     char b '"'; Buffer.add_string buf (String.escaped s); char b '"'
   let option e b = function None -> char b 'n' | Some x -> char b 's'; e b x
@@ -521,33 +675,15 @@ module Checksum = struct
 
   let ident (v,_,_,_ as b) id = match v with
     | CV1 -> ident_v1 b id
-    | CV2 -> ident_v2 b id
+    | CV2 | CV3 -> ident_v2 b id
 
-  let integer_const b = function
-    | IConstRaw i -> raw_string b (BigInt.to_string i)
-    | IConstDec s -> raw_string b s
-    | IConstHex s -> raw_string b "0x"; raw_string b s
-    | IConstOct s -> raw_string b "0o"; raw_string b s
-    | IConstBin s -> raw_string b "0b"; raw_string b s
-
-  let real_const b = function
-    | RConstDec (i,f,None)   ->
-       raw_string b i; raw_string b "."; raw_string b f
-    | RConstDec (i,f,Some e) ->
-       raw_string b i; raw_string b "."; raw_string b f; raw_string b "e"; raw_string b e
-    | RConstHex (i,f,Some e) ->
-       raw_string b "0x"; raw_string b i; raw_string b "."; raw_string b f;
-       raw_string b "p"; raw_string b e
-    | RConstHex (i,f,None)   ->
-       raw_string b "0x"; raw_string b i; raw_string b "."; raw_string b f
-
-  let sign b is_negative =
-    if is_negative then raw_string b "-"
-
-  let const b c =
-    match c with
-    | ConstInt c -> sign b c.ic_negative; integer_const b c.ic_abs
-    | ConstReal c -> sign b c.rc_negative; real_const b c.rc_abs
+  let const (v,_,_,buf) c =
+    let fmt = Format.formatter_of_buffer buf in
+    begin match v with
+    | CV1 | CV2 -> Common.const_v1 fmt c
+    | CV3 -> Common.const_v2 fmt c
+    end;
+    Format.pp_print_flush fmt ()
 
   let tvsymbol b tv = ident b tv.Ty.tv_name
 
@@ -557,7 +693,7 @@ module Checksum = struct
 
   let vsymbol (v,_,_,_ as b) vs = match v with
     | CV1 -> ty b vs.vs_ty
-    | CV2 -> ident b vs.vs_name; ty b vs.vs_ty
+    | CV2 | CV3 -> ident b vs.vs_name; ty b vs.vs_ty
 
   (* start: _ V ident a o *)
   let rec pat b p = match p.pat_node with
@@ -660,6 +796,7 @@ module Checksum = struct
     | Theory.MTprsymbol -> char b 'p'
     | Theory.MTstring   -> char b 's'
     | Theory.MTint      -> char b 'i'
+    | Theory.MTid    -> char b 'd'
 
   let meta b m =
     string b m.Theory.meta_name;
@@ -673,6 +810,7 @@ module Checksum = struct
     | Theory.MApr pr -> char b 'p'; ident b pr.Decl.pr_name
     | Theory.MAstr s -> char b 's'; string b s
     | Theory.MAint i -> char b 'i'; int b i
+    | Theory.MAid i -> char b 'd'; ident b i
 
   let tdecl b d = match d.Theory.td_node with
     | Theory.Decl d -> decl b d
@@ -752,10 +890,44 @@ module Checksum = struct
       let _,_,dnew = Trans.apply tr t in
       Digest.to_hex dnew
 
+  module DMid = Diffmap.MakeOfMap (Ident.Mid)
+
+  (* WARNING: The occurence of [Trans.fold] in [task_v3] needs to be executed
+     once at initialization in order for all the applications of this
+     transformation to share the same Wtask ([h] created on first line of
+     [Trans.fold]). In short, the closure is here just so that Trans.fold is
+     executed once and shared for all functions.
+     So, in particular, don't add arguments to task_v3 here. *)
+  let task_v3 =
+    let c = ref 0 in
+    let m = ref Ident.Mid.empty in
+    let b = Buffer.create 8192 in
+    let task_hd t (cold,mold,dold) =
+      c := cold;
+      let mo = DMid.get mold in
+      m := mo;
+      tdecl (CV3,c,m,b) t.Task.task_decl;
+      Buffer.add_string b (Digest.to_hex dold);
+      let dnew = Digest.string (Buffer.contents b) in
+      Buffer.clear b;
+      let mnew = match t.Task.task_decl.Theory.td_node with
+        | Theory.Decl { Decl.d_news = s } ->
+            (* as metas might be have been put on some symbols before their
+               declaration, ignore such symbols *)
+            let s = Ident.Mid.set_diff s mo in
+            Ident.Mid.mapi (fun id () -> Ident.Mid.find id !m) s
+        | _ -> Ident.Mid.set_diff !m mo in
+      !c, DMid.union mold mnew, dnew
+    in
+    let tr = Trans.fold task_hd (0, DMid.empty (), Digest.string "") in
+    fun t ->
+      let _,_,dnew = Trans.apply tr t in
+      Digest.to_hex dnew
 
   let task ~version t = match version with
     | CV1 -> task_v1 t
     | CV2 -> task_v2 t
+    | CV3 -> task_v3 t
 
 end
 
@@ -767,6 +939,7 @@ let task_checksum ?(version=current_shape_version) t =
   let version = match version with
     | 1 | 2 | 3 -> CV1
     | 4 | 5 -> CV2
+    | 6 -> CV3
     | _ -> assert false
   in
 (*
@@ -780,16 +953,6 @@ let task_checksum ?(version=current_shape_version) t =
 *)
   s
 
-(* not used anymore
-let theory_checksum ?(version=current_shape_version) t =
-  let version = match version with
-    | 1 | 2 | 3 -> CV1
-    | 4 -> CV2
-    | _ -> assert false
-  in
-  Checksum.theory ~version t
- *)
-
 (*************************************************************)
 (* Pairing of new and old subgoals                           *)
 (*************************************************************)
@@ -815,17 +978,37 @@ let theory_checksum ?(version=current_shape_version) t =
 module type S = sig
   type 'a t
   val checksum : 'a t -> checksum option
-  val shape    : 'a t -> string
+  val shape    : 'a t -> string * (int list)
   val name     : 'a t -> Ident.ident
 end
 
-module Pairing(Old: S)(New: S) = struct
+module Pairing (Old: S)(New: S) = struct
 
   let rec lcp n s1 s2 =
     if String.length s1 <= n || String.length s2 <= n then n
     else if s1.[n] = s2.[n] then lcp (n+1) s1 s2 else n
   let lcp = lcp 0
 
+  let rec count_same_sorted n l1 l2 =
+    match l1, l2 with
+    | hd1 :: tl1, hd2 :: _ when hd1 < hd2 ->
+        count_same_sorted n tl1 l2
+    | hd1 :: _, hd2 :: tl2 when hd1 > hd2 ->
+        count_same_sorted n l1 tl2
+    | hd1 :: tl1, hd2 :: tl2 when hd1 = hd2 ->
+        count_same_sorted (n + 1) tl1 tl2
+    | _ -> n
+
+  let count_common sl1 sl2 =
+    (* The lists are sorted in mk_node *)
+    count_same_sorted 0 sl1 sl2
+
+  (* An implicit invariant when comparing lists is that the session share part
+     of their global_shape (int -> shape) because this global value is copied
+     from old session to new session at the start of the merge process *)
+  let lcp_list (sh1, sl1) (sh2, sl2) =
+    (lcp sh1 sh2, count_common sl1 sl2)
+
   open Ident
 
   type goal_index = Old of int | New of int
@@ -839,7 +1022,7 @@ module Pairing(Old: S)(New: S) = struct
 
   type node = {
     mutable  prev: node;
-            shape: string;
+            shape: string * (int list);
               elt: goal_index;
     mutable valid: bool;
     mutable  next: node;
@@ -847,8 +1030,14 @@ module Pairing(Old: S)(New: S) = struct
 
   let mk_node table g =
     let s = match g with
-      | Old g -> Old.shape table.table_old.(g)
-      | New g -> New.shape table.table_new.(g)
+      | Old g ->
+          let (sh, l) = Old.shape table.table_old.(g) in
+          (* Sanitazing: sort the lists only once *)
+          (sh, List.sort Int.compare l)
+      | New g ->
+          let (sh, l) = New.shape table.table_new.(g) in
+          (* Sanitazing: sort the lists only once *)
+          (sh, List.sort Int.compare l)
     in
     let rec n = { prev = n; shape = s; elt = g; next = n; valid = true }
     in n
@@ -901,7 +1090,8 @@ module Pairing(Old: S)(New: S) = struct
     in
     let newgoals =
       let acc = ref old_goals_without_checksum in
-      for newi = 0 to Array.length table.table_new - 1 do
+      (* in reverse order, since the hashtable acts like a LIFO *)
+      for newi = Array.length table.table_new - 1 downto 0 do
         try
           let newg = table.table_new.(newi) in
           match New.checksum newg with
@@ -910,8 +1100,7 @@ module Pairing(Old: S)(New: S) = struct
              let oldi = Hashtbl.find old_checksums c in
              let oldg = table.table_old.(oldi) in
              Hashtbl.remove old_checksums c;
-             let obs = Old.shape oldg <> New.shape newg in
-             result.(new_goal_index newg) <- (newg, Some (oldg, obs))
+             result.(new_goal_index newg) <- (newg, Some (oldg, false))
         with Not_found ->
           acc := mk_node table (New newi) :: !acc
       done;
@@ -926,15 +1115,18 @@ module Pairing(Old: S)(New: S) = struct
     build_list allgoals;
     if allgoals <> [] then begin
       let module E = struct
-        let dummy = let n = List.hd allgoals (* safe *) in 0, (n, n)
-        type t = int * (node * node)
-        let compare (v1, _) (v2, _) = Pervasives.compare v2 v1
+        let dummy = let n = List.hd allgoals (* safe *) in (0, 0), (n, n)
+        type t = (int * int) * (node * node)
+        let compare ((u1,u2), _) ((v1,v2), _) =
+          let c = Int.compare v1 u1 in
+          if c <> 0 then c
+          else Int.compare v2 u2
       end in
       let module PQ = Pqueue.Make(E) in
       let pq = PQ.create () in
       let add x y = match x.elt, y.elt with
         | Old _, New _ | New _, Old _ ->
-            PQ.insert (lcp x.shape y.shape, (x, y)) pq
+            PQ.insert (lcp_list x.shape y.shape, (x, y)) pq
         | Old _, Old _ | New _, New _ -> () in
       iter_pairs add allgoals;
       (* FIXME: exit earlier, as soon as we get min(old,new) pairs *)
