@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -60,10 +60,10 @@ let extra_help fmt commands =
 let available_commands () =
   let commands = Sys.readdir command_path in
   Array.sort String.compare commands;
-  let re = Str.regexp "^why3\\([^.]+\\)\\([.].*\\)?" in
+  let re = Re.Str.regexp "^why3\\([^.]+\\)\\([.].*\\)?" in
   let commands = Array.fold_left (fun acc v ->
-    if Str.string_match re v 0 then
-      let w = Str.matched_group 1 v in
+    if Re.Str.string_match re v 0 then
+      let w = Re.Str.matched_group 1 v in
       match acc with
       | (h,_)::_ when h = w -> acc
       | _ -> (w, v) :: acc
@@ -103,7 +103,10 @@ let command sscmd =
       Filename.concat command_path scmd
     end in
   let scmd = "why3 " ^ sscmd in
-  Unix.execv cmd (Array.of_list (scmd :: args))
+  let args = Array.of_list (scmd :: args) in
+  (* add double quotes to avoid splitting issues with execv in Windows *)
+  let args = if Sys.win32 then Array.map Filename.quote args else args in
+  Unix.execv cmd args
 
 let () = try
   let extra_help fmt () = extra_help fmt (available_commands ()) in
@@ -124,8 +127,8 @@ let () = try
       (Pp.print_list Pp.newline2 print_trans_desc)
       (List.sort sort_pair (Trans.list_transforms_l ()));
     let list_transform_with_arg =
-      Trans.list_transforms_with_args () @ Trans.list_transforms_with_args_l ()
-    in
+      Trans.list_transforms_with_args () @
+      Trans.list_transforms_with_args_l () in
     printf "@[<hov 2>Known transformations with arguments:@\n%a@]@\n@."
       (Pp.print_list Pp.newline2 print_trans_desc)
       (List.sort sort_pair list_transform_with_arg)
@@ -148,7 +151,8 @@ let () = try
     in
     printf "@[Known input formats:@\n  @[%a@]@]@."
       (Pp.print_list Pp.newline2 print)
-      (List.sort Pervasives.compare (Env.list_formats Env.base_language))
+      (List.sort (fun (u,_,_) (v,_,_) -> String.compare u v)
+         (Env.list_formats Env.base_language))
   end;
   if !opt_list_provers then begin
     opt_list := true;
@@ -166,7 +170,7 @@ let () = try
       (Pp.print_list Pp.space Pretty.print_meta_arg_type) m.meta_type
       Pp.formatted m.meta_desc
     in
-    let cmp m1 m2 = Pervasives.compare m1.meta_name m2.meta_name in
+    let cmp m1 m2 = String.compare m1.meta_name m2.meta_name in
     printf "@[<hov 2>Known metas:@\n%a@]@\n@."
       (Pp.print_list Pp.newline2 print) (List.sort cmp (Theory.list_metas ()))
   end;
