@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -34,4 +34,5 @@ let elim_less (d:decl) =
 
 let () =
   Trans.register_transform "abstract_quantifiers" (Trans.decl elim_less None)
-    ~desc:"abstract@ quantifiers@ in@ the@ axioms@ of@ the@ context and the goals@."
+    ~desc:"Abstract@ quantifiers@ in@ the@ axioms@ of@ the@ context \
+           and@ the@ goals."
