@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -144,6 +144,17 @@ let add (d: decl) (tables: naming_table): naming_table =
       let s = id_unique tables id in
       add_unsafe s (Pr pr) tables
 
+(* Adds meta arguments of type ident to tables *)
+let add_meta_id_args (al: meta_arg list) (tables: naming_table): naming_table =
+  List.fold_left
+    (fun t a ->
+      match a with
+      | MAid id ->
+         let s = id_unique tables id in
+         { tables with meta_id_args = Mstr.add s id tables.meta_id_args }
+      | _ -> t)
+    tables al
+
 (* Takes the set of meta defined in the tasks and build the coercions from it.
    TODO we could have a set of coercions in the task ? Same problem for naming
    table ?
@@ -173,6 +184,7 @@ let build_naming_tables task : naming_table =
       coercion = Coercion.empty;
       printer = pr;
       aprinter = apr;
+      meta_id_args = Mstr.empty;
   } in
 (*  We want conflicting names to be named as follows:
     names closer to the goal should be named with lowest
@@ -182,7 +194,10 @@ let build_naming_tables task : naming_table =
   (* TODO:imported theories should be added in the namespace too *)
   let tables = Task.task_fold
     (fun t d ->
-     match d.td_node with Decl d -> add d t | _ -> t) tables task
+      match d.td_node with
+      | Decl d -> add d t
+      | Meta (_,al) -> add_meta_id_args al t
+      | _ -> t) tables task
   in
   let crc_map = build_coercion_map km_meta in
   {tables with coercion = crc_map}
@@ -210,6 +225,7 @@ type (_, _) trans_typ =
   | Tlist       : ('a, 'b) trans_typ -> ((symbol list -> 'a), 'b) trans_typ
   | Tidentlist  : ('a, 'b) trans_typ -> ((string list -> 'a), 'b) trans_typ
   | Ttermlist   : ('a, 'b) trans_typ -> ((term list -> 'a), 'b) trans_typ
+  | Ttermlist_same : int * ('a, 'b) trans_typ -> ((term list -> 'a), 'b) trans_typ
   | Tterm       : ('a, 'b) trans_typ -> ((term -> 'a), 'b) trans_typ
   | Tformula    : ('a, 'b) trans_typ -> ((term -> 'a), 'b) trans_typ
   | Ttheory     : ('a, 'b) trans_typ -> ((Theory.theory -> 'a), 'b) trans_typ
@@ -233,6 +249,17 @@ let find_symbol q tables =
             try Tstysymbol (find_ts q tables) with
             | Not_found -> raise (Arg_qid_not_found q)
 
+let find_pr_list pr_list tables =
+  Lists.map_filter (fun id ->
+    try Some (find_pr id tables) with
+    | Not_found -> Warning.emit "Symbol '%a' not found, ignored"
+       Typing.print_qualid id; None) pr_list
+
+let find_symbol_list pr_list tables =
+  Lists.map_filter (fun id ->
+    try Some (find_symbol id tables) with Arg_qid_not_found _ ->
+      Warning.emit "Symbol '%a' not found, ignored"
+        Typing.print_qualid id; None) pr_list
 
 let type_ptree ~as_fmla t tables =
   let km = tables.known_map in
@@ -244,50 +271,150 @@ let type_ptree ~as_fmla t tables =
 
 exception Arg_parse_type_error of Loc.position * string * exn
 
-let parse_and_type ~as_fmla s task =
+let registered_lang_parsing_trans = Hashtbl.create 63
+
+exception Add_language_parser
+
+let set_argument_parsing_functions lang ~parse_term ~parse_term_list
+    ~parse_qualid ~parse_list_qualid ~parse_list_ident =
+  if Hashtbl.mem registered_lang_parsing_trans lang then
+    raise Add_language_parser
+  else
+    Hashtbl.add registered_lang_parsing_trans lang
+      (parse_term,
+       parse_term_list,
+       parse_qualid,
+       parse_list_qualid,
+       parse_list_ident)
+
+let () =
+  set_argument_parsing_functions Lexer.whyml_format
+    ~parse_term:(fun _ -> Lexer.parse_term)
+    ~parse_term_list:(fun _ -> Lexer.parse_term_list)
+    ~parse_qualid:Lexer.parse_qualid
+    ~parse_list_qualid:Lexer.parse_list_qualid
+    ~parse_list_ident:Lexer.parse_list_ident
+
+let get_parse_term lang =
+  let (p, _, _, _, _) =
+    try Hashtbl.find registered_lang_parsing_trans lang with
+    | Not_found -> Hashtbl.find registered_lang_parsing_trans Lexer.whyml_format in
+  p
+
+let get_parse_term_list lang =
+  let (_, p, _, _, _) =
+    try Hashtbl.find registered_lang_parsing_trans lang with
+    | Not_found -> Hashtbl.find registered_lang_parsing_trans Lexer.whyml_format in
+  p
+
+let get_parse_qualid lang =
+  let (_, _, p, _, _) =
+    try Hashtbl.find registered_lang_parsing_trans lang with
+    | Not_found -> Hashtbl.find registered_lang_parsing_trans Lexer.whyml_format in
+  p
+
+let get_parse_list_qualid lang =
+  let (_, _, _, p, _) =
+    try Hashtbl.find registered_lang_parsing_trans lang with
+    | Not_found -> Hashtbl.find registered_lang_parsing_trans Lexer.whyml_format in
+  p
+
+let get_parse_list_ident lang =
+  let (_, _, _, _, p) =
+    try Hashtbl.find registered_lang_parsing_trans lang with
+    | Not_found -> Hashtbl.find registered_lang_parsing_trans Lexer.whyml_format in
+  p
+
+(*
+  parse_term_ref        := parse_term;
+  parse_term_list_ref   := parse_term_list;
+  parse_qualid_ref      := parse_qualid;
+  parse_list_qualid_ref := parse_list_qualid;
+  parse_list_ident_ref  := parse_list_ident
+*)
+
+let parse_and_type ~lang ~as_fmla s naming_table =
   try
     let lb = Lexing.from_string s in
-    let t =
-      Lexer.parse_term lb
-    in
-    let t =
-      type_ptree ~as_fmla:as_fmla t task
-    in
-    t
+    let t = get_parse_term lang naming_table lb in
+    type_ptree ~as_fmla:as_fmla t naming_table
   with
   | Loc.Located (loc, e) -> raise (Arg_parse_type_error (loc, s, e))
 
-let parse_and_type_list ~as_fmla s task =
+let parse_and_type_list ~lang ~as_fmla s naming_table =
   try
     let lb = Lexing.from_string s in
-    let t_list =
-      Lexer.parse_term_list lb
-    in
-    let t_list =
-      List.map (fun t -> type_ptree ~as_fmla:as_fmla t task) t_list
-    in
-    t_list
+    let t_list = get_parse_term_list lang naming_table lb in
+    List.map (fun t -> type_ptree ~as_fmla:as_fmla t naming_table) t_list
   with
   | Loc.Located (loc, e) -> raise (Arg_parse_type_error (loc, s, e))
 
-let parse_qualid s =
+(* This parses and types a list of terms that should have the same type
+   (example of replace). Because of coercions, this is not trivial to do in
+   this file. To force unification we use a trick that takes the list
+   [t1; t2; t3;....tn] and try to type the term
+   (t1 = t2 /\ t1 = t3 /\ ... t1 = tn) instead. This enforces the correct
+   use of coercions during typing. We then recover the separated terms. *)
+let parse_list_same ~lang ~as_fmla ls naming_table =
+  (* Deconstruct the term built for typing into the list of typed terms *)
+  let rec collect acc t =
+    match t.t_node with
+    | Tbinop (Tand, t1, {t_node = Tapp (eq, [_; t3]); _}) when ls_equal eq ps_equ ->
+        collect (t3 :: acc) t1
+    | Tapp (_, [t2; t3]) -> t2 :: t3 :: acc
+    | _ -> t :: acc in
+  try
+    let llb = List.map (fun s -> Lexing.from_string s) ls in
+    let t_list = List.map (get_parse_term lang naming_table) llb in
+    (* Type them somehow so that they have the same type *)
+    let id_equal = Ptree.{id_str = Ident.op_infix "=";
+                          id_ats = [];
+                          id_loc = Loc.dummy_position } in
+    if List.length t_list < 2 then
+      (* This case should not happen, fallback to normal typing *)
+      List.map (fun t -> type_ptree ~as_fmla t naming_table) t_list
+    else
+      let hds, t_list = Lists.split 2 t_list in
+      let open Ptree in
+      match hds with
+      | hd1 :: hd2 :: [] ->
+          let tfst_eq = {term_desc = Tidapp (Qident id_equal, [hd1; hd2]);
+                         term_loc = Loc.dummy_position} in
+          let term_to_type = List.fold_left
+              (fun acc t ->
+                 let teq = {term_desc = Tidapp (Qident id_equal, [hd1; t]);
+                            term_loc = Loc.dummy_position} in
+                 {term_desc = Tbinnop (acc, Dterm.DTand, teq);
+                  term_loc = Loc.dummy_position })
+              tfst_eq t_list in
+          let typed_term = type_ptree ~as_fmla:true term_to_type naming_table in
+          (* Recollect the (now) typed terms from the conjunction *)
+          collect [] typed_term
+      | _ -> assert false (* Lists.split should give a list of 2 elements *)
+  with
+  | Loc.Located (loc, e) ->
+      let arg = List.fold_left (fun acc x ->
+          if acc = "" then x else acc ^ " " ^ x) "" ls in
+      raise (Arg_parse_type_error (loc, arg, e))
+
+let parse_qualid ~lang s =
   try
     let lb = Lexing.from_string s in
-    Lexer.parse_qualid lb
+    get_parse_qualid lang lb
   with
   | Loc.Located (loc, e) -> raise (Arg_parse_type_error (loc, s, e))
 
-let parse_list_qualid s =
+let parse_list_qualid ~lang s =
   try
     let lb = Lexing.from_string s in
-    Lexer.parse_list_qualid lb
+    get_parse_list_qualid lang lb
   with
   | Loc.Located (loc, e) -> raise (Arg_parse_type_error (loc, s, e))
 
-let parse_list_ident s =
+let parse_list_ident ~lang s =
   try
     let lb = Lexing.from_string s in
-    Lexer.parse_list_ident lb
+    get_parse_list_ident lang lb
   with
   | Loc.Located (loc, e) -> raise (Arg_parse_type_error (loc, s, e))
 
@@ -354,6 +481,7 @@ let rec is_trans_typ_l: type a b. (a, b) trans_typ -> b trans_typ_is_l =
     | Tterm t        -> is_trans_typ_l t
     | Tidentlist t   -> is_trans_typ_l t
     | Ttermlist t    -> is_trans_typ_l t
+    | Ttermlist_same (_, t) -> is_trans_typ_l t
     | Tformula t     -> is_trans_typ_l t
     | Ttheory t      -> is_trans_typ_l t
     | Topt (_,t)     -> is_trans_typ_l t
@@ -379,6 +507,7 @@ let rec string_of_trans_typ : type a b. (a, b) trans_typ -> string =
     | Tformula _     -> "formula"
     | Tidentlist _   -> "list ident"
     | Ttermlist _    -> "list term"
+    | Ttermlist_same _ -> "list term with same type"
     | Ttheory _      -> "theory"
     | Topt (s,t)     -> "?" ^ s ^ (string_of_trans_typ t)
     | Toptbool (s,_) -> "?" ^ s ^ ":bool"
@@ -403,14 +532,13 @@ let rec print_type : type a b. Format.formatter -> (a, b) trans_typ -> unit =
     | Tformula t     -> Format.fprintf fmt "formula -> %a" print_type t
     | Tidentlist t   -> Format.fprintf fmt "list ident -> %a" print_type t
     | Ttermlist t    -> Format.fprintf fmt "list term -> %a" print_type t
+    | Ttermlist_same (n, t) -> Format.fprintf fmt "list term (same type arity: %d) -> %a" n print_type t
     | Ttheory t      -> Format.fprintf fmt "theory -> %a" print_type t
     | Topt (s,t)     -> Format.fprintf fmt "?%s -> %a" s print_type t
     | Toptbool (s,t) -> Format.fprintf fmt "?%s:bool -> %a" s print_type t
 
-exception Unnecessary_arguments of string list
-
-let rec wrap_to_store : type a b. (a, b) trans_typ -> a -> string list -> Env.env -> naming_table -> task -> b =
-  fun t f l env tables task ->
+let rec wrap_to_store : type a b. (a, b) trans_typ -> a -> string list -> Env.env -> naming_table -> Env.fformat -> task -> b =
+  fun t f l env tables lang task ->
     match t, l with
     | Ttrans, []-> apply f task
     | Ttrans_l, [] -> apply f task
@@ -421,118 +549,124 @@ let rec wrap_to_store : type a b. (a, b) trans_typ -> a -> string list -> Env.en
     | Tenvtrans, _ -> raise (Unnecessary_arguments l)
     | Tenvtrans_l, _ -> raise (Unnecessary_arguments l)
     | Tint t', s :: tail ->
-      let arg = parse_int s in wrap_to_store t' (f arg) tail env tables task
+      let arg = parse_int s in wrap_to_store t' (f arg) tail env tables lang task
     | Tstring t', s :: tail ->
-       wrap_to_store t' (f s) tail env tables task
+       wrap_to_store t' (f s) tail env tables lang task
     | Tformula t', s :: tail ->
-      let te = parse_and_type ~as_fmla:true s tables in
-      wrap_to_store t' (f te) tail env tables task
+      let te = parse_and_type ~lang ~as_fmla:true s tables in
+      wrap_to_store t' (f te) tail env tables lang task
     | Tterm t', s :: tail ->
-      let te = parse_and_type ~as_fmla:false s tables in
-      wrap_to_store t' (f te) tail env tables task
+      let te = parse_and_type ~lang ~as_fmla:false s tables in
+      wrap_to_store t' (f te) tail env tables lang task
     | Tty t', _s :: tail ->
       let ty = Ty.ty_int in (* TODO: parsing + typing of s *)
-      wrap_to_store t' (f ty) tail env tables task
+      wrap_to_store t' (f ty) tail env tables lang task
     | Ttysymbol t', _s :: tail ->
       let tys = Ty.ts_int in (* TODO: parsing + typing of s *)
-      wrap_to_store t' (f tys) tail env tables task
+      wrap_to_store t' (f tys) tail env tables lang task
     | Tprsymbol t', s :: tail ->
-       let q = parse_qualid s in
+       let q = parse_qualid ~lang s in
        let pr = try (find_pr q tables) with
                 | Not_found -> raise (Arg_qid_not_found q) in
-      wrap_to_store t' (f pr) tail env tables task
+      wrap_to_store t' (f pr) tail env tables lang task
     | Tprlist t', s :: tail ->
-        let pr_list = parse_list_qualid s in
-        let pr_list =
-        List.map (fun id ->
-                    try find_pr id tables with
-                    | Not_found -> raise (Arg_qid_not_found id))
-                 pr_list in
-        wrap_to_store t' (f pr_list) tail env tables task
+        let pr_list = parse_list_qualid ~lang s in
+        let pr_list = find_pr_list pr_list tables in
+        wrap_to_store t' (f pr_list) tail env tables lang task
     | Tlsymbol t', s :: tail ->
-       let q = parse_qualid s in
+       let q = parse_qualid ~lang s in
        let pr = try (find_ls q tables) with
                | Not_found -> raise (Arg_qid_not_found q) in
-      wrap_to_store t' (f pr) tail env tables task
+      wrap_to_store t' (f pr) tail env tables lang task
     | Tsymbol t', s :: tail ->
-       let q = parse_qualid s in
+       let q = parse_qualid ~lang s in
        let symbol = find_symbol q tables in
-       wrap_to_store t' (f symbol) tail env tables task
+       wrap_to_store t' (f symbol) tail env tables lang task
     | Tlist t', s :: tail ->
-       let pr_list = parse_list_qualid s in
-       let pr_list =
-         List.map (fun id -> find_symbol id tables) pr_list in
-       wrap_to_store t' (f pr_list) tail env tables task
+       let pr_list = parse_list_qualid ~lang s in
+       let pr_list = find_symbol_list pr_list tables in
+       wrap_to_store t' (f pr_list) tail env tables lang task
     | Ttheory t', s :: tail ->
        let th = parse_theory env s in
-       wrap_to_store t' (f th) tail env tables task
+       wrap_to_store t' (f th) tail env tables lang task
     | Tidentlist t', s :: tail ->
-       let list = List.map (fun id -> id.Ptree.id_str) (parse_list_ident s) in
-       wrap_to_store t' (f list) tail env tables task
+       let list = List.map (fun id -> id.Ptree.id_str) (parse_list_ident ~lang s) in
+       wrap_to_store t' (f list) tail env tables lang task
     | Ttermlist t', s :: tail ->
-       let term_list = parse_and_type_list ~as_fmla:false s tables in
-       wrap_to_store t' (f term_list) tail env tables task
+       let term_list = parse_and_type_list ~lang ~as_fmla:false s tables in
+       wrap_to_store t' (f term_list) tail env tables lang task
+    | Ttermlist_same (arity, t'), s :: tail ->
+        let n_firsts, lasts =
+          try Lists.split arity (s :: tail) with
+          | _ ->
+              let msg = "list of " ^ string_of_int arity ^ " elements of same type" in
+              let args = List.fold_left (fun acc x ->
+                  if acc = "" then x else acc ^ " " ^ x) "" (s::tail) in
+              raise (Arg_expected (msg, args))
+        in
+        let term_list = parse_list_same ~lang ~as_fmla:false n_firsts tables in
+        wrap_to_store t' (f term_list) lasts env tables lang task
     | Topt (optname, t'), s :: s' :: tail when s = optname ->
        begin match t' with
         | Tint t' ->
           let arg = Some (parse_int s') in
-          wrap_to_store t' (f arg) tail env tables task
+          wrap_to_store t' (f arg) tail env tables lang task
         | Tprsymbol t' ->
-           let q = parse_qualid s' in
+           let q = parse_qualid ~lang s' in
            let arg = try Some (find_pr q tables) with
                     | Not_found -> raise (Arg_qid_not_found q) in
-          wrap_to_store t' (f arg) tail env tables task
+          wrap_to_store t' (f arg) tail env tables lang task
         | Tsymbol t' ->
-           let q = parse_qualid s' in
+           let q = parse_qualid ~lang s' in
            let arg = Some (find_symbol q tables) in
-           wrap_to_store t' (f arg) tail env tables task
+           wrap_to_store t' (f arg) tail env tables lang task
         | Tformula t' ->
-           let arg = Some (parse_and_type ~as_fmla:true s' tables) in
-           wrap_to_store t' (f arg) tail env tables task
+           let arg = Some (parse_and_type ~lang ~as_fmla:true s' tables) in
+           wrap_to_store t' (f arg) tail env tables lang task
         | Tterm t' ->
-           let arg = Some (parse_and_type ~as_fmla:false s' tables) in
-           wrap_to_store t' (f arg) tail env tables task
+           let arg = Some (parse_and_type ~lang ~as_fmla:false s' tables) in
+           wrap_to_store t' (f arg) tail env tables lang task
         | Ttheory t' ->
            let arg = Some (parse_theory env s') in
-           wrap_to_store t' (f arg) tail env tables task
+           wrap_to_store t' (f arg) tail env tables lang task
         | Tstring t' ->
            let arg = Some s' in
-           wrap_to_store t' (f arg) tail env tables task
+           wrap_to_store t' (f arg) tail env tables lang task
         | Tprlist t' ->
-            let pr_list = parse_list_qualid s' in
-            let pr_list =
-              List.map (fun id ->
-                try find_pr id tables with
-                | Not_found -> raise (Arg_qid_not_found id))
-                pr_list in
+            let pr_list = parse_list_qualid ~lang s' in
+            let pr_list = find_pr_list pr_list tables in
             let arg = Some pr_list in
-            wrap_to_store t' (f arg) tail env tables task
+            wrap_to_store t' (f arg) tail env tables lang task
         | Ttermlist t' ->
-            let term_list = parse_and_type_list ~as_fmla:false s' tables in
-            wrap_to_store t' (f (Some term_list)) tail env tables task
+            let term_list = parse_and_type_list ~lang ~as_fmla:false s' tables in
+            wrap_to_store t' (f (Some term_list)) tail env tables lang task
         | Tidentlist t' ->
             let list =
-              List.map (fun id -> id.Ptree.id_str) (parse_list_ident s') in
-            wrap_to_store t' (f (Some list)) tail env tables task
+              List.map (fun id -> id.Ptree.id_str) (parse_list_ident ~lang s') in
+            wrap_to_store t' (f (Some list)) tail env tables lang task
+        | Tlist t' ->
+            let pr_list = parse_list_qualid ~lang s' in
+            let pr_list = find_symbol_list pr_list tables in
+            wrap_to_store t' (f (Some pr_list)) tail env tables lang task
         | _ -> raise (Arg_expected (string_of_trans_typ t', s'))
        end
     | Topt (_, t'), _ ->
-       wrap_to_store (trans_typ_tail t') (f None) l env tables task
+       wrap_to_store (trans_typ_tail t') (f None) l env tables lang task
     | Toptbool (optname, t'), s :: tail when s = optname ->
-      wrap_to_store t' (f true) tail env tables task
+      wrap_to_store t' (f true) tail env tables lang task
     | Toptbool (_, t'), _ ->
-      wrap_to_store t' (f false) l env tables task
+      wrap_to_store t' (f false) l env tables lang task
     | _, [] -> raise (Arg_expected_none (string_of_trans_typ t))
 
 let wrap_l : type a. (a, task list) trans_typ -> a -> trans_with_args_l =
-  fun t f l env tables -> Trans.store (wrap_to_store t f l env tables)
+  fun t f l env tables lang -> Trans.store (wrap_to_store t f l env tables lang)
 
 let wrap   : type a. (a, task) trans_typ -> a -> trans_with_args =
-  fun t f l env tables -> Trans.store (wrap_to_store t f l env tables)
+  fun t f l env tables lang -> Trans.store (wrap_to_store t f l env tables lang)
 
 let wrap_any : type a b. (a, b) trans_typ -> a -> string list -> Env.env ->
-                    Trans.naming_table -> b trans =
-  fun t f l env tables -> Trans.store (wrap_to_store t f l env tables)
+                    Trans.naming_table -> Env.fformat -> b trans =
+  fun t f l env tables lang -> Trans.store (wrap_to_store t f l env tables lang)
 
 (* the one in Scanf is awfully broken with respect to backslashes *)
 let format_from_string s fmt =
@@ -550,4 +684,5 @@ let wrap_and_register : type a b. desc:Pp.formatted -> string -> (a, b) trans_ty
     | No  -> Trans.register_transform_with_args   ~desc name trans
 
 
-let find_symbol s tables = find_symbol (parse_qualid s) tables
+let find_symbol s tables =
+  find_symbol (parse_qualid ~lang:Lexer.whyml_format s) tables
