@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -27,7 +27,6 @@ exception Arg_pr_not_found of Decl.prsymbol
 exception Arg_qid_not_found of Ptree.qualid
 exception Arg_error of string
 exception Arg_parse_type_error of Loc.position * string * exn
-exception Unnecessary_arguments of string list
 
 
 val build_naming_tables : Task.task -> Trans.naming_table
@@ -86,6 +85,9 @@ type (_, _) trans_typ =
       to exist in the task, typically they could be fresh symbols *)
   | Ttermlist   : ('a, 'b) trans_typ -> ((Term.term list -> 'a), 'b) trans_typ
   (** transformation with a list of terms as argument. *)
+  | Ttermlist_same : int * ('a, 'b) trans_typ -> ((Term.term list -> 'a), 'b) trans_typ
+  (** transformation with a list of same type term: the int is the predefined
+      length of the list *)
   | Tterm       : ('a, 'b) trans_typ -> ((Term.term -> 'a), 'b) trans_typ
   (** transformation with a Why3 term as argument *)
   | Tformula    : ('a, 'b) trans_typ -> ((Term.term -> 'a), 'b) trans_typ
@@ -112,3 +114,12 @@ val wrap_l : ('a, task list) trans_typ -> 'a -> Trans.trans_with_args_l
 val wrap   : ('a, task) trans_typ -> 'a -> Trans.trans_with_args
 
 val wrap_and_register : desc:Pp.formatted -> string -> ('a, 'b) trans_typ -> 'a -> unit
+
+val set_argument_parsing_functions:
+  Env.fformat ->
+  parse_term:(Trans.naming_table -> Lexing.lexbuf -> Ptree.term) ->
+  parse_term_list:(Trans.naming_table -> Lexing.lexbuf -> Ptree.term list) ->
+  parse_qualid:(Lexing.lexbuf -> Ptree.qualid) ->
+  parse_list_qualid:(Lexing.lexbuf -> Ptree.qualid list) ->
+  parse_list_ident:(Lexing.lexbuf -> Ptree.ident list) ->
+  unit
