@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -22,7 +22,7 @@ let meta_rewrite_def = Theory.register_meta "rewrite_def" [Theory.MTlsymbol]
 
 let meta_compute_max_steps = Theory.register_meta_excl "compute_max_steps"
   [Theory.MTint]
-  ~desc:"Maximal@ number@ of@ reduction@ steps@ done@ by@ compute@ \
+  ~desc:"Maximal@ number@ of@ reduction@ steps@ done@ by@ the@ 'compute'@ \
          transformation"
 
 let compute_max_steps = ref 1024
@@ -116,12 +116,13 @@ let normalize_goal_transf_few env =
 
 let () =
   Trans.register_env_transform_l "compute_in_goal" normalize_goal_transf_all
-  ~desc:"Performs@ possible@ computations@ in@ goal, including@ by@ \
-         declared@ rewrite@ rules"
+  ~desc:"Perform@ computations@ in@ the@ goal,@ also@ using@ \
+    the@ automatically@ derived@ rules)."
 
 let () =
   Trans.register_env_transform_l "compute_specified" normalize_goal_transf_few
-  ~desc:"Rewrite@ goal@ using@ specified@ rules"
+  ~desc:"Perform@ computations@ in@ the@ goal,@ only@ using@ \
+    the@ user-specified@ rules)."
 
 let normalize_hyp step_limit pr_norm env =
   let p = { compute_defs = true;
@@ -130,19 +131,24 @@ let normalize_hyp step_limit pr_norm env =
           } in
   normalize_goal_transf ?pr_norm ?step_limit p env
 
-let () = wrap_and_register ~desc:"experimental: Takes a prsymbol and reduce \
-    one \"elementary\" step."
+let () = wrap_and_register
+    ~desc:"step [in] <name>@ performs@ one@ \"elementary\"@ \
+      reduction@ step@ in@ the@ given@ premise@ (experimental)."
     "step"
-    (Topt ("in", Tprsymbol Tenvtrans_l)) (normalize_hyp (Some 1))
+    (Topt ("in", Tprsymbol Tenvtrans_l))
+    (normalize_hyp (Some 1))
 
-let () = wrap_and_register ~desc:"experimental: Same as step except that it \
-    reduces the given number of steps."
+let () = wrap_and_register
+    ~desc:"steps <n> [in] <name>@ performs@ <n>@ \"elementary\"@ \
+      reduction@ steps@ in@ the@ given@ premise@ (experimental)."
     "steps"
-    (Tint (Topt ("in", Tprsymbol Tenvtrans_l))) (fun n -> normalize_hyp (Some n))
+    (Tint (Topt ("in", Tprsymbol Tenvtrans_l)))
+    (fun n -> normalize_hyp (Some n))
 
-
-let () = wrap_and_register ~desc:"Performs@ possible@ computations@ in@ given \
-    hypothesis@ including@ by@ declared@ rewrite@ rules"
+let () = wrap_and_register
+    ~desc:"compute_hyp [in] <name>@ performs@ computations@ \
+      in@ the@ given@ premise, also@ using@ the@ automatically@ \
+      derived@ rules."
     "compute_hyp"
     (Topt ("in", Tprsymbol Tenvtrans_l)) (normalize_hyp None)
 
@@ -153,7 +159,49 @@ let normalize_hyp_few step_limit pr_norm env =
           } in
   normalize_goal_transf ?pr_norm ?step_limit p env
 
-let () = wrap_and_register ~desc:"Performs@ possible@ computations@ in@ given \
-    hypothesis@ using@ specified@ rules"
+let () = wrap_and_register
+    ~desc:"compute_hyp_specified [in] <name>@ performs@ computations@ \
+      in@ the@ given@ premise, only@ using@ the@ user-specified@ rules."
     "compute_hyp_specified"
     (Topt ("in", Tprsymbol Tenvtrans_l)) (normalize_hyp_few None)
+
+let simplify check_ls env : 'a Trans.trans =
+  let normalize_transf engine =
+    Trans.decl (fun d ->
+      match d.d_node with
+      | Dprop (k, pr, t) ->
+          let t = normalize ~step_limit:(1024*1024) ~limit:(1024*1024) engine t in
+          let d = Decl.create_prop_decl k pr t in
+          [d]
+      | _ -> [d]) None
+  in
+  let collect t =
+    let acc = Task.task_fold
+      (fun acc td -> match td.Theory.td_node with
+        | Theory.Decl { d_node = Dprop((Plemma|Paxiom), pr, t) } ->
+            begin match t.t_node with
+            | Tapp(ls,[t1;_]) when ls == ps_equ ->
+                begin match t1.t_node with
+                | Tapp(ls,_::_) when not (check_ls ls) -> (pr,t) :: acc
+                | _ -> acc
+                end
+            | _ -> acc
+            end
+        | _ -> acc)
+      [] t
+    in
+    let p = { compute_defs = false;
+              compute_builtin = true;
+              compute_def_set = Term.Mls.empty;
+            } in
+    List.fold_left
+      (fun e (pr,t) ->
+        try add_rule t e
+        with NotARewriteRule msg ->
+          Warning.emit "proposition %a cannot be turned into a rewrite rule: %s"
+            Pretty.print_pr pr msg;
+          e
+      )
+      (create p env (Task.task_known t)) acc in
+  let tr = Trans.store collect in
+  Trans.bind tr (fun engine -> normalize_transf engine)
