@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -295,7 +295,7 @@ let add_projections (state,task) _ts _ty csl =
             create_lsymbol id [Opt.get cs.ls_value] t.t_ty
       in
       let tsk = add_param_decl tsk ls in
-      let id = id_derive (ls.ls_name.id_string ^ "_def") ls.ls_name in
+      let id = id_derive (ls.ls_name.id_string ^ "'def") ls.ls_name in
       let pr = create_prsymbol id in
       let hh = t_app ls [hd] t.t_ty in
       let ax = t_forall_close (List.rev vl) [] (t_equ hh t) in
@@ -405,7 +405,7 @@ let comp t (state,task) = match t.task_decl.td_node with
       let state = match dl with
         | _ when state.keep_t -> state
         | [ts, [_]]
-          when state.keep_r && not (Sid.mem ts.ts_name d.d_syms) ->
+          when state.keep_r && not (Sid.mem ts.ts_name (get_decl_syms d)) ->
             { state with keep_t = true }
         | [{ ts_args = [] }, csl]
           when state.keep_e && List.for_all (fun (_,l) -> l = []) csl ->
@@ -433,7 +433,7 @@ let comp t (state,task) = match t.task_decl.td_node with
         let task = add_tdecl task (create_use th) in
         rstate := state ; rtask := task ; None
       in
-      let tp_map = Mid.diff add state.tp_map d.d_syms in
+      let tp_map = Mid.diff add state.tp_map (get_decl_syms d) in
       comp t ({ !rstate with tp_map = tp_map }, !rtask)
   | _ ->
       comp t (state,task)
