@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -97,7 +97,7 @@ let rec t_insert hd t = match t.t_node with
 let add_ld which meta_rewrite_def (ls,ld) (abst,defn,axl,metas) =
   if which ls then
     let vl,e = open_ls_defn ld in
-    let nm = ls.ls_name.id_string ^ "_def" in
+    let nm = ls.ls_name.id_string ^ "'def" in
     let pr = create_prsymbol (id_derive nm ls.ls_name) in
     let hd = t_app ls (List.map t_var vl) e.t_ty in
     let ax = t_forall_close vl [] (t_insert hd e) in
@@ -125,7 +125,7 @@ let elim which meta_rewrite_def d = match d.d_node with
 
 let elim_recursion d = match d.d_node with
   | Dlogic ([s,_] as l)
-    when Sid.mem s.ls_name d.d_syms -> elim_decl Util.ttrue Sls.empty l
+    when Sid.mem s.ls_name (get_decl_syms d) -> elim_decl Util.ttrue Sls.empty l
   | Dlogic l when List.length l > 1 -> elim_decl Util.ttrue Sls.empty l
   | _ -> [Theory.create_decl d]
 
@@ -135,7 +135,7 @@ let is_struct dl = (* FIXME? Shouldn't 0 be allowed too? *)
 (* FIXME? We can have non-recursive functions in a group *)
 let elim_non_struct_recursion d = match d.d_node with
   | Dlogic ((s,_) :: _ as dl)
-    when Sid.mem s.ls_name d.d_syms && not (is_struct dl) ->
+    when Sid.mem s.ls_name (get_decl_syms d) && not (is_struct dl) ->
       elim_decl Util.ttrue Sls.empty dl
   | _ ->
       [Theory.create_decl d]
@@ -208,7 +208,12 @@ let () =
 open Task
 open Theory
 
-type rem = { rem_pr : Spr.t; rem_ls : Sls.t; rem_ts : Sts.t }
+type rem = {
+  rem_pr : Decl.Spr.t;
+  rem_ls : Term.Sls.t;
+  rem_ts : Ty.Sts.t;
+  rem_nt : Trans.naming_table;
+}
 
 type bisect_step =
  | BSdone of rem
@@ -220,40 +225,19 @@ let _print_rem fmt rem = Format.fprintf fmt
   (Pp.print_iter1 Sls.iter Pp.comma Pretty.print_ls) rem.rem_ls
   (Pp.print_iter1 Sts.iter Pp.comma Pretty.print_ts) rem.rem_ts
 
-(*
-let rec elim_task task rem =
-  match task with
-  | Some ({task_decl = {td_node = Decl decl}} as task) ->
-    let task = elim_task task.task_prev rem in
-    let l = elim_abstract Sls.empty
-      rem.rem_pr rem.rem_ls rem.rem_ts decl in
-    List.fold_left Task.add_decl task l
-  | Some task ->
-    Task.add_tdecl (elim_task task.task_prev rem) task.task_decl
-  | None      -> None
- *)
-
 let add_rem rem decl =
-  let remove_ts rem ts =
+  let _remove_ts rem ts =
     { rem with rem_ts = Sts.add ts rem.rem_ts} in
   let remove_ls rem ls =
     { rem with rem_ls = Sls.add ls rem.rem_ls} in
   let remove_pr rem pr =
     { rem with rem_pr = Spr.add pr rem.rem_pr} in
   match decl.d_node with
-  | Dtype ts -> remove_ts rem ts
-  | Ddata l -> List.fold_left (fun rem (ts,_) -> remove_ts rem ts) rem l
-  | Dparam ls -> remove_ls rem ls
+  | Dtype _ | Dparam _ | Ddata _ -> rem
   | Dlogic l -> List.fold_left (fun rem (ls,_) -> remove_ls rem ls) rem l
   | Dind (_,l) -> List.fold_left (fun rem (ls,_) -> remove_ls rem ls) rem l
   | Dprop (_,pr,_) -> remove_pr rem pr
 
-let _union_rem rem1 rem2 =
-  { rem_ts = Sts.union rem1.rem_ts rem2.rem_ts;
-    rem_ls = Sls.union rem1.rem_ls rem2.rem_ls;
-    rem_pr = Spr.union rem1.rem_pr rem2.rem_pr;
-  }
-
 let fold_sub f acc a i1 i2 =
   let acc = ref acc in
   for i=i1 to i2-1 do
@@ -265,9 +249,7 @@ let rec bisect_aux task a i1 i2 rem cont       (* lt i lk *) =
   (* Format.eprintf "i1: %i, i2: %i@\nrem:%a@." i1 i2 *)
   (*   print_rem rem; *)
   let call rem valid invalid =
-    try BSstep (rem,
-                fun b -> if b then valid () else invalid ())
-    with UnknownIdent _ -> invalid ()
+    BSstep (rem, fun b -> if b then valid () else invalid ())
   in
   if i2 - i1 < 2 then
     let rem1 = add_rem rem a.(i1) in
@@ -276,55 +258,37 @@ let rec bisect_aux task a i1 i2 rem cont       (* lt i lk *) =
       (fun () -> cont rem)
   else
     let m = (i1+i2)/2 in
-    let rem1 = fold_sub add_rem rem a m i2 in
+    let rem1 = fold_sub add_rem rem a i1 m in
     call rem1
-      (fun () -> bisect_aux task a i1 m rem1 cont)
+      (fun () -> bisect_aux task a m i2 rem1 cont)
       (fun () ->
-        bisect_aux task a m i2 rem
+        bisect_aux task a i1 m rem
           (fun rem1 -> (* rem c rem1 c \old(rem1) *)
-            let rem2 = fold_sub add_rem rem1 a i1 m in
+            let rem2 = fold_sub add_rem rem1 a m i2 in
             call rem2
               (fun () -> cont rem2)
-              (fun () -> bisect_aux task a i1 m rem1 cont)))
+              (fun () -> bisect_aux task a m i2 rem1 cont)))
 
 let bisect_step task0 =
-  let task= match task0 with
+  let task = match task0 with
     | Some {task_decl = {td_node = Decl {d_node = Dprop (Pgoal,_,_)}};
             task_prev = task} -> task
     | _ -> raise GoalNotFound in
   let rec length acc = function
-    | Some {task_decl = {td_node = Decl _};
+    | Some {task_decl = {td_node = Decl {d_node = Dlogic _ | Dind _ | Dprop _}};
             task_prev = t} -> length (acc + 1) t
     | Some {task_prev = t} -> length acc t
     | None -> acc in
   let n = length 0 task in
   let a = Array.make n (Obj.magic 0) in
   let rec init acc = function
-    | Some {task_decl = {td_node = Decl d}; task_prev = t} ->
+    | Some {task_decl = {td_node = Decl ({d_node = Dlogic _ | Dind _ | Dprop _} as d)};
+            task_prev = t} ->
       a.(acc) <- d; init (acc - 1) t
-    | Some { task_prev = t} -> init acc t
+    | Some {task_prev = t} -> init acc t
     | None -> assert (acc = -1) in
   init (n-1) task;
+  let namt = Args_wrapper.build_naming_tables task0 in
   let empty_rem = {rem_ts = Sts.empty; rem_ls = Sls.empty;
-                   rem_pr = Spr.empty} in
-  bisect_aux task0 a 0 n empty_rem
-    (fun rem -> BSdone rem)
-
-(*
-let bisect f task =
-  let rec run = function
-    | BSdone r -> r
-    | BSstep (rem,c) -> let t = elim_task task rem in run (c (f t)) in
-  run (bisect_step task)
- *)
-
-(** catch exception for debug *)
-(* let bisect_step task0 = *)
-(*   let res = try bisect_step task0 with exn -> *)
-(*     Format.eprintf "bisect_step fail: %a@." Exn_printer.exn_printer exn; *)
-(*     raise exn in *)
-(*   match res with *)
-(*   | BSdone _ as d -> d *)
-(*   | BSstep (t,f) -> BSstep (t,fun b -> try f b with exn -> *)
-(*     Format.eprintf "bisect_step fail: %a@." Exn_printer.exn_printer exn; *)
-(*     raise exn) *)
+                   rem_pr = Spr.empty; rem_nt = namt} in
+  bisect_aux task0 a 0 n empty_rem (fun rem -> BSdone rem)
