@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -173,7 +173,7 @@ let rec lift_f el acc t0 =
             let ls = create_fsymbol (id_clone vs.vs_name) tyl vs.vs_ty in
             let t = fs_app ls (List.map t_var vl) vs.vs_ty in
             let f = t_forall_close_merge vl (t_subst_single vs t f) in
-            let id = id_derive (vs.vs_name.id_string ^ "_def") vs.vs_name in
+            let id = id_derive (vs.vs_name.id_string ^ "'def") vs.vs_name in
             let ax = create_prop_decl Paxiom (create_prsymbol id) f in
             (create_param_decl ls :: abst, ax :: axml), t
       in
@@ -196,7 +196,7 @@ let lift_l el (acc,dl) (ls,ld) =
       let (abst,axml), f = lift_f el acc f in
       let t = t_app ls (List.map t_var vl) t.t_ty in
       let f = t_forall_close_merge vl (t_subst_single vs t f) in
-      let id = id_derive (ls.ls_name.id_string ^ "_def") ls.ls_name in
+      let id = id_derive (ls.ls_name.id_string ^ "'def") ls.ls_name in
       let ax = create_prop_decl Paxiom (create_prsymbol id) f in
       (create_param_decl ls :: abst, ax :: axml), dl
   | _ ->
@@ -209,7 +209,7 @@ let lift_d el d = match d.d_node with
       if dl = [] then List.rev_append abst (List.rev axml) else
       let d = create_logic_decl (List.rev dl) in
       let add_ax (axml1, axml2) ax =
-        if Sid.disjoint ax.d_syms d.d_news
+        if Sid.disjoint (get_decl_syms ax) d.d_news
         then ax :: axml1, axml2 else axml1, ax :: axml2 in
       let axml1, axml2 = List.fold_left add_ax ([],[]) axml in
       List.rev_append abst (axml1 @ d :: axml2)
