@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,22 +9,12 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Task
-
 let debug = Debug.register_info_flag "prepare_for_counterexmp"
   ~desc:"Print@ debugging@ messages@ about@ preparing@ the@ task@ \
     for@ getting@ counter-example."
 
-let meta_get_counterexmp =
-  Theory.register_meta_excl "get_counterexmp" [Theory.MTstring]
-  ~desc:"Set@ when@ counter-example@ should@ be@ get."
-
-let get_counterexmp task =
-  let ce_meta = Task.find_meta_tds task meta_get_counterexmp in
-  not (Theory.Stdecl.is_empty ce_meta.tds_set)
-
 let prepare_for_counterexmp2 env task =
-  if not (get_counterexmp task) then begin
+  if not (Inlining.get_counterexmp task) then begin
     (* Counter-example will not be queried, do nothing *)
     Debug.dprintf debug "Not get ce@.";
     task
@@ -44,6 +34,6 @@ let prepare_for_counterexmp env = Trans.store (prepare_for_counterexmp2 env)
 
 let () = Trans.register_env_transform "prepare_for_counterexmp"
   prepare_for_counterexmp
-  ~desc:"Transformation@ that@ prepares@ the@ task@ for@ querying@ for@ \
+  ~desc:"Prepare@ the@ task@ for@ querying@ for@ \
     the@ counter-example@ model.@ This@ transformation@ does@ so@ only@ \
     when@ the@ solver@ will@ be@ asked@ for@ the@ counter-example."
