@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,11 +9,6 @@
 (*                                                                  *)
 (********************************************************************)
 
-val get_counterexmp : Task.task -> bool
-(**
-   Returns true if counter-example should be get for the task.
-*)
-
 val prepare_for_counterexmp :  Env.env -> Task.task Trans.trans
  (**
     Transformation that prepares the task for querying for
