@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -351,7 +351,7 @@ let why3_run f lang code =
     output_string ch code;
     close_out ch;
 
-    let theories = Env.read_file lang env temp_file_name in
+    let (theories, _) = Env.read_file lang env temp_file_name in
     W.send (Warning !Task.warnings);
     f theories
   with
