@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -110,7 +110,7 @@ module Args = struct
         Format.printf "@[<hov 2>Known debug flags \
             (`*' marks the flags selected by --debug-all):@\n%a@]@."
           (Pp.print_list Pp.newline print)
-          (List.sort Pervasives.compare list);
+          (List.sort (fun (u,_,_) (v,_,_) -> String.compare u v) list);
       end;
       !opt_list_flags in
     desc,list
@@ -135,13 +135,16 @@ module Args = struct
         " set all debug flags that do not change Why3 behaviour" in
     ("--debug-all", Arg.Set opt_debug_all, desc_debug)
 
-  let set_flags_selected () =
+  let set_flags_selected ?(silent=false) () =
     if !opt_debug_all then
       List.iter
         (fun (s,f,_,_) -> if is_info_flag s then set_flag f)
         (list_flags ());
-    Queue.iter (fun flag -> let flag = lookup_flag flag in set_flag flag)
-      opt_list_flags;
+    let check flag =
+      match lookup_flag flag with
+      | f -> set_flag f
+      | exception UnknownFlag _ when silent -> () in
+    Queue.iter check opt_list_flags;
     if test_flag stack_trace then Printexc.record_backtrace true
 end
 
