@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -11,6 +11,14 @@
 
 (* useful list combinators *)
 
+let rev_filter f l =
+  let rec rev_filter acc l =
+    match l with
+    | [] -> acc
+    | hd :: tl when f hd -> rev_filter (hd :: acc) tl
+    | _ :: tl -> rev_filter acc tl in
+  rev_filter [] l
+
 let rev_map_fold_left f acc l =
   let acc, rev =
     List.fold_left
@@ -126,7 +134,7 @@ let fold_lefti f acc l =
 
 let rec prefix n l =
   if n = 0 then []
-  else if n < 0 || l = [] then invalid_arg "Util.chop"
+  else if n < 0 || l = [] then invalid_arg "Util.prefix"
   else List.hd l :: prefix (n - 1) (List.tl l)
 
 let rec chop n l =
@@ -138,3 +146,16 @@ let rec chop_last = function
   | [] -> invalid_arg "Util.chop_last"
   | [r] -> [], r
   | x :: s -> let s, r = chop_last s in x :: s, r
+
+let rec split n l =
+  if n = 0 then [], l
+  else if n < 0 || l = [] then invalid_arg "Util.split"
+  else
+    let prefix, suffix = split (n - 1) (List.tl l) in
+    (List.hd l :: prefix, suffix)
+
+let init n f =
+  let rec loop acc i =
+    if i >= n then List.rev acc
+    else loop (f i :: acc) (i + 1) in
+  loop [] 0
