@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -203,11 +203,7 @@ let string_of_wnl p x =
   fprintf fmt "%a@?" p x;
   Buffer.contents b
 
-let sprintf p =
-  (* useless: this is the same as Format.asprintf *)
-  let b = Buffer.create 100 in
-  let fmt = formatter_of_buffer b in
-  kfprintf (fun fmt -> Format.pp_print_flush fmt (); Buffer.contents b) fmt p
+let sprintf = asprintf
 
 let sprintf_wnl p =
   let b = Buffer.create 100 in
