@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,13 +9,12 @@
 (*                                                                  *)
 (********************************************************************)
 
-(* Replace with Char.uppercase_ascii as soon as we can assume
-  OCaml version at least 4.03.0  *)
-let char_is_uppercase c = c = Char.uppercase c
+let char_is_uppercase c = c = Char.uppercase_ascii c
 
-let lowercase = String.lowercase
-let capitalize = String.capitalize
-let uncapitalize = String.uncapitalize
+let lowercase = String.lowercase_ascii
+let uppercase = String.uppercase_ascii
+let capitalize = String.capitalize_ascii
+let uncapitalize = String.uncapitalize_ascii
 
 
 let rev_split c s =
