@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -105,6 +105,22 @@ let rec copy_dir from to_ =
 let concat f1 f2 =
   if Filename.is_relative f2 then Filename.concat f1 f2 else f2
 
+type file_path = string list
+let empty_path = []
+let add_to_path p fn = p @ [fn]
+let is_empty_path p = match p with [] -> true | _ -> false
+let decompose_path p = p
+
+let rec basename p =
+  match p with
+  | [] -> raise Not_found
+  | [f] -> f
+  | _ :: tl -> basename tl
+
+(* deprecated: let string_of_file_path p = String.concat "/" p *)
+let print_file_path fmt p = Format.fprintf fmt "%a" (Pp.print_list Pp.slash Pp.string) p
+
+
 let system_independent_path_of_file f =
   let rec aux acc f =
     let d = Filename.dirname f in
@@ -135,15 +151,19 @@ let p1 = path_of_file "/bin/bash"
 let p1 = path_of_file "../src/f.why"
   *)
 
+let is_regular_dir fn =
+  match Unix.lstat fn with
+  | s -> s.Unix.st_kind = Unix.S_DIR
+  | exception _ -> false
+
 let system_dependent_absolute_path dir p =
-  let rec file_of_path l =
+  let rec aux dir l =
     match l with
-    | [] -> ""
-    | [x] -> x
-    | x::l -> Filename.concat x (file_of_path l)
+    | [] -> dir
+    | ".." :: xs when is_regular_dir dir -> aux (Filename.dirname dir) xs
+    | x :: xs -> aux (Filename.concat dir x) xs
   in
-  let f = file_of_path p in
-  Filename.concat dir f
+  aux dir p
 
 let relativize_filename base f =
   let rec aux abs ab af =
