@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2020   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -23,14 +23,14 @@ let set_stdlib_url u =
 let output_dir = ref None
 let set_output_dir d = output_dir := d
 
-let dir_sep = Str.regexp_string Filename.dir_sep
+let dir_sep = Re.Str.regexp_string Filename.dir_sep
 
 let output_file fname =
   let fname = Filename.chop_extension fname in
   let base = match !output_dir with
     | None -> fname
     | Some dir ->
-      let f = Str.global_replace dir_sep "." fname in
+      let f = Re.Str.global_replace dir_sep "." fname in
       Filename.concat dir f
   in
   base ^ ".html"
